package com.yizhi.practice.application.aop;

import com.google.gson.Gson;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.util.application.domain.BizResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.formula.functions.T;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;

/**
 * 智能陪练aop
 */
@Component
@Aspect
@Slf4j
public class PracticeAop {


    /**
     * 定义切入点，切入点为com.yizhi.practice.application.controller包下所有controller中的所有函数
     *通过@Pointcut注解声明频繁使用的切点表达式
     */
    @Pointcut("execution(public * com.yizhi.practice.application.controller.manage.*.*(..)))")
    public void webLogAspect(){
    }

    /**
     * 方法执行前后打印日志
     */
    @Around("webLogAspect()")
    public Object doAround(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        RequestContext context = ContextHolder.get();
        Long accountId = context.getAccountId();
        Long siteId = context.getSiteId();
        String siteCode = context.getSiteCode();
        Long orgId = context.getOrgId();
        String accountName = context.getAccountName();
        Gson gson = new Gson();
        //打印请求信息
        log.info("===START=====Request AccountId = {} ,AccountName = {},SiteId = {} ,SiteCode = {},OrgId = {}; " +
                        "URL : {}, Request Class-Method: [{}-{}] " +
                " ,Request Args : {} ======\r\n"
                ,accountId,accountName,siteId,siteCode,orgId
                ,request.getRequestURL().toString(),proceedingJoinPoint.getTarget().getClass().getName(),proceedingJoinPoint.getSignature().getName()
                ,gson.toJson(proceedingJoinPoint.getArgs()));
        BizResponse<T> result = (BizResponse<T>) proceedingJoinPoint.proceed();
        // 打印出参
        log.info("Response Args  : {}", gson.toJson(result));
        log.info("===END=====Request AccountId = {} ,AccountName = {},SiteId = {} ,SiteCode = {},OrgId = {}; " +
                        "URL : {}, Request Class-Method: [{}-{}] " +
                        " ,Response : {} ======\r\n"
                ,accountId,accountName,siteId,siteCode,orgId
                ,request.getRequestURL().toString(),proceedingJoinPoint.getTarget().getClass().getName(),proceedingJoinPoint.getSignature().getName()
                ,gson.toJson(result.getData()));
        return result;
    }


}
