package com.yizhi.practice.application.config

import org.apache.commons.io.FileUtils
import org.apache.commons.lang.math.RandomUtils
import org.slf4j.LoggerFactory
import ws.schild.jave.AudioAttributes
import ws.schild.jave.Encoder
import ws.schild.jave.EncodingAttributes
import ws.schild.jave.MultimediaObject
import java.io.File
import java.net.URL

object AudioUtils {

    private val logger = LoggerFactory.getLogger(javaClass)

    val dir = "tmp/audio/"

    init {
        val dirFile = File(dir)
        if (!dirFile.exists()) {
            dirFile.mkdirs()
        }
    }

    private fun wavToMp3(sourceFile: File): File? {
        val targetFileName = System.currentTimeMillis().toString() + RandomUtils.nextInt(100000).toString()
        val target = File("$dir$targetFileName.mp3")
        val audio = AudioAttributes()
        audio.setCodec("libmp3lame")
        audio.setBitRate(16000)
        audio.setChannels(1)
        audio.setSamplingRate(16000)
        val attrs = EncodingAttributes()
        attrs.setFormat("mp3")
        attrs.setAudioAttributes(audio)
        val encoder = Encoder()
        val multimediaObject = MultimediaObject(sourceFile)
        encoder.encode(multimediaObject, target, attrs)
        return target
    }

    private fun wavToPcm(sourceFile: File): File? {
        val targetFileName = System.currentTimeMillis().toString() + RandomUtils.nextInt(100000).toString()
        val target = File("$dir$targetFileName.pcm")
        val audio = AudioAttributes()
        audio.setCodec("pcm_s16le")
        audio.setBitRate(16)
        audio.setChannels(1)
        audio.setSamplingRate(16000)
        val attrs = EncodingAttributes()
        attrs.setFormat("s16le")
        attrs.setAudioAttributes(audio)
        val encoder = Encoder()
        val multimediaObject = MultimediaObject(sourceFile)
        encoder.encode(multimediaObject, target, attrs)
        return target
    }

    fun wavToPcm(fileUrl: String):File? {
        val type = fileUrl.split(".").last()
        val httpurl = URL(fileUrl)
        // 生成随机文件名
        val fileName = System.currentTimeMillis().toString() + RandomUtils.nextInt(100000).toString() + "." + type
        val file = File(dir + fileName)
        var mp3File: File? = null
        try {
            FileUtils.copyURLToFile(httpurl, file)
            mp3File = wavToPcm(file)!!
            return mp3File
        } catch (e: Exception) {
            logger.error("拷贝文件或转MPCM出错。URL:$fileUrl")
            e.printStackTrace()
            file.deleteOnExit()
        } finally {
            file.deleteOnExit()
        }
        return null
    }

    fun wavToMp3(fileUrl: String): File? {
        val type = fileUrl.split(".").last()
        val httpurl = URL(fileUrl)
        // 生成随机文件名
        val fileName = System.currentTimeMillis().toString() + RandomUtils.nextInt(100000).toString() + "." + type
        println("fileName-->>>"+fileName)
        val file = File(dir + fileName)
        var mp3File: File? = null
        try {
            FileUtils.copyURLToFile(httpurl, file)
            mp3File = wavToMp3(file)!!
            return mp3File
        } catch (e: Exception) {
            logger.error("拷贝文件或转MP3出错。URL:$fileUrl")
            e.printStackTrace()
            //file.deleteOnExit()
        } finally {
            //file.deleteOnExit()
        }
        return null
    }

    /**
     * 获取音频文件时长
     */
    fun getAudioDuration(file: File): Long {
        val multimediaObject = MultimediaObject(file)
        return multimediaObject.info.duration
    }

    /**
     * 获取网络音频资源时长
     */
    fun getAudioDuration(fileUrl: String): Long {
        val type = fileUrl.split(".").last()
        val httpurl = URL(fileUrl)
        // 生成随机文件名
        val fileName = System.currentTimeMillis().toString() + RandomUtils.nextInt(100000).toString() + "." + type
        var file: File? = File(dir + fileName)
        var duration = 0L;
        try {
            FileUtils.copyURLToFile(httpurl, file)
            duration = getAudioDuration(file!!)
            file.deleteOnExit()
            return duration
        } catch (e: Exception) {
            e.printStackTrace()
            logger.error("获取音频时长异常。URL:$fileUrl")
            file!!.deleteOnExit()
        } finally {
            // 删除生成的文件
            file!!.deleteOnExit()
        }
        return duration
    }

}