package com.yizhi.practice.application.controller.manage;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.practice.application.pojo.vo.PracticeChatLogBaseInfoVo;
import com.yizhi.practice.application.pojo.vo.PracticeChatLogDetailParamVo;
import com.yizhi.practice.application.pojo.vo.PracticeChatRoundLogDetailVo;
import com.yizhi.practice.application.service.IPracticeChatLogService;
import com.yizhi.practice.application.xservice.PracticeChatRoundLogXService;
import com.yizhi.util.application.domain.BizResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Api(tags = "智能演练-陪练记录-陪练详情")
@RequestMapping(path = "/manage/practice/log/detail")
@RestController
public class PracticeChatRoundLogController {

	@Autowired
	private PracticeChatRoundLogXService roundXservice;
	
	@Autowired
	private IPracticeChatLogService chatservice;
	
	@ApiOperation(value = "本题分析")
	@GetMapping("/analysis/get")
	public BizResponse<PracticeChatRoundLogDetailVo> analysis(@ApiParam(value="某句对话的ID") @RequestParam("roundId")Long roundId){
		return BizResponse.ok(roundXservice.analysis(roundId));
	}
	
	@ApiOperation(value = "陪练详情（包含学员信息,总得分信息,场景对话及得分）")
	@GetMapping("/get")
	public BizResponse<PracticeChatLogBaseInfoVo> baseInfo(@ApiParam(value="陪练记录的ID") @RequestParam("practiceLogId") Long practiceLogId){
		return BizResponse.ok(roundXservice.getLogByPracticeChatId(practiceLogId));
	}
	
	@ApiOperation(value = "上一份")
	@PostMapping("/previous/get")
	public BizResponse<PracticeChatLogBaseInfoVo> previous(@RequestBody PracticeChatLogDetailParamVo param){
		RequestContext requestContext = ContextHolder.get();
		Long previousId = chatservice.previous(param, requestContext);
		if(null == previousId) {
			log.warn("Previous practice log not exist!");
			return BizResponse.ok();
		}
		return BizResponse.ok(roundXservice.getLogByPracticeChatId(previousId));
	}
	
	@ApiOperation(value = "下一份")
	@PostMapping("/next/get")
	public BizResponse<PracticeChatLogBaseInfoVo> next(@RequestBody PracticeChatLogDetailParamVo param){
		RequestContext requestContext = ContextHolder.get();
		Long nextId = chatservice.next(param, requestContext);
		if(null == nextId) {
			log.warn("Next practice log not exist!");
			return BizResponse.ok();
		}
		return BizResponse.ok(roundXservice.getLogByPracticeChatId(nextId));
	}
}
