package com.yizhi.practice.application.controller.manage;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.practice.application.pojo.vo.*;
import com.yizhi.practice.application.request.*;
import com.yizhi.practice.application.xservice.PracticeXservice;
import com.yizhi.util.application.domain.BizResponse;
import com.yizhi.util.application.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;


@Slf4j
@Api(tags = "智能演练")
@RequestMapping(path = "/manage/practice")
@RestController
public class PracticeController {

    @Autowired
    PracticeXservice practiceXservice;

    /**
     * 新增陪练 初始化空白陪练
     * @return 陪练唯一id
     */
    @ApiOperation(value = "初始化空白陪练")
    @PostMapping(path = "/init/blank/draft/save/")
    public BizResponse<PracticeBlankDraftVo> initBlankDraftPractice(){
        PracticeBlankDraftVo baseInfoVo = practiceXservice.initBlankDraftPractice();
        return BizResponse.ok(baseInfoVo);
    }

    /**
     * 保存陪练基本信息
     */
    @ApiOperation(value = "保存陪练基本信息")
    @PostMapping(path = "/base/info/save")
    public BizResponse savePracticeConfigInfo(@RequestBody PracticeConfigInfoReq practiceConfigInfoReq){

        return practiceXservice.savePracticeConfInfo(practiceConfigInfoReq);
    }
    /**
     * 保存陪练策略配置
     */
    @ApiOperation(value = "保存陪练策略信息")
    @PostMapping(path = "/strategy/info/save")
    public BizResponse savePracticeStrategyInfo(@RequestBody PracticeConfigInfoReq practiceConfigInfoReq){

        return practiceXservice.savePracticeStrategyInfo(practiceConfigInfoReq);
    }

    /**
     * 保存陪练 对话
     */
    @ApiOperation(value = "保存陪练对话信息")
    @PostMapping(path = "/dialogue/save")
    public BizResponse savePracticeDialogue(@RequestBody List<PracticeSceneDialogueReq> sceneNodeList){

        if (null == sceneNodeList || sceneNodeList.size() == 0){
            return BizResponse.fail("陪练对话参数为空");
        }
        return practiceXservice.savePracticeDialogue(sceneNodeList);
    }

    /**
     * 保存智能陪练指定用户
     */
    @ApiOperation(value = "保存智能陪练指定用户")
    @PostMapping(path = "/account/save")
    public BizResponse addPracticeAccount(@RequestBody PracticeAccountReq practiceAccountReq){

        return practiceXservice.addPracticeAccount(practiceAccountReq);
    }

    /**
     * 智能陪练上架、下架
     */
    @ApiOperation(value = "智能陪练上架、下架")
    @PostMapping(path = "/status/update")
    public BizResponse setPracticeStatus(@RequestBody PracticeStatusReq practiceStatusReq) throws Exception {

        Long practiceId = practiceStatusReq.getPracticeId();
        Integer status = practiceStatusReq.getStatus();
        if (null == practiceId || null == status){
            log.error("上架下架陪练失败,参数包含空值 practiceId = {},status ={}",practiceId,status);
            return BizResponse.fail("操作失败");
        }
        BizResponse bizResponse = practiceXservice.updatePracticeStatus(practiceId, status);
        return bizResponse;
    }
    
    /**
     * 复制智能陪练
     * @return 返回复制成功的新的陪练的ID
     * @throws Exception 
     */
    @ApiOperation(value = "复制智能陪练")
    @PostMapping(path = "/copy/save")
    public BizResponse<Long> copyPractice(@RequestBody PracticeReq practiceReq) throws Exception{
        Long practiceId = practiceReq.getPracticeId();
        return BizResponse.ok(practiceXservice.copyPractice(practiceId));
    }

    /**
     * 导入陪练对话
     * @param sceneChatImportReq 导入文件信息
     */
    @ApiOperation(value = "导入陪练对话")
    @PostMapping(value = "/import/chat/update")
    public BizResponse<String> importScore(@RequestBody SceneChatImportReq sceneChatImportReq) throws Exception {

        String fileName = sceneChatImportReq.getFileName();
        String fileUrl = sceneChatImportReq.getFileUrl();
        Long practiceId = sceneChatImportReq.getPracticeId();
        RequestContext context = ContextHolder.get();
        Long sceneId = sceneChatImportReq.getSceneId();
        if (StringUtils.isEmpty(fileUrl) || null == practiceId || null == sceneId || (!fileUrl.endsWith(".xlsx") && !fileUrl.endsWith(".xls"))){
            log.error("智能陪练导入对话失败；文件名不符合要求或必要参数为空。accountId = {} ,入参: {}",context.getAccountId(),JSONObject.toJSONString(sceneChatImportReq));
            return BizResponse.fail("导入陪练对话失败");
        }
        return practiceXservice.importPracticeChat(sceneChatImportReq);
    }

    /**
     * 清空陪练下场景对话
     */
    @ApiOperation(value = "清空场景对话")
    @PostMapping(path = "/clear/scene/chat/update")
    public BizResponse clearScene(@RequestBody SceneChatReq chatReq){

        Long sceneId = chatReq.getSceneId();
        RequestContext context = ContextHolder.get();
        if (null == sceneId){
            log.error("清空场景对话失败! 场景id为空;accountId = {}",context.getAccountId());
            return BizResponse.fail("清空场景对话失败");
        }
        return practiceXservice.clearSceneChat(sceneId);
    }

    /**
     * 删除陪练
     */
    @ApiOperation(value = "删除陪练")
    @PostMapping(path = "/delete")
    public BizResponse deletePractice(@RequestBody PracticeReq practiceReq ){

        RequestContext context = ContextHolder.get();
        Long practiceId = practiceReq.getPracticeId();
        if (null == practiceId){
            log.error("[PracticeController:deletePractice]删除陪练失败! practiceId为空;accountId = {}",context.getAccountId());
            return BizResponse.fail("删除陪练失败");
        }
        return practiceXservice.delete(practiceId);
    }

    /**
     * 导入可见范围
     * 导入用户
     */
    @ApiOperation(value = "导入可见范围")
    @PostMapping(path = "/import/account/update")
    public BizResponse importPracticeUsers(@RequestBody PracticeImportAccountReq importAccountReq){

        return practiceXservice.importAccount(importAccountReq);
    }

    /**
     * 导出可见范围
     * 导出用户
     */
    @ApiOperation(value = "导出可见范围")
    @GetMapping(path = "/export/account/update")
    public BizResponse exportPracticeUsers(@RequestParam("practiceId") Long practiceId){

        RequestContext context = ContextHolder.get();
        if (null == practiceId){
            log.error("导出可见范围失败! 传入practiceId 为空；accountId = {}",context.getAccountId());
            return BizResponse.fail("导出可见范围失败");
        }
        return practiceXservice.exportAccount(practiceId);
    }

    /**
     * 设置对话‘下一步’校验场景和对话
     */
    @ApiOperation(value = "校验陪练场景和对话是否完整")
    @GetMapping(path = "/scene/check")
    public BizResponse checkSceneAndChat(@RequestParam("practiceId") Long practiceId){
        return practiceXservice.checkPracticeSceneValid(practiceId);
    }


    /**
     * 查询机器人
     */
    @ApiOperation(value = "查询机器人信息")
    @GetMapping(path = "/robot/list")
    public BizResponse<List<RobotVo>> getRobotList(){

        List<RobotVo> list = practiceXservice.getRobotList();
        return BizResponse.ok(list);
    }


    /**
     * 分页查询陪练列表
     */
    @ApiOperation(value = "分页查询陪练列表" )
    @PostMapping(path = "/practices/list")
    public BizResponse<PageInfo<PracticeBaseInfoVo>> selectPractices(@RequestBody PracticesGetReq practicesGetReq){

        BizResponse<PageInfo<PracticeBaseInfoVo>> response = practiceXservice.getPracticeList(practicesGetReq);
        return response;
    }

    /**
     * 查询陪练基本信息和策略信息
     */
    @ApiOperation(value = "查询陪练基本信息和策略信息")
    @GetMapping(path = "/baseInfo/list")
    public BizResponse<PracticeConfigInfoVo> selectPracticeBaseInfo(@RequestParam(value = "practiceId") Long practiceId){

        PracticeConfigInfoVo configInfoVo = practiceXservice.getPracticeConfigInfo(practiceId);
        if (null != configInfoVo){
            return BizResponse.ok(configInfoVo);
        }
        return BizResponse.fail("查询陪练基本信息失败");
    }

    /**
     * 查询陪练指定用户信息
     */
    @ApiOperation(value = "查询陪练指定用户信息")
    @GetMapping(path = "/account/list")
    public BizResponse<PracticeVisibleAccountVo> selectPracticeAccountInfo(@RequestParam(value = "practiceId") Long practiceId){
        RequestContext context = ContextHolder.get();
        if (null == practiceId){
            log.error("查询陪练指定用户信息失败；practiceId 为空; accountId = {}",context.getAccountId());
            return BizResponse.fail("查询陪练指定用户信息失败");
        }
        PracticeVisibleAccountVo visibleAccountVo = practiceXservice.getPracticeVisibleList(practiceId);
        return BizResponse.ok(visibleAccountVo);
    }

    /**
     * 查询陪练场景对话
     */
    @ApiOperation(value = "查询场景对话")
    @GetMapping(path = "/scene/chat/list")
    public BizResponse<List<PracticeSceneDialogueVo>> selectPracticeSceneChatList(@RequestParam(value = "practiceId") Long practiceId,@RequestParam(value = "sceneId") Long sceneId){
        if (null == practiceId || null == sceneId){
            log.error("查询场景失败；参数都为空");
            return BizResponse.fail("查询场景失败");
        }
        List<PracticeSceneDialogueVo> vo = new ArrayList<>();
        try {
            vo = practiceXservice.getPracticeSceneChatList(practiceId,sceneId);
        } catch (JsonProcessingException e) {
            log.error("查询场景对话 .",e);
            return BizResponse.fail("查询场景对话失败");
        }
        return BizResponse.ok(vo);
    }


}
