package com.yizhi.practice.application.controller.manage;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.practice.application.util.NumberUtil;
import com.yizhi.statistics.application.feign.StatisticsPracticeByUserClient;
import com.yizhi.statistics.application.request.practice.PracticeReportByUserDetailReq;
import com.yizhi.statistics.application.request.practice.PracticeReportByUserReq;
import com.yizhi.statistics.application.vo.practice.PracticeReportByUserDetailVo;
import com.yizhi.statistics.application.vo.practice.PracticeReportByUserListVo;
import com.yizhi.util.application.domain.BizResponse;
import com.yizhi.util.application.page.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

@Api(tags = "智能演练-报表-按用户统计")
@RequestMapping(path = "/manage/practice/report/user")
@RestController
@Slf4j
public class PracticeReportByUserController {

	@Autowired
	private StatisticsPracticeByUserClient statisticsClient;
	
	@ApiOperation(value = "列表")
	@PostMapping("/list/get")
	public BizResponse<PageInfo<PracticeReportByUserListVo>> list(@RequestBody PracticeReportByUserReq param){
		param.setContext(ContextHolder.get());
		Page<PracticeReportByUserListVo> result = statisticsClient.list(param);
		
		PageInfo<PracticeReportByUserListVo> page = new PageInfo<PracticeReportByUserListVo>();
		page.setPageNo(param.getPageNo());
		page.setPageSize(param.getPageSize());
		page.setPageTotal(NumberUtil.getPageCount((long) result.getTotal(), param.getPageSize()));
		page.setPageRecords(result.getTotal());
		page.setRecords(result.getRecords());
		log.info("Practice report by user - list page - size : {}.", result.getTotal());
		return BizResponse.ok(page);
	}
	
	@ApiOperation(value = "列表-下载")
	@PostMapping("/list/download")
	public BizResponse<String> listDownload(@RequestBody PracticeReportByUserReq param){
		param.setContext(ContextHolder.get());
		return BizResponse.ok(statisticsClient.listDownload(param));
	}
	
	@ApiOperation(value = "详情")
	@PostMapping("/detail/get")
	public BizResponse<PageInfo<PracticeReportByUserDetailVo>> detail(@RequestBody PracticeReportByUserDetailReq param){
		param.setContext(ContextHolder.get());
		Page<PracticeReportByUserDetailVo> result = statisticsClient.detail(param);
		
		PageInfo<PracticeReportByUserDetailVo> page = new PageInfo<PracticeReportByUserDetailVo>();
		page.setPageNo(param.getPageNo());
		page.setPageSize(param.getPageSize());
		page.setPageTotal(NumberUtil.getPageCount((long) result.getTotal(), param.getPageSize()));
		page.setPageRecords(result.getTotal());
		page.setRecords(result.getRecords());
		log.info("Practice report by user - detail page - size : {}.", result.getTotal());
		return BizResponse.ok(page);
	}
	
	@ApiOperation(value = "详情-下载")
	@PostMapping("/detail/download")
	public BizResponse<String> detailDownload(@RequestBody PracticeReportByUserDetailReq param){
		param.setContext(ContextHolder.get());
		return BizResponse.ok(statisticsClient.detailDownload(param));
	}
}
