package com.yizhi.practice.application.controller.manage;

import com.alibaba.fastjson.JSONObject;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.enums.InternationalEnums;
import com.yizhi.core.application.exception.BizException;
import com.yizhi.practice.application.pojo.vo.SceneVo;
import com.yizhi.practice.application.request.SceneChatReq;
import com.yizhi.practice.application.request.SceneReq;
import com.yizhi.practice.application.xservice.PracticeXSceneservice;
import com.yizhi.util.application.domain.BizResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@Slf4j
@Api(tags = "智能演练-场景")
@RequestMapping(path = "/manage/scene")
@RestController(value = "ManagePracticeSceneController")
public class PracticeSceneController {

    private static Logger logger = LoggerFactory.getLogger(PracticeSceneController.class);

    @Autowired
    PracticeXSceneservice xSceneservice;

    /**
     * 新增场景
     */
    @ApiOperation(value = "新增陪练场景")
    @PostMapping(path = "/save")
    public BizResponse<SceneVo> addScene(@RequestBody SceneReq sceneReq){
        if (null == sceneReq.getPracticeId()){
            logger.error("新增陪练场景，陪练id为空! 请求入参: {}", JSONObject.toJSONString(sceneReq));
            throw new BizException(InternationalEnums.MATERIALCONTROLLER1);
        }
        return xSceneservice.save(sceneReq);
    }

    /**
     * 修改场景
     */
    @ApiOperation(value = "修改场景")
    @PostMapping(path = "/update")
    public BizResponse<SceneVo> updateScene(@RequestBody SceneReq sceneReq){
        if (null == sceneReq.getPracticeId()){
            logger.error("修改陪练场景，陪练id为空! 请求入参: {}", JSONObject.toJSONString(sceneReq));
            throw new BizException(InternationalEnums.MATERIALCONTROLLER1);
        }
        return xSceneservice.save(sceneReq);
    }

    /**
     * 修改场景排序
     */
    @ApiOperation(value = "修改场景排序")
    @PostMapping(path = "/sort/update")
    public BizResponse updateSceneSort(@RequestBody List<SceneReq> sceneReqList){
        if (null == sceneReqList || sceneReqList.size() == 0){
            logger.error("修改场景排序失败,入参为空!");
            throw new BizException(InternationalEnums.MATERIALCONTROLLER1);
        }
        return xSceneservice.updateSceneSort(sceneReqList);
    }


    /**
     * 删除场景
     */
    @ApiOperation(value = "删除场景")
    @PostMapping(path = "/delete")
    public BizResponse deleteScene(@RequestBody SceneChatReq sceneReq){

        //删除场景
        RequestContext context = ContextHolder.get();
        Long sceneId = sceneReq.getSceneId();
        if (null == sceneId){
            log.error("[PracticeSceneController:deleteScene]删除场景失败。sceneId is null. accountId = {}",context.getAccountId());
            throw new BizException(InternationalEnums.CLASSIFYCONTROLLER4);
        }
        return xSceneservice.deleteScene(sceneId);
    }


    /**
     * 查询场景
     */
    @ApiOperation(value = "查询场景详情")
    @GetMapping(path = "/one/get")
    public BizResponse<SceneVo> selectScene(@RequestParam(name = "sceneId") Long sceneId){
        SceneVo sceneVo = xSceneservice.selectScene(sceneId);
        return BizResponse.ok(sceneVo);
    }

    /**
     * 查询场景列表
     */
    @ApiOperation(value = "查询场景列表")
    @GetMapping(path = "/list")
    public BizResponse<List<SceneVo>> selectSceneList(@RequestParam(name = "practiceId") Long practiceId){
        RequestContext context = ContextHolder.get();
        if (null == practiceId){
            log.error("查询场景列表失败.传入practiceId 为空;accountId = {}",context.getAccountId());
            return BizResponse.fail("查询场景列表失败");
        }
        List<SceneVo> list = xSceneservice.selectSceneList(practiceId);
        return BizResponse.ok(list);
    }


}
