package com.yizhi.practice.application.controller.manage;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.plugins.Page;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.enums.InternationalEnums;
import com.yizhi.core.application.exception.BizException;
import com.yizhi.practice.application.request.PracticeStatisticsReq;
import com.yizhi.statistics.application.feign.StatisticsPracticeClient;
import com.yizhi.statistics.application.request.practice.PracticeStatByPracticeIdReq;
import com.yizhi.statistics.application.request.practice.PracticeStatByPracticeReq;
import com.yizhi.statistics.application.vo.practice.PracticeStatByPracticeVo;
import com.yizhi.statistics.application.vo.practice.PracticeStatViewByIdVo;
import com.yizhi.util.application.domain.BizResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

@Slf4j
@Api(tags = "智能陪练统计")
@RequestMapping("/manage/practice/report")
@RestController
public class PracticeStatisticsController {

    private static Logger logger = LoggerFactory.getLogger(PracticeStatisticsController.class);
    private SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
    @Autowired
    StatisticsPracticeClient practiceClient;


    @PostMapping("/group/list")
    @ApiOperation(value = "智能陪练统计-按陪练统计")
    public BizResponse<Page<PracticeStatByPracticeVo>> practiceGroup(@RequestBody PracticeStatisticsReq req) throws JsonProcessingException, ParseException {

        PracticeStatByPracticeReq practiceReq = new PracticeStatByPracticeReq();
        checkAndSetTime(req);
        practiceReq.setStartDate(req.getStartOfDate());
        practiceReq.setEndDate(req.getEndOfDate());
        practiceReq.setName(req.getPracticeName());
        practiceReq.setPageNo(req.getPageNo());
        practiceReq.setPageSize(req.getPageSize());
        practiceReq.setContext(ContextHolder.get());
        return BizResponse.ok(practiceClient.practiceGroup(practiceReq));
    }

    @PostMapping("/group/list/download")
    @ApiOperation(value = "智能陪练统计-按陪练统计-按陪练统计下载")
    public BizResponse practiceGroupDownload(@RequestBody PracticeStatisticsReq req) throws ParseException {

        checkAndSetTime(req);
        PracticeStatByPracticeReq practiceReq = new PracticeStatByPracticeReq();
        practiceReq.setPageSize(req.getPageSize());
        practiceReq.setPageNo(req.getPageNo());
        practiceReq.setName(req.getPracticeName());
        practiceReq.setStartDate(req.getStartOfDate());
        practiceReq.setEndDate(req.getEndOfDate());
        practiceReq.setContext(ContextHolder.get());
        return BizResponse.ok(practiceClient.practiceGroupDownload(practiceReq));
    }


    @PostMapping("/group/view/list")
    @ApiOperation(value = "智能陪练统计-按陪练统计-学员陪练结果列表")
    public BizResponse<Page<PracticeStatViewByIdVo>> practiceGroupView(@RequestBody PracticeStatisticsReq req) throws ParseException {

        checkAndSetTime(req);
        Long practiceId = req.getPracticeId();
        if (null == practiceId){
            logger.error("智能陪练学员训练详情列表陪练id为空!");
            return BizResponse.fail(InternationalEnums.DROOLSMANAGERCONTROLLER1.getCode(),InternationalEnums.DROOLSMANAGERCONTROLLER1.getName());
        }
        PracticeStatByPracticeIdReq practiceViewReq = new PracticeStatByPracticeIdReq();
        practiceViewReq.setPageSize(req.getPageSize());
        practiceViewReq.setPageNo(req.getPageNo());
        practiceViewReq.setPracticeId(req.getPracticeId());
        practiceViewReq.setPracticeName(req.getPracticeName());
        practiceViewReq.setStartDate(req.getStartOfDate());
        practiceViewReq.setEndDate(req.getEndOfDate());
        practiceViewReq.setNameKwd(req.getNameKwd());
        practiceViewReq.setOrgKwd(req.getOrgKwd());
        practiceViewReq.setContext(ContextHolder.get());
        return BizResponse.ok(practiceClient.practiceGroupView(practiceViewReq));
    }

    @PostMapping("/group/view/list/download")
    @ApiOperation(value = "智能陪练统计-按陪练统计-详情-下载结果数据")
    public BizResponse PagepracticeGroupViewDownload(@RequestBody PracticeStatisticsReq req) throws ParseException {
        checkAndSetTime(req);
        Long practiceId = req.getPracticeId();
        if (null == practiceId){
            logger.error("智能陪练统计-按陪练统计-详情-下载结果数据,陪练id为空!");
            return BizResponse.fail(InternationalEnums.DROOLSMANAGERCONTROLLER1.getCode(),InternationalEnums.DROOLSMANAGERCONTROLLER1.getName());
        }
        PracticeStatByPracticeIdReq practiceViewReq = new PracticeStatByPracticeIdReq();
        practiceViewReq.setPageSize(req.getPageSize());
        practiceViewReq.setPageNo(req.getPageNo());
        practiceViewReq.setPracticeId(req.getPracticeId());
        practiceViewReq.setPracticeName(req.getPracticeName());
        practiceViewReq.setStartDate(req.getStartOfDate());
        practiceViewReq.setEndDate(req.getEndOfDate());
        practiceViewReq.setNameKwd(req.getNameKwd());
        practiceViewReq.setOrgKwd(req.getOrgKwd());
        practiceViewReq.setContext(ContextHolder.get());
        return BizResponse.ok(practiceClient.practiceGroupViewDownload(practiceViewReq));

    }

    @PostMapping("/group/view/list/detail/download")
    @ApiOperation(value = "智能陪练统计-按陪练统计-详情-下载陪练明细")
    public BizResponse practiceGroupViewDetailDownload(@RequestBody PracticeStatisticsReq req) throws ParseException {

        checkAndSetTime(req);
        Long practiceId = req.getPracticeId();
        if (null == practiceId){
            logger.error("智能陪练统计-按陪练统计-详情-下载陪练明细,陪练id为空!");
            return BizResponse.fail(InternationalEnums.DROOLSMANAGERCONTROLLER1.getCode(),InternationalEnums.DROOLSMANAGERCONTROLLER1.getName());
        }
        PracticeStatByPracticeIdReq practiceViewReq = new PracticeStatByPracticeIdReq();
        practiceViewReq.setPageSize(req.getPageSize());
        practiceViewReq.setPageNo(req.getPageNo());
        practiceViewReq.setPracticeId(req.getPracticeId());
        practiceViewReq.setPracticeName(req.getPracticeName());
        practiceViewReq.setStartDate(req.getStartOfDate());
        practiceViewReq.setEndDate(req.getEndOfDate());
        practiceViewReq.setNameKwd(req.getNameKwd());
        practiceViewReq.setOrgKwd(req.getOrgKwd());
        practiceViewReq.setContext(ContextHolder.get());
        return BizResponse.ok(practiceClient.practiceGroupViewDetailDownload(practiceViewReq));
    }


    //时间转换
    private PracticeStatisticsReq checkAndSetTime(PracticeStatisticsReq req) throws ParseException {
        String startDate = req.getStartDate();
        String endDate = req.getEndDate();
        if (StringUtils.isEmpty(startDate) || StringUtils.isEmpty(endDate)){
            logger.error("智能陪练统计-按陪练统计时间段包含空值");
            throw new BizException(InternationalEnums.OFFLINECOURSEREPORTCONTROLLER);
        }
        Date startOfDate = DateUtil.beginOfDay(format.parse(startDate));
        Date endOfDate = DateUtil.endOfDay(format.parse(endDate));
        req.setStartOfDate(startOfDate);
        req.setEndOfDate(endOfDate);
        return req;
    }
}
