package com.yizhi.practice.application.controller.remote;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.yizhi.practice.application.pojo.vo.PracticeChatLogVo;
import com.yizhi.practice.application.pojo.vo.PracticeVisibleVo;
import com.yizhi.practice.application.pojo.vo.PracticeVo;
import com.yizhi.practice.application.xservice.PracticeChatLogXService;
import com.yizhi.practice.application.xservice.PracticeXservice;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;


@Api(tags = "智能演练remote api")
@RestController
@Slf4j
public class PracticeControllerApi {

    @Autowired
    PracticeXservice practiceXservice;
    @Autowired
    private PracticeChatLogXService xservice;

    private static final String DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";
    
    
    private Date getStartDate(String startDate) throws ParseException {
    	log.info("Incoming param start date string:{}", startDate);
    	SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT);
    	Date start = null;
    	if(StringUtils.isEmpty(startDate)) {
        	Date previousDay = com.yizhi.util.application.date.DateUtil.add(new Date(), Calendar.DAY_OF_MONTH, -1);
        	start = cn.hutool.core.date.DateUtil.beginOfDay(previousDay);
        }else {
        	start = sdf.parse(startDate);
        }
    	return start;
    }
    
    private Date getEndDate(String endDate) throws ParseException {
    	log.info("Incoming param end date string:{}", endDate);
    	SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT);
    	Date end = null;
        
        if(StringUtils.isEmpty(endDate)) {
        	Date previousDay = com.yizhi.util.application.date.DateUtil.add(new Date(), Calendar.DAY_OF_MONTH, -1);
        	end = cn.hutool.core.date.DateUtil.endOfDay(previousDay);
        }else {
        	end = sdf.parse(endDate);
        }
        return end;
    }
    
    /**
     * 查询指定时间范围内更新的陪练
     */
    @ApiOperation(value = "查询指定时间范围内更新的陪练")
    @GetMapping(path = "/remote/manage/practice/range/list")
    public List<PracticeVo> selectRangeList(@RequestParam(name = "startDate",value="startDate",required=false)String startDate,
                                       @RequestParam(name = "endDate",value="endDate",required=false)String endDate) {
        List<PracticeVo> practiceVos = null;
		try {
			practiceVos = practiceXservice.selectRangeList(getStartDate(startDate), getEndDate(endDate));
		} catch (ParseException e) {
			log.error("Parse error", e);
		}
        return practiceVos;
    }


    /**
     * 查询指定时间范围内更新的陪练可见范围
     */
    @ApiOperation(value = "查询指定时间范围内更新的陪练可见范围")
    @GetMapping(path = "/remote/manage/practice/visible/range/list")
    public List<PracticeVisibleVo> selectVisibleRangeList(@RequestParam(name = "startDate",value="startDate",required=false) String startDate,
                                       @RequestParam(name = "endDate",value="endDate",required=false) String endDate) {
        List<PracticeVisibleVo> visibleVos= null;
		try {
			visibleVos = practiceXservice.selectVisibleRangeList(getStartDate(startDate), getEndDate(endDate));
		} catch (ParseException e) {
			log.error("Parse error", e);
		}
        return visibleVos;
    }


    /**
     * 查询指定时间范围内更新的陪练记录
     */
    @ApiOperation(value = "查询指定时间范围内更新的陪练记录")
    @GetMapping(path = "/remote/manage/practice/chat/range/list")
    public List<PracticeChatLogVo> selectChatLogRangeList(@RequestParam(name = "startDate",value="startDate",required=false) String startDate,
                                              @RequestParam(name = "endDate",value="endDate",required=false) String endDate) {
        List<PracticeChatLogVo> visibleVos= null;
		try {
			visibleVos = xservice.selectChatLogRangeList(getStartDate(startDate), getEndDate(endDate));
		} catch (ParseException e) {
			log.error("Parse error", e);
		}
        return visibleVos;
    }

}
