package com.yizhi.practice.application.controller.student

import com.yizhi.core.application.context.ContextHolder
import com.yizhi.practice.application.pojo.po.AnalyzeStudentVoicePo
import com.yizhi.practice.application.pojo.po.StartPracticePO
import com.yizhi.practice.application.pojo.vo.*
import com.yizhi.practice.application.xservice.PracticeStuChatXService
import com.yizhi.util.application.domain.BizResponse
import io.swagger.annotations.Api
import io.swagger.annotations.ApiOperation
import io.swagger.annotations.ApiParam
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.web.bind.annotation.*


/**
 * @Date 2020/10/26 3:54 下午
 * @Author lvjianhui
 **/
@Api(tags = ["开始陪练"])
@RestController
@RequestMapping(path = ["/student/chat"])
class PracticeStuSceneController {

    @Autowired
    lateinit var practiceStuChatXService: PracticeStuChatXService

    @PostMapping("/start/practice/save")
    @ApiOperation("陪练开始,新增一条陪练记录")
    fun startPractice(@RequestBody startPracticePO: StartPracticePO): BizResponse<StuPracticeLogVo> {
        startPracticePO.requestContext = ContextHolder.get()
        val vo = practiceStuChatXService.startPractice(startPracticePO)
        return BizResponse.ok(vo)
    }

    @GetMapping("/scene/list")
    @ApiOperation("获取所有场景")
    fun getAllScene(
            @ApiParam("陪练ID")
            @RequestParam("practiceId") practiceId: Long): BizResponse<List<SceneChatVO>> {
        val list = practiceStuChatXService.getAllScene(practiceId)
        return BizResponse.ok(list)
    }

    @GetMapping("/scene/first/list")
    @ApiOperation("按顺序开始场景，返回机器人话术")
    fun startChatByScene(
            @ApiParam("场景ID")
            @RequestParam("sceneId") sceneId: Long,
            @ApiParam("陪练模式 模式 10：训练 20：考核", allowableValues = "10,20")
            @RequestParam("useType") useType: Int,
            @ApiParam("陪练ID")
            @RequestParam("practiceId") practiceId: Long
    ): BizResponse<List<ChatContextVo>> {
        val list = practiceStuChatXService.getRobotFirstChat(practiceId, sceneId, useType)
        return BizResponse.ok(list)
    }

    @GetMapping("/context/student/list")
    @ApiOperation("根据机器人话术，查询学员话术")
    fun getStudentChatByRobot(
            @ApiParam("场景ID")
            @RequestParam("sceneId") sceneId: Long,
            @ApiParam("陪练模式 模式 10：训练 20：考核", allowableValues = "10,20")
            @RequestParam("useType") useType: Int,
            @ApiParam("陪练ID")
            @RequestParam("practiceId") practiceId: Long,
            @ApiParam("机器人话术Id")
            @RequestParam robotChatId: Long,
            @ApiParam("陪练记录Id")
            @RequestParam practiceLogId: Long
    ): BizResponse<List<StudentChatContextVo>> {
        val list = practiceStuChatXService.getStudentContent(sceneId, useType, practiceId,practiceLogId, robotChatId)
        return BizResponse.ok(list)
    }

    @PostMapping("/analyze/voice/save")
    @ApiOperation("用户上传录音，返回结果")
    fun analyzeStudentVoice(@RequestBody po: AnalyzeStudentVoicePo): BizResponse<ChatResultVo> {
        po.requestContext = ContextHolder.get()
        val vo = practiceStuChatXService.analyzeStudentVoice(po)
        return BizResponse.ok(vo)
    }

    @GetMapping("/chat/log/delete")
    fun deletedAccountUploadId(@RequestParam("accountUploadId") accountUploadId:Long):BizResponse<String> {
        practiceStuChatXService.deletedAccountUploadId(accountUploadId,ContextHolder.get())
        return BizResponse.ok()
    }

    @GetMapping("/robot/context/get")
    @ApiOperation("根据学员话术返回机器人话术")
    fun getRobotContext(@ApiParam("场景ID")
                        @RequestParam("sceneId") sceneId: Long,
                        @ApiParam("陪练模式 模式 10：训练 20：考核", allowableValues = "10,20")
                        @RequestParam("useType") useType: Int,
                        @ApiParam("陪练ID")
                        @RequestParam("practiceId") practiceId: Long,
                        @ApiParam("学员话术Id")
                        @RequestParam chatRoundConfigId: Long): BizResponse<List<ChatContextVo>> {
        val vo = practiceStuChatXService.getRobotLevelChat(practiceId, sceneId, chatRoundConfigId, useType)
        return BizResponse.ok(vo)
    }

    @GetMapping("/end/save")
    fun endAccountPractice(@ApiParam("陪练模式 模式 10：训练 20：考核", allowableValues = "10,20")
                           @RequestParam("useType") useType: Int,
                           @ApiParam("陪练ID")
                           @RequestParam("practiceId") practiceId: Long,
                           @ApiParam("陪练记录ID")
                           @RequestParam("practiceLogId") practiceLogId: Long,
                           @ApiParam("异常退出 传true")
                           @RequestParam("errorEnd", defaultValue = "false") errorEnd: Boolean
    ): BizResponse<PracticeEndResultVo> {
        val vo = practiceStuChatXService.endPractice(practiceId, practiceLogId, useType, ContextHolder.get(), errorEnd)
        return BizResponse.ok(vo)
    }

    @GetMapping("/score/get")
    fun getResultByLogResult(@ApiParam("陪练记录ID")
                       @RequestParam("practiceLogId") practiceLogId: Long):BizResponse<PracticeEndResultVo> {
        val vo = practiceStuChatXService.getLogResult(practiceLogId,ContextHolder.get().accountId)
        return BizResponse.ok(vo)
    }

}