package com.yizhi.practice.application.controller.student

import com.yizhi.core.application.context.ContextHolder
import com.yizhi.core.application.context.RequestContext
import com.yizhi.practice.application.feign.MyPracticeStudentClient
import com.yizhi.practice.application.pojo.vo.PracticeConfigVo
import com.yizhi.practice.application.pojo.vo.PracticeJoinedVo
import com.yizhi.practice.application.xservice.PracticeStudentXSerice
import com.yizhi.util.application.domain.BizResponse
import com.yizhi.util.application.page.PageInfo
import io.swagger.annotations.Api
import io.swagger.annotations.ApiOperation
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.web.bind.annotation.*

@RestController
@Api(tags = ["学员陪练"])
@RequestMapping(path = ["/student/practice"])
class PracticeStudentController : MyPracticeStudentClient {

    @Autowired
    lateinit var practiceStudentXSerice: PracticeStudentXSerice

//    @ApiOperation("获取用户进行中的配练数")
    @GetMapping("/running/count/get")
    override fun getRuningPracticeCount(@RequestBody requestContext: RequestContext): BizResponse<Int> {
        val count = practiceStudentXSerice.getRunningCount(
                requestContext.accountId,
                requestContext.orgId,
                requestContext.siteId,
                requestContext.companyId
        )
        return BizResponse.ok(count)
    }

    @ApiOperation(value = "校验用户是否有陪练可见权限")
    @GetMapping("/user/auth")
    override fun getUserPracticeAuth(@RequestBody requestContext: RequestContext
                                     ,@RequestParam("practiceId") practiceId: Long): BizResponse<Boolean> {
        return practiceStudentXSerice.getUserPracticeAuth(requestContext,practiceId);
    }

    @GetMapping("/list/tab/index/get")
    @ApiOperation("获取前端走哪个tab 0:未开始 1:进行中 2:已完成")
    fun getAccountTab():BizResponse<Int> {
        val requestContext = ContextHolder.get()
        val index = practiceStudentXSerice.getTabIndex(requestContext)
        return BizResponse.ok(index)

    }

    @GetMapping("/by/state/list")
    @ApiOperation("根据状态获取陪练列表 40:进行中 10：未开始 80：已结束")
    fun getRunningPracticeList(@RequestParam(defaultValue = "1") pageIndex: Int,
                               @RequestParam(defaultValue = "10") pageSize: Int,
                               @RequestParam(defaultValue = "40") state: Int): BizResponse<PageInfo<PracticeConfigVo>> {
        val requestContext = ContextHolder.get()
        val page = practiceStudentXSerice.getRunningList(
                requestContext.accountId,
                requestContext.orgId,
                requestContext.siteId,
                requestContext.companyId,
                state,
                pageIndex,
                pageSize
        )
        return BizResponse.ok(page)
    }

    @GetMapping("/msg/by/id/list")
    @ApiOperation("获取配练时间及次数信息")
    fun getStartMsgById(
            @RequestParam(name = "practiceId") practiceId: Long): BizResponse<PracticeConfigVo> {
        val requestContext = ContextHolder.get()
        val vo: PracticeConfigVo = practiceStudentXSerice.getPracticeInfo(
                practiceId,
                requestContext.accountId,
                requestContext.orgId,
                requestContext.siteId,
                requestContext.companyId
        )
        return BizResponse.ok(vo)
    }

    @GetMapping("/joined/count")
    fun getPracticeJoinEd(@RequestParam practiceId: Long):BizResponse<PracticeJoinedVo> {
        val vo = practiceStudentXSerice.getPracticeJoined(practiceId, ContextHolder.get())
        return BizResponse.ok(vo)
    }

}