package com.yizhi.practice.application.excel;

import com.alibaba.fastjson.JSONObject;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.practice.application.enums.PracticeVisibleTypeEnum;
import com.yizhi.practice.application.pojo.domain.PracticeConfig;
import com.yizhi.practice.application.pojo.domain.PracticeVisible;
import com.yizhi.practice.application.repository.PracticeConfigRepository;
import com.yizhi.practice.application.repository.PracticeVisibleRepository;
import com.yizhi.system.application.model.AccountRangeExportParam;
import com.yizhi.system.application.system.remote.AccountRangeClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Component
public class PracticeAccountExport extends AbstractDefaultTask<String, PracticeAccountExportParam> {


    private static final Logger logger = LoggerFactory.getLogger(PracticeAccountExport.class);

    @Autowired
    private AccountRangeClient accountRangeClient;
    @Autowired
    private PracticeVisibleRepository visibleRepository;
    @Autowired
    private PracticeConfigRepository practiceConfigRepository;



    @Override
    public String execute(PracticeAccountExportParam accountExportParam) {

        RequestContext context = accountExportParam.getContext();
        Long practiceId = accountExportParam.getPracticeId();
        try {
            AccountRangeExportParam accountRangeImportParam=new AccountRangeExportParam();
            accountRangeImportParam.setContext(context);
            PracticeConfig practice = practiceConfigRepository.findById(practiceId).get();
            Integer practiceVisibleType = practice.getVisibleType();
            List<Long> accountIds = new ArrayList<>();
            List<Long> orgIds = new ArrayList<>();
            if (PracticeVisibleTypeEnum.ALL.getStatus().equals(practiceVisibleType)){
                return "";
            }else if (PracticeVisibleTypeEnum.PART.getStatus().equals(practiceVisibleType)){
                //指定用户可见
                List<PracticeVisible> practiceVisibleList = visibleRepository.findByPracticeId(practiceId);
                taskDetail(accountExportParam.getTaskId(),"智能陪练导出用户可见范围:查询可见范围成功;practiceId="+practiceId);
                if (null != practiceVisibleList && practiceVisibleList.size() > 0){
                    for (PracticeVisible practiceVisible : practiceVisibleList) {
                        Integer visibleType = practiceVisible.getVisibleType();
                        if (PracticeVisibleTypeEnum.PART.getStatus().equals(visibleType)){
                            accountIds.add(practiceVisible.getVisibleAccountId());
                        }
                        else if (PracticeVisibleTypeEnum.ORG.getStatus().equals(visibleType)){
                            orgIds.add(practiceVisible.getVisibleOrgId());
                        }
                    }
                    accountRangeImportParam.setAccountIds(accountIds);
                    accountRangeImportParam.setOrgIds(orgIds);
                    taskDetail(accountExportParam.getTaskId(),"智能陪练导出用户可见范围:获取到可见范围accountIdsSize="+accountIds.size()+",orgIdsSize="+orgIds.size());
                }
            }
            accountRangeImportParam.setBizName(practice.getName());
            logger.info("智能陪练-[PracticeAccountExport:execute]导出可见范围参数: {}", JSONObject.toJSONString(accountRangeImportParam));
            taskDetail(accountExportParam.getTaskId(),"智能陪练导出用户可见范围:调用system导出可见范围入参="+JSONObject.toJSONString(accountRangeImportParam));
            String serialNo=accountRangeClient.doExport(accountRangeImportParam);
            logger.info("智能陪练-[PracticeAccountExport:execute]导出可见范围返回结果: {}",serialNo);
            return serialNo;
        }catch (Exception e){
            logger.error("智能陪练可见范围导出异常;",e);
        }
        return null;
    }
}
