package com.yizhi.practice.application.excel;

import com.alibaba.fastjson.JSONObject;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.event.LogEvent;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.log.LogEventPublisher;
import com.yizhi.core.application.log.TaskLogDetailEvent;
import com.yizhi.core.application.log.TaskLogEvent;
import com.yizhi.practice.application.enums.PracticeVisibleTypeEnum;
import com.yizhi.practice.application.request.PracticeAccountReq;
import com.yizhi.practice.application.request.PracticeVisibleAccount;
import com.yizhi.practice.application.xservice.PracticeXservice;
import com.yizhi.system.application.model.AccountRangeImportModel;
import com.yizhi.system.application.model.AccountRangeImportParam;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.system.remote.AccountRangeClient;
import com.yizhi.system.application.vo.AccountVO;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Component
public class PracticeAccountImport extends AbstractDefaultTask<String, PracticeAccountImportParam> {


    private static final Logger logger = LoggerFactory.getLogger(PracticeAccountImport.class);

    @Autowired
    private AccountRangeClient accountRangeClient;
    @Autowired
    IdGenerator idGenerator;
    @Autowired
    AccountClient accountClient;
    @Autowired
    PracticeXservice practiceXservice;
    @Autowired
    LogEventPublisher publisher;



    @Override
    protected String execute(PracticeAccountImportParam accountImportParam) {

        String ossUrl = accountImportParam.getFileUrl();
        RequestContext context = accountImportParam.getContext();
        ContextHolder.set(context);
        Long accountId = context.getAccountId();
        TaskContext taskContext = new TaskContext(accountImportParam.getTaskId(), accountImportParam.getSerialNo()
                , accountImportParam.getTaskName(), accountId, new Date(), context.getSiteId(), context.getCompanyId());
        working(taskContext);
        AccountRangeImportParam param = new AccountRangeImportParam();
        param.setType(AccountRangeImportParam.Type.PRACTICE);
        param.setOssUrl(ossUrl);
        param.setRequestContext(context);
        List<AccountRangeImportModel.SuccessAccount> successList = new ArrayList<>();
        List<AccountRangeImportModel.ErrorAccount> errorList = new ArrayList<>();
        //进行入库
        try {
            logger.info("智能演练导入用户可见范围:[AccountRangeClient.doImport]入参: {}",JSONObject.toJSONString(param));
            AccountRangeImportModel accountRangeImportModel = accountRangeClient.doImport(param);
            if (null == accountRangeImportModel) {
                logger.error("智能演练导入用户可见范围:[AccountRangeClient.doImport]出参is null");
                logEventPublish(taskContext,successList,errorList,null);
                return null;
            }
            if (null != accountRangeImportModel) {
                logger.info("智能演练导入用户可见范围,[AccountRangeClient.doImport]出参: {}", JSONObject.toJSONString(accountRangeImportModel));
                successList = accountRangeImportModel.getSuccessList();
                errorList = accountRangeImportModel.getErrorList();
            }
            PracticeAccountReq practiceAccountReq = new PracticeAccountReq();
            practiceAccountReq.setPracticeId(accountImportParam.getPracticeId());
            List<PracticeVisibleAccount> accountList = new ArrayList<>();
            for (AccountRangeImportModel.SuccessAccount successAccount : successList) {
                PracticeVisibleAccount visibleAccount = new PracticeVisibleAccount();
                Long id = successAccount.getId();
                AccountVO accountVO = accountClient.findById(id);
                visibleAccount.setName(accountVO.getName());
                visibleAccount.setFullName(accountVO.getFullName());
                visibleAccount.setWorkNum(accountVO.getWorkNum());
                visibleAccount.setVisibleId(id);
                visibleAccount.setVisibleType(PracticeVisibleTypeEnum.PART.getStatus());
                accountList.add(visibleAccount);
            }
            practiceAccountReq.setAccountList(accountList);
            practiceXservice.addPracticeAccount(practiceAccountReq);
            logEventPublish(taskContext,successList,errorList,null);
        } catch (Exception e) {
            fail(taskContext, "智能陪练可见范围异步导入过程中出现错误");
            logger.error("智能陪练可见范围异步导入过程中出现错误.", e);
        }
        return "ok";
    }




    public void logEventPublish(TaskContext taskContext, List<AccountRangeImportModel.SuccessAccount> successList,
                                List<AccountRangeImportModel.ErrorAccount> errorList, String url) {

        TaskLogEvent taskLogEvent = TaskLogEvent.success(taskContext, "成功导入" + successList.size() + "条," + "导入失败" + errorList.size() + "条", url);
        LogEvent<TaskLogEvent> event = new LogEvent("taskLog", taskLogEvent);
        this.publisher.publish(event);

        if (CollectionUtils.isNotEmpty(errorList)) {
            List<TaskLogDetailEvent> details = new ArrayList(errorList.size());
            Long taskId = taskContext.getTaskId();
            for (AccountRangeImportModel.ErrorAccount a : errorList) {
                details.add(new TaskLogDetailEvent(taskId, "第"+a.getLine()+"行,"+a.getErrMsg()));
            }
            LogEvent<List<TaskLogDetailEvent>> eventDetails = new LogEvent("batchTaskDetailLog", details);
            this.publisher.publish(eventDetails);
        }
    }
}
