package com.yizhi.practice.application.service.impl;

import java.util.List;
import java.util.stream.Collectors;

import org.springframework.stereotype.Service;

import com.querydsl.core.Tuple;
import com.querydsl.core.types.Predicate;
import com.yizhi.practice.application.enums.PracticeSkipEnum;
import com.yizhi.practice.application.pojo.domain.QPracticeChatRoundLog;
import com.yizhi.practice.application.pojo.domain.QSceneChatConfig;
import com.yizhi.practice.application.pojo.domain.QSceneConfig;
import com.yizhi.practice.application.pojo.dto.PracticeAllScoreDto;
import com.yizhi.practice.application.pojo.dto.PracticeChartRoundLogDto;
import com.yizhi.practice.application.pojo.vo.PracticeChatRoundLogVo;
import com.yizhi.practice.application.service.BaseService;
import com.yizhi.practice.application.service.IPracticeChatRoundLogService;
import com.yizhi.practice.application.util.LocalStringUtil;

import kotlin.Pair;

/**
 * <p>
 * 场景中每轮对话记录 服务实现类
 * </p>
 *
 * @author MybatisCodeGenerator123
 * @since 2020-10-22
 */
@Service
public class PracticeChatRoundLogServiceImpl extends BaseService implements IPracticeChatRoundLogService {

    @Override
    public PracticeAllScoreDto getAccountChatLog(Long practiceChatId, Long accountId, Long orgId, Long siteId, Long companyId) {
        QPracticeChatRoundLog roundLog = QPracticeChatRoundLog.practiceChatRoundLog;

        Predicate pre = roundLog.practiceChatId.eq(practiceChatId)
                .and(roundLog.accountId.eq(accountId))
                .and(roundLog.companyId.eq(companyId))
                .and(roundLog.siteId.eq(siteId))
                .and(roundLog.orgId.eq(orgId))
                .and(roundLog.deleted.eq(false))
                .and(roundLog.roundSkipType.eq(PracticeSkipEnum.NOT_SKIP.getCode()));
        Tuple result =  jpaqueryFactory
                .select(roundLog.keywordScore.avg(),
                        roundLog.completeScore.avg(),
                        roundLog.smoothScore.avg(),
                        roundLog.politeScore.avg(),
                        roundLog.expressScore.avg())
                .from(roundLog)
                .where(pre)
                .fetchOne();
        PracticeAllScoreDto dto = new PracticeAllScoreDto();

        Double expressScore = result.get(roundLog.expressScore.avg());
        Double completeScore = result.get(roundLog.completeScore.avg());
        Double smoothScore = result.get(roundLog.smoothScore.avg());
        Double politeScore = result.get(roundLog.politeScore.avg());
        Double keywordScore = result.get(roundLog.keywordScore.avg());
        if (null != expressScore) {
            dto.setExpressScore(expressScore.intValue());
        }
        if (null != completeScore ) {
            dto.setComplateScore(completeScore.intValue());
        }
        if (null != smoothScore ) {
            dto.setFluencyScore(smoothScore.intValue());
        }
        if (null != politeScore ) {
            dto.setPoliteScore(politeScore.intValue());
        }
        if (null != keywordScore ) {
            dto.setKeyWordScore(new Pair(keywordScore.intValue(),null));
        }
        return dto;
    }

    @Override
    public Long getLevelCountByAccount(Integer level, Long practiceLogId, Long practiceId, Long accountId, Long orgId, Long siteId, Long companyId) {
        QPracticeChatRoundLog roundLog = QPracticeChatRoundLog.practiceChatRoundLog;
        Predicate pre = roundLog.roundLevel.eq(level)
                .and(roundLog.practiceId.eq(practiceId))
                .and(roundLog.practiceChatId.eq(practiceLogId))
                .and(roundLog.accountId.eq(accountId))
                .and(roundLog.companyId.eq(companyId))
                .and(roundLog.siteId.eq(siteId))
                .and(roundLog.deleted.eq(false))
                .and(roundLog.orgId.eq(orgId));
        Long count = jpaqueryFactory.select(roundLog.count())
                .from(roundLog)
                .where(pre)
                .fetchCount();
        return count;
    }

    @Override
    public Long getCountByUserId(Long practiceLogId, Long chatId, Long accountId, Long orgId, Long siteId, Long companyId) {
        QPracticeChatRoundLog roundLog = QPracticeChatRoundLog.practiceChatRoundLog;
        Predicate pre = roundLog.sceneChatId.eq(chatId)
                .and(roundLog.practiceChatId.eq(practiceLogId))
                .and(roundLog.accountId.eq(accountId))
                .and(roundLog.companyId.eq(companyId))
                .and(roundLog.siteId.eq(siteId))
                .and(roundLog.deleted.eq(false))
                .and(roundLog.orgId.eq(orgId));
        return jpaqueryFactory.select(roundLog.count())
                .from(roundLog)
                .where(pre)
                .fetchCount();
    }

    @Override
    public List<PracticeChatRoundLogVo> getByPracticeChatId(Long practiceChatId) {
        QPracticeChatRoundLog roundLog = QPracticeChatRoundLog.practiceChatRoundLog;
        QSceneConfig scene = QSceneConfig.sceneConfig;
        QSceneChatConfig sceneChat = QSceneChatConfig.sceneChatConfig;
        return jpaqueryFactory.select(
                roundLog.id
                , roundLog.practiceChatId
                , roundLog.sceneId
                , scene.name
                , scene.sort
                , roundLog.robotContent
                , roundLog.robotContentVoiceUrl
                , roundLog.robotContentVoiceDuration
                , roundLog.accountContent
                , roundLog.accountHitKeyword
                , roundLog.accountContentVoiceUrl
                , roundLog.accountContentVoiceDuration
                , roundLog.roundLevel
                , roundLog.roundSkipType
                , roundLog.averageScore
                , sceneChat.sceneNodeType
                , roundLog.createdAt
                , roundLog.accountHitTabooWord)
                .from(roundLog)
                .leftJoin(scene)
                .on(roundLog.sceneId.eq(scene.id))
                .leftJoin(sceneChat)
                .on(roundLog.sceneChatId.eq(sceneChat.id))
                .where(roundLog.practiceChatId.eq(practiceChatId).and(roundLog.deleted.eq(false)))
                .orderBy(scene.sort.asc(), roundLog.roundLevel.asc())
                .fetch()
                .stream()
                .map(tuple -> PracticeChatRoundLogVo.builder()
                        .id(tuple.get(roundLog.id))
                        .practiceChatId(tuple.get(roundLog.practiceChatId))
                        .sceneId(tuple.get(roundLog.sceneId))
                        .sceneName(tuple.get(scene.name))
                        .sort(tuple.get(scene.sort))
                        .robotContent(tuple.get(roundLog.robotContent))
                        .robotContentUrl(tuple.get(roundLog.robotContentVoiceUrl))
                        .robotDuration(tuple.get(roundLog.robotContentVoiceDuration))
                        .accountContent(tuple.get(roundLog.accountContent))
                        .accountHitKeyWord(LocalStringUtil.hit(tuple.get(roundLog.accountHitKeyword)))
                        .accountContentVoiceUrl(tuple.get(roundLog.accountContentVoiceUrl))
                        .accountDuration(tuple.get(roundLog.accountContentVoiceDuration))
                        .roundLevel(tuple.get(roundLog.roundLevel))
                        .roundSkipType(tuple.get(roundLog.roundSkipType))
                        .isSkip(PracticeSkipEnum.NOT_SKIP.getCode().equals(tuple.get(roundLog.roundSkipType)) ? false : true)
                        .averageScore(tuple.get(roundLog.averageScore))
                        .sceneNodeType(tuple.get(sceneChat.sceneNodeType))
                        .createdAt(tuple.get(roundLog.createdAt))
                        .hitForbidWord(LocalStringUtil.hit(tuple.get(roundLog.accountHitTabooWord)))
                        .build())
                .collect(Collectors.toList());
    }
    
    @Override
    public PracticeChartRoundLogDto getRoundLogById(Long roundLogId) {
        QPracticeChatRoundLog roundLog = QPracticeChatRoundLog.practiceChatRoundLog;
        QSceneChatConfig sceneChat = QSceneChatConfig.sceneChatConfig;

        Tuple result = jpaqueryFactory.select(
                roundLog.averageScore
                , roundLog.accountHitKeyword
                , roundLog.keywordScore
                , roundLog.completeScore
                , roundLog.smoothScore
                , roundLog.expressScore
                , roundLog.politeScore
                , roundLog.accountContent
                , roundLog.accountContentVoiceUrl
                , roundLog.accountContentVoiceDuration
                , sceneChat.accountContentKeyword
                , roundLog.robotContent
                , roundLog.robotContentVoiceUrl
                , roundLog.robotContentVoiceDuration
                , roundLog.accountHitTabooWord)
                .from(roundLog)
                .leftJoin(sceneChat)
                .on(roundLog.sceneChatId.eq(sceneChat.id))
                .where(roundLog.id.eq(roundLogId).and(roundLog.deleted.eq(false)))
                .fetchOne();

        return PracticeChartRoundLogDto.builder()
                .totalScore(result.get(roundLog.averageScore))
                .keywordScore(result.get(roundLog.keywordScore))
                .completeScore(result.get(roundLog.completeScore))
                .smoothScore(result.get(roundLog.smoothScore))
                .expressScore(result.get(roundLog.expressScore))
                .politeScore(result.get(roundLog.politeScore))
                .accountContent(result.get(roundLog.accountContent))
                .accountContentVoiceUrl(result.get(roundLog.accountContentVoiceUrl))
                .accountDuration(result.get(roundLog.accountContentVoiceDuration))
                .keyWord(result.get(sceneChat.accountContentKeyword))
                .accountHitKeyword(result.get(roundLog.accountHitKeyword))
                .robotContent(result.get(roundLog.robotContent))
                .robotContentUrl(result.get(roundLog.robotContentVoiceUrl))
                .robotDuration(result.get(roundLog.robotContentVoiceDuration))
                .forbidHit(result.get(roundLog.accountHitTabooWord))
                .build();
    }

}
