package com.yizhi.practice.application.service.impl

import com.querydsl.core.types.dsl.BooleanExpression
import com.yizhi.practice.application.config.PracticeRunningStatusEnum
import com.yizhi.practice.application.enums.PracticeStatusEnum
import com.yizhi.practice.application.enums.PracticeVisibleTypeEnum
import com.yizhi.practice.application.pojo.domain.PracticeConfig
import com.yizhi.practice.application.pojo.domain.QPracticeConfig
import com.yizhi.practice.application.pojo.vo.PracticeBaseInfoVo
import com.yizhi.practice.application.pojo.vo.PracticeConfigVo
import com.yizhi.practice.application.request.PracticesGetReq
import com.yizhi.practice.application.service.BaseService
import com.yizhi.practice.application.service.IPracticeConfigService
import org.springframework.stereotype.Service
import java.util.*
import kotlin.collections.ArrayList

/**
 *
 *
 * 陪练配置 服务实现类
 *
 *
 * @author MybatisCodeGenerator123
 * @since 2020-10-22
 */
@Service
class PracticeConfigServiceImpl : IPracticeConfigService, BaseService() {

    override fun getSiteAllConfig(siteId: Long, companyId: Long): List<Long> {
        val config = QPracticeConfig.practiceConfig
        val predicate = config.companyId.eq(companyId)
                .and(config.siteId.eq(siteId))
                .and(config.visibleType.eq(PracticeVisibleTypeEnum.ALL.status))
        return jpaqueryFactory.select(config.id)
                .from(config)
                .where(predicate)
                .fetch()

    }

    override fun getConfigByIds(ids: List<Long>): List<PracticeConfigVo> {
        val config = QPracticeConfig.practiceConfig
        val predicate = config.id.`in`(ids)
        val result = jpaqueryFactory.select(
                config.name,
                config.practiceStartAt,
                config.practiceEndAt,
                config.examStartAt,
                config.examEndAt,
                config.id,
                config.skipType,
                config.repeatPush)
                .from(config)
                .where(predicate)
                .orderBy(config.createdAt.desc())
                .fetch()
        val vos = ArrayList<PracticeConfigVo>()
        result.forEach {
            val vo = PracticeConfigVo()
            vo.name = it.get(config.name)
            vo.practiceStartAt = it.get(config.practiceStartAt)
            vo.practiceEndAt = it.get(config.practiceEndAt)
            vo.examStartAt = it.get(config.examStartAt)
            vo.examEndAt = it.get(config.examEndAt)
            vo.practiceId = it.get(config.id)
            vo.practiceSkipType = it.get(config.skipType)
            vo.examRepeatPush = it.get(config.repeatPush)
            vos.add(vo)
        }
        return vos
    }

    override fun getRunningCount(orgId: Long, siteId: Long, companyId: Long, date: Date?, state: PracticeRunningStatusEnum): List<Long>? {
        val time = date ?: Date()
        val config = QPracticeConfig.practiceConfig

        var pre = config.id.isNotNull

        // 进行中的陪练
        if (state.status == PracticeRunningStatusEnum.RUNNING.status) {
            pre = pre.and((config.practiceStartAt.isNotNull.and(config.practiceStartAt.lt(time)).and(
                    config.practiceEndAt.isNotNull.and(config.practiceEndAt.goe(time))))
                    .or(
                            config.examStartAt.isNotNull.and(config.examStartAt.lt(time)).and(
                                    config.examEndAt.isNotNull.and(config.examEndAt.goe(time)))
                    ))
        }
        // 已结束的陪练
        if (state.status == PracticeRunningStatusEnum.ENDED.status) {
            pre = pre.and((config.practiceEndAt.isNotNull.and(config.practiceEndAt.lt(time)).and(
                    config.examEndAt.isNotNull.and(config.examEndAt.lt(time))
            )).or(
                    config.practiceEndAt.isNull.and(config.examEndAt.isNotNull.and(config.examEndAt.lt(time)))
            ).or(
                    config.examEndAt.isNull.and(config.practiceEndAt.isNotNull.and(config.practiceEndAt.lt(time)))
            ))
        }

        // 未开始
        if (state.status == PracticeRunningStatusEnum.NOT_STARTED.status) {
            pre = pre.and(((config.practiceStartAt.isNotNull.and(config.practiceStartAt.goe(time)).and(
                    config.examStartAt.isNotNull.and(config.examStartAt.goe(time))
            ))).or(
                    config.practiceStartAt.isNull.and(config.examStartAt.isNotNull.and(config.examStartAt.goe(time)))
            ).or(
                    config.examStartAt.isNull.and(config.practiceStartAt.isNotNull.and(config.practiceStartAt.goe(time)))
            ))
        }

        val predicate = config.companyId.eq(companyId)
                .and(config.siteId.eq(siteId))
                .and(pre)
                .and(config.status.eq(PracticeStatusEnum.UP.status))
                .and(config.deleted.isFalse)

        return jpaqueryFactory
                .selectDistinct(config.id)
                .from(config)
                .where(predicate)
                .orderBy(config.createdAt.desc())
                .fetch()
    }

    override fun getListByTime(startDate: Date, endDate: Date): List<PracticeConfig> {
        val config = QPracticeConfig.practiceConfig
        val expression = config.updatedAt.between(startDate,endDate).and(config.status.ne(PracticeStatusEnum.DRAFT.status))
        return jpaqueryFactory.select(
                config)
                .from(config)
                .where(expression)
                .fetch()
    }

    fun getPracticeConfigExpresion(params: PracticesGetReq): BooleanExpression {
        val nameOrKeyword = params.name
        val practiceStatus = params.status
        val models = params.models
        val companyId = params.companyId
        val siteId = params.siteId

        val config = QPracticeConfig.practiceConfig

        var expression = config.deleted.eq(false).and(config.companyId.eq(companyId)).and(config.siteId.eq(siteId))
        if (null != params.ids && params.ids.isNotEmpty()){
            expression = expression.and(config.createdUid.`in`(params.ids))
        }
        if (null != nameOrKeyword && nameOrKeyword.isNotBlank()){
            expression = expression.and(config.name.like("%${params.name}%").or(config.keyWord.like("%${params.name}%")))
        }
        if (null != practiceStatus && practiceStatus > 0) {
            expression = expression.and(config.status.eq(practiceStatus))
        }
        if (null != models && models.isNotEmpty()) {
            expression = expression.and(config.useType.`in`(models))
        }
        return expression
    }

    override fun getPracticeConfigList(params: PracticesGetReq): List<PracticeBaseInfoVo> {

        val config = QPracticeConfig.practiceConfig
        val pageNo = params.pageNo
        val pageSize = params.pageSize
        val offset = (pageNo-1) * pageSize
        val expression = getPracticeConfigExpresion(params)
        val fetch = jpaqueryFactory.select(
                config.id,
                config.name,
                config.useType,
                config.practiceStartAt,
                config.practiceEndAt,
                config.examStartAt,
                config.examEndAt,
                config.status,
        		config.visibleType,
                config.logoUrl)
                .from(config)
                .where(expression)
                .orderBy(config.createdAt.desc())
                .offset(offset.toLong())
                .limit(params.pageSize.toLong())
                .fetch()
        var vos = ArrayList<PracticeBaseInfoVo>()
        fetch.forEach {
            val vo = PracticeBaseInfoVo()
            vo.id = it.get(config.id)
            vo.name = it.get(config.name)
            vo.useType = it.get(config.useType)
            vo.practiceStartAt = it.get(config.practiceStartAt)
            vo.practiceEndAt = it.get(config.practiceEndAt)
            vo.examStartAt = it.get(config.examStartAt)
            vo.examEndAt = it.get(config.examEndAt)
            vo.status = it.get(config.status)
			vo.visibleType = it.get(config.visibleType)
            vo.logoUrl = it.get(config.logoUrl)
            vos.add(vo)
        }
        return vos
    }

    override fun getPracticeConfigCount(params: PracticesGetReq): Long {
        val config = QPracticeConfig.practiceConfig
        val expression = getPracticeConfigExpresion(params)
        return jpaqueryFactory.select(
                config.id,
                config.name,
                config.useType,
                config.practiceStartAt,
                config.practiceEndAt,
                config.examStartAt,
                config.examEndAt,
                config.status)
                .from(config)
                .where(expression)
                .orderBy(config.createdAt.desc())
                .fetchCount()
    }

    override fun delete(practiceId: Long) {
        val config = QPracticeConfig.practiceConfig
        val expression = config.id.eq(practiceId)
        jpaqueryFactory.update(config).set(config.deleted,true).where(expression).execute();
    }
}