package com.yizhi.practice.application.third.baidu;

import java.math.BigDecimal;
import java.util.HashMap;

import org.apache.commons.lang3.StringUtils;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.baidu.aip.nlp.AipNlp;

/**
 * 
 * @author xiaoyu
 *
 */
public class BaiduUtil {
	
	private static final Logger LOG = LoggerFactory.getLogger(BaiduUtil.class);
	
	private static final String RESULT_KEY = "score";

	private AipNlp client;

	public BaiduUtil() {
		super();
	}

	public BaiduUtil(AipNlp client) {
		super();
		this.client = client;
	}

	public AipNlp getClient() {
		return client;
	}

	public void setClient(AipNlp client) {
		this.client = client;
	}
	
	/**
	 * 获取两个文本的相似度
	 * @param text1
	 * @param text2
	 * @return
	 */
	public BigDecimal getSimilarRate(String text1, String text2) {
		if(StringUtils.isEmpty(text1) || StringUtils.isEmpty(text2)) {
			LOG.warn("Baidu similarRate : Input text is empty!");
			return BigDecimal.ZERO;
		}
		
		HashMap<String, Object> options = new HashMap<String, Object>();
        options.put("model", SimnetMode.CNN.getValue());
        
        JSONObject result = client.simnet(text1, text2, options);
        LOG.info("Baidu result : {}", result.toString());
        
        BigDecimal rate = result.getBigDecimal(RESULT_KEY);
        return rate;
	}
	
	/**
	 * 获取两个文本的相似度
	 * @param text1 不可为空
	 * @param text2 不可为空
	 * @param mode 可为空
	 * @return 相似度结果取值(0,1]，分数越高说明相似度越高
	 */
	public BigDecimal getSimilarRate(String text1, String text2, String mode) {
		if(StringUtils.isEmpty(text1) || StringUtils.isEmpty(text2)) {
			LOG.warn("Baidu similarRate : Input text is empty!");
			return BigDecimal.ZERO;
		}
		
		HashMap<String, Object> options = new HashMap<String, Object>();
        options.put("model", null == mode ? SimnetMode.CNN.getValue() : mode);
        
        JSONObject result = client.simnet(text1, text2, options);
        LOG.info("Baidu result : {}", result.toString());
        
        BigDecimal rate = result.getBigDecimal(RESULT_KEY);
        return rate;
	}
}
