package com.yizhi.practice.application.third.xunfei;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.Base64;
import java.util.Date;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.yizhi.practice.application.enums.PracticeVoiceTypeEnum;
import com.yizhi.practice.application.enums.PracticeXunFeiBusinessTypeEnum;
import com.yizhi.practice.application.third.util.FrameUtil;

import lombok.Builder;
import lombok.experimental.Tolerate;
import okhttp3.WebSocket;

/**
 * 
 * @author xiaoyu
 *
 */
@Builder
public class EvaluateRunnable implements Runnable {

	private static final Logger LOG = LoggerFactory.getLogger(EvaluateRunnable.class);
	private static final String sub="ise";//服务类型sub,开放评测值为ise
	private static final String ent="cn_vip";//语言标记参数 ent(cn_vip中文,en_vip英文)
	//题型、文本、音频要请注意做同步变更(如果是英文评测,请注意变更ent参数的值)
	private static final String category="read_sentence";//题型
	private static long beginTime=(new Date()).getTime();
	private static long endTime=(new Date()).getTime();
	
	private String appId;
	private String inputText;
	private InputStream inputVoiceStream;
	private WebSocket webSocket;
	private PracticeVoiceTypeEnum voiceType;
	private Integer recognizeFactor;
	private Integer evaluateFactor;
	
    @Tolerate
	public EvaluateRunnable() {
		super();
	}

	@Override
	public void run() {
		//连接成功，开始发送数据
		int frameSize = FrameUtil.determineFrameSize(PracticeXunFeiBusinessTypeEnum.EVALUATE, this.recognizeFactor, this.evaluateFactor); //每一帧音频的大小,建议每 40ms 发送 1280B，大小可调整，但是不要超过19200B，即base64压缩后能超过26000B，否则会报错10163数据过长错误
		int intervel = 40;
		int status = 0;  // 音频的状态
		ssb(webSocket);
		//ttp(webSocket);
		beginTime=(new Date()).getTime();
		try {
			byte[] buffer = new byte[frameSize];
			// 发送音频
			end:
				while (true) {
					int len = this.inputVoiceStream.read(buffer);
					if (len == -1) {
						status = Constants.StatusLastFrame;  //文件读完，改变status 为 2
					}

					switch (status) {
					case Constants.StatusFirstFrame:   // 第一帧音频status = 0
						send(webSocket,1,1,Base64.getEncoder().encodeToString(Arrays.copyOf(buffer, len)));
						status=Constants.StatusContinueFrame;//中间帧数
						break;

					case Constants.StatusContinueFrame:  //中间帧status = 1
						send(webSocket,2,1,Base64.getEncoder().encodeToString(Arrays.copyOf(buffer, len)));
						break;

					case Constants.StatusLastFrame:    // 最后一帧音频status = 2 ，标志音频发送结束
						send(webSocket,4,2,"");
						LOG.info("sendlast");
						endTime=(new Date()).getTime();
						LOG.info("总耗时："+(endTime-beginTime)+"ms");
						break end;
					}
					Thread.sleep(intervel); //模拟音频采样延时
				}
			LOG.info("all data is send");
		} catch (FileNotFoundException e) {
			LOG.error("Error occurred:", e);
		} catch (IOException e) {
			LOG.error("Error occurred:", e);
		} catch (Exception e) {
			LOG.error("Error occurred:", e);
		} finally {
			try {
				this.inputVoiceStream.close();
			} catch (IOException e) {
				LOG.error("Error occurred:", e);
			}
		}
	}
	
	//上传参数添加与发送
	private void ssb(WebSocket webSocket) {
		ParamBuilder p = new ParamBuilder();
		p.add("common", new ParamBuilder()
		.add("app_id", appId)
				)
				.add("business", new ParamBuilder()
				.add("category", category)
				.add("rstcd", "utf8")

				//群体：adult成人 、youth（中学，效果与设置pupil参数一致）、pupil小学
				//仅中文字、词、句题型支持
				.add("group", "adult")

				//打分门限值：hard、common、easy
				//仅英文引擎支持
				//.add("check_type","common")

				//学段：junior(1,2年级) middle(3,4年级) senior(5,6年级)
				//仅中文题型：中小学的句子、篇章题型支持
				//.add("grade","junior")

				//extra_ability生效条件：ise_unite=1,rst=entirety
				//.add("ise_unite","1")
				//.add("rst","entirety")
				/*1.全维度(准确度分、流畅度分、完整度打分) ,extra_ability值为multi_dimension    
				  2.支持因素错误信息显示(声韵、调型是否正确),extra_ability值为syll_phone_err_msg
				  3.单词基频信息显示（基频开始值、结束值）,extra_ability值为pitch ，仅适用于单词和句子题型
			      4.(字词句篇均适用,如选多个能力，用分号;隔开如:syll_phone_err_msg;pitch;multi_dimension)*/
				//.add("extra_ability","multi_dimension")

				//试卷部分添加拼音,限制条件：添加拼音的汉字个数不超过整个试卷中汉字个数的三分之一。
				//jin1|tian1|天气怎么样支持

				//分制转换，rst=entirety是默认值，请根据文档推荐选择使用百分制


				.add("sub",sub)
				.add("ent",ent)
				.add("tte", "utf-8")
				.add("cmd", "ssb")
				.add("auf", "audio/L16;rate=16000")
				.add("aue", getAue(this.voiceType))
				//评测文本(new String(new byte[] { (byte) 0xEF, (byte) 0xBB, (byte) 0xBF })+text)
				.add("text",'\uFEFF'+inputText)//Base64.getEncoder().encodeToString(text.getBytes())
						).add("data", new ParamBuilder()
						.add("status", 0)
						.add("data", ""));
		//System.err.println(p.toString());
		webSocket.send(p.toString());
	}
	
	//客户端给服务端发送数据
	public void send(WebSocket webSocket, int aus,int status, String data) {
		ParamBuilder p = new ParamBuilder();
		p.add("business", new ParamBuilder()
		.add("cmd", "auw")
		.add("aus", aus)
		.add("aue", getAue(this.voiceType))
				).add("data",new ParamBuilder()
				.add("status",status)
				.add("data",data)
				.add("data_type",1)
				.add("encoding", getAue(this.voiceType))
						);
		//LOG.info("发送的数据"+p.toString());
		webSocket.send(p.toString());
	}
	
	private String getAue(PracticeVoiceTypeEnum voiceType) {
		String result = null;
		switch(voiceType) {
		case MP3:
			result = "lame";
			break;
		case PCM:
			result = "raw";
			break;
		case WAV:
			result = "raw";
			break;
		default:
			break;
		}
		return result;
	}

}
