package com.yizhi.practice.application.third.xunfei;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.Base64;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.gson.JsonObject;
import com.yizhi.practice.application.enums.PracticeVoiceTypeEnum;
import com.yizhi.practice.application.enums.PracticeXunFeiBusinessTypeEnum;
import com.yizhi.practice.application.third.util.FrameUtil;

import lombok.Builder;
import lombok.experimental.Tolerate;
import okhttp3.WebSocket;

/**
 * 
 * @author xiaoyu
 *
 */
@Builder
public class RecognizeRunnable implements Runnable {

	private static final Logger LOG = LoggerFactory.getLogger(RecognizeRunnable.class);
	private WebSocket webSocket;
	
	private InputStream fileStream;
    private String appId;
    private PracticeVoiceTypeEnum voiceType;
    
    private Integer recognizeFactor;
	private Integer evaluateFactor;
	
    @Tolerate
	public RecognizeRunnable() {
		super();
	}

	@Override
	public void run() {
		//连接成功，开始发送数据
        int frameSize = FrameUtil.determineFrameSize(PracticeXunFeiBusinessTypeEnum.RECOGNIZE, this.recognizeFactor, this.evaluateFactor); //每一帧音频的大小,建议每 40ms 发送 122B
        int intervel = 40;
        int status = 0;  // 音频的状态
        try {
            byte[] buffer = new byte[frameSize];
            // 发送音频
            end:
            while (true) {
                int len = this.fileStream.read(buffer);
                if (len == -1) {
                    status = Constants.StatusLastFrame;  //文件读完，改变status 为 2
                }
                switch (status) {
                    case Constants.StatusFirstFrame:   // 第一帧音频status = 0
                        JsonObject frame = new JsonObject();
                        JsonObject business = new JsonObject();  //第一帧必须发送
                        JsonObject common = new JsonObject();  //第一帧必须发送
                        JsonObject data = new JsonObject();  //每一帧都要发送
                        // 填充common
                        common.addProperty("app_id", this.appId);
                        //填充business
                        business.addProperty("language", "zh_cn");
                        //business.addProperty("language", "en_us");//英文
                        //business.addProperty("language", "ja_jp");//日语，在控制台可添加试用或购买
                        //business.addProperty("language", "ko_kr");//韩语，在控制台可添加试用或购买
                        //business.addProperty("language", "ru-ru");//俄语，在控制台可添加试用或购买
                        business.addProperty("domain", "iat");
                        business.addProperty("accent", "mandarin");//中文方言请在控制台添加试用，添加后即展示相应参数值
                        //business.addProperty("nunum", 0);
                        //business.addProperty("ptt", 0);//标点符号
                        //business.addProperty("rlang", "zh-hk"); // zh-cn :简体中文（默认值）zh-hk :繁体香港(若未授权不生效，在控制台可免费开通)
                        //business.addProperty("vinfo", 1);
                        business.addProperty("dwa", "wpgs");//动态修正(若未授权不生效，在控制台可免费开通)
                        //business.addProperty("nbest", 5);// 句子多候选(若未授权不生效，在控制台可免费开通)
                        //business.addProperty("wbest", 3);// 词级多候选(若未授权不生效，在控制台可免费开通)
                        //填充data
                        data.addProperty("status", Constants.StatusFirstFrame);
                        data.addProperty("format", "audio/L16;rate=16000");
                        
                        data.addProperty("encoding", getEncoding(this.voiceType)); // lame: mp3
                        data.addProperty("audio", Base64.getEncoder().encodeToString(Arrays.copyOf(buffer, len)));
                        //填充frame
                        frame.add("common", common);
                        frame.add("business", business);
                        frame.add("data", data);
                        webSocket.send(frame.toString());
                        status = Constants.StatusContinueFrame;  // 发送完第一帧改变status 为 1
                        break;
                    case Constants.StatusContinueFrame:  //中间帧status = 1
                        JsonObject frame1 = new JsonObject();
                        JsonObject data1 = new JsonObject();
                        data1.addProperty("status", Constants.StatusContinueFrame);
                        data1.addProperty("format", "audio/L16;rate=16000");
                        data1.addProperty("encoding", getEncoding(this.voiceType));
                        data1.addProperty("audio", Base64.getEncoder().encodeToString(Arrays.copyOf(buffer, len)));
                        frame1.add("data", data1);
                        webSocket.send(frame1.toString());
                        // System.out.println("send continue");
                        break;
                    case Constants.StatusLastFrame:    // 最后一帧音频status = 2 ，标志音频发送结束
                        JsonObject frame2 = new JsonObject();
                        JsonObject data2 = new JsonObject();
                        data2.addProperty("status", Constants.StatusLastFrame);
                        data2.addProperty("audio", "");
                        data2.addProperty("format", "audio/L16;rate=16000");
                        data2.addProperty("encoding", getEncoding(this.voiceType));
                        frame2.add("data", data2);
                        webSocket.send(frame2.toString());
                        LOG.info("Recognize:send last.");
                        break end;
                }
                Thread.sleep(intervel); //模拟音频采样延时
            }
            LOG.info("Recognize:all data is send");
        } catch (FileNotFoundException e) {
        	LOG.error("Exception occurred:", e);
        } catch (IOException e) {
        	LOG.error("Exception occurred:", e);
        } catch (InterruptedException e) {
        	LOG.error("Exception occurred:", e);
        } finally {
        	try {
				this.fileStream.close();
			} catch (IOException e) {
				LOG.error("Error occurred:", e);
			}
        }
	}
	
	/**
	 * 参数encoding对于不同的文件格式，参数的值不同
	 * raw：原生音频（支持单声道的pcm）
		speex：speex压缩后的音频（8k）
		speex-wb：speex压缩后的音频（16k）
		请注意压缩前也必须是采样率16k或8k单声道的pcm。
		lame：mp3格式（仅中文普通话和英文支持，方言及小语种暂不支持）
	 * @param voiceType
	 * @return
	 */
	private String getEncoding(PracticeVoiceTypeEnum voiceType) {
		String result = null;
		switch(voiceType) {
		case MP3:
			result = "lame";
			break;
		case PCM:
			result = "raw";
			break;
		default:
			break;
		}
		return result;
	}

}
