package com.yizhi.practice.application.third.xunfei;

import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.ExecutorService;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.gson.Gson;
import com.yizhi.practice.application.enums.PracticeVoiceTypeEnum;
import com.yizhi.practice.application.third.xunfei.vo.Decoder;
import com.yizhi.practice.application.third.xunfei.vo.OkCondition;
import com.yizhi.practice.application.third.xunfei.vo.ResponseData;
import com.yizhi.practice.application.third.xunfei.vo.Text;

import lombok.Builder;
import lombok.experimental.Tolerate;
import okhttp3.Response;
import okhttp3.WebSocket;
import okhttp3.WebSocketListener;

/**
 * 
 * @author xiaoyu
 *
 */
@Builder
public class RecognizeTask extends WebSocketListener {

	private static final Logger LOG = LoggerFactory.getLogger(RecognizeTask.class);
	private static final Gson json = new Gson();
	@Builder.Default
	private Decoder decoder = new Decoder();
    // 开始时间
    private static Date dateBegin = new Date();
    // 结束时间
    private static Date dateEnd = new Date();
    private static final SimpleDateFormat sdf = new SimpleDateFormat("yyy-MM-dd HH:mm:ss.SSS");
    
    private InputStream fileStream;
    private String appId;
    
    private OkCondition condition;
    private ExecutorService threadPool;
    private PracticeVoiceTypeEnum voiceType;
    private Integer recognizeFactor;
	private Integer evaluateFactor;
    
    @Tolerate
	public RecognizeTask() {
		super();
	}

	@Override
	public void onOpen(WebSocket webSocket, Response response) {
		super.onOpen(webSocket, response);
		this.threadPool.submit(RecognizeRunnable.builder().webSocket(webSocket)
				.fileStream(this.fileStream)
				.appId(this.appId)
				.voiceType(this.voiceType)
				.recognizeFactor(this.recognizeFactor)
				.evaluateFactor(this.evaluateFactor)
				.build());
	}

	@Override
	public void onMessage(WebSocket webSocket, String text) {
		super.onMessage(webSocket, text);
		ResponseData resp = json.fromJson(text, ResponseData.class);
        if (resp != null) {
            if (resp.getCode() != 0) {
            	LOG.info("Received failed result.");
            	LOG.info("code=>" + resp.getCode() + " error=>" + resp.getMessage() + " sid=" + resp.getSid());
                closeWebSocket(webSocket);
                return;
            }
            if (resp.getData() != null) {
            	if (resp.getData().getResult() != null) {
                    Text te = resp.getData().getResult().getText();
                    try {
                        this.decoder.decode(te);
                        LOG.info("中间识别结果 [{}]", decoder.toString());
                    } catch (Exception e) {
                        LOG.error("Error occurred:", e);
                    }
                }
                if (resp.getData().getStatus() == 2) {
                    // resp.data.status ==2 说明数据全部返回完毕，可以关闭连接，释放资源
                    dateEnd = new Date();
                    String result = decoder.toString();
                    condition.ok(result);
                    LOG.info(sdf.format(dateBegin) + "start");
                    LOG.info(sdf.format(dateEnd) + "end");
                    LOG.info("Last result:[{}]", result);
                    LOG.info("sid :[{}]", resp.getSid());
                    decoder.discard();
                    closeWebSocket(webSocket);
                }
            }
        }
	}

	@Override
	public void onFailure(WebSocket webSocket, Throwable t, Response response) {
		super.onFailure(webSocket, t, response);
		try {
            if (null != response) {
                int code = response.code();
                LOG.info("onFailure code:" + code);
                LOG.info("onFailure body:" + response.body().string());
                if (101 != code) {
                    LOG.info("connection failed");
                }
            }
        } catch (IOException e) {
            LOG.error("Fail when resolve response body:",e);
        } finally {
        	closeWebSocket(webSocket);
        }
	}
	
	private void closeWebSocket(WebSocket webSocket) {
		webSocket.close(1000, "");
	}

}
