package com.yizhi.practice.application.third.xunfei;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.gson.Gson;
import com.google.gson.JsonObject;
import com.yizhi.practice.application.enums.PracticeVoiceTypeEnum;
import com.yizhi.practice.application.third.xunfei.vo.OkCondition;
import com.yizhi.practice.application.third.xunfei.vo.ResponseData;

import okhttp3.Response;
import okhttp3.WebSocket;
import okhttp3.WebSocketListener;

/**
 * 
 * @author xiaoyu
 *
 */
public class SynthesisTask extends WebSocketListener {
	 private static final Logger LOG = LoggerFactory.getLogger(SynthesisTask.class);
	 public static final Gson json = new Gson();
	 
	 private String appId;
	 private String inputText;
	 private Map<String, Object> robotConfig;
	 private OkCondition condition;
	 private PracticeVoiceTypeEnum voiceType;
	 private List<String> results = new ArrayList<String>();

	public SynthesisTask() {
		super();
	}

	public SynthesisTask(String appId, String inputText, Map<String, Object> robotConfig
			, OkCondition condition, PracticeVoiceTypeEnum voiceType) {
		super();
		this.appId = appId;
		this.inputText = inputText;
		this.robotConfig = robotConfig;
		this.condition = condition;
		this.voiceType = voiceType;
	}

	@Override
	public void onOpen(WebSocket webSocket, Response response) {
		super.onOpen(webSocket, response);
         //发送数据
         JsonObject frame = new JsonObject();
         JsonObject business = new JsonObject();
         JsonObject common = new JsonObject();
         JsonObject data = new JsonObject();
         // 填充common
         common.addProperty("app_id", appId);
         //填充business
         business.addProperty("aue", getAue(this.voiceType));
         if(PracticeVoiceTypeEnum.MP3.equals(this.voiceType)) {
        	 business.addProperty("sfl", 1);
         }
         business.addProperty("tte", "UTF8");//小语种必须使用UNICODE编码
         business.addProperty("vcn", (String) this.robotConfig.get("name"));//到控制台-我的应用-语音合成-添加试用或购买发音人，添加后即显示该发音人参数值，若试用未添加的发音人会报错11200
         business.addProperty("pitch", 50);
         business.addProperty("speed", 50);
         //填充data
         data.addProperty("status", 2);//固定位2
         try {
        	 LOG.info("Xunfei string -> voice, ready to send: [{}]", inputText);
             data.addProperty("text", Base64.getEncoder().encodeToString(inputText.getBytes("utf8")));
				//使用小语种须使用下面的代码，此处的unicode指的是 utf16小端的编码方式，即"UTF-16LE"”
             //data.addProperty("text", Base64.getEncoder().encodeToString(text.getBytes("UTF-16LE")));
         } catch (UnsupportedEncodingException e) {
             LOG.error("Error occurred:", e);
         }
         //填充frame
         frame.add("common", common);
         frame.add("business", business);
         frame.add("data", data);
         webSocket.send(frame.toString());
	}

	@Override
	public void onMessage(WebSocket webSocket, String text) {
		super.onMessage(webSocket, text);
		 //处理返回数据
		
        LOG.trace("receive=>" + text);
        ResponseData resp = null;
        try {
            resp = json.fromJson(text, ResponseData.class);
        } catch (Exception e) {
        	LOG.error("Error occurred:", e);
        }
        if (resp != null) {
            if (resp.getCode() != 0) {
                LOG.info("error=>" + resp.getMessage() + " sid=" + resp.getSid());
                closeWebSocket(webSocket);
                return;
            }
            if (resp.getData() != null) {
                String result = resp.getData().getAudio();
                this.results.add(result);
                if (resp.getData().getStatus() == 2) {
                    LOG.info("session end");
                    if(null != this.results) {
                    	LOG.info("Results size from xunfei is {}.", this.results.size());
                    }
                    condition.ok(this.results);
                    closeWebSocket(webSocket);
                }
            }
        }
	}

	@Override
	public void onFailure(WebSocket webSocket, Throwable t, Response response) {
		super.onFailure(webSocket, t, response);
		try {
            if (null != response) {
                int code = response.code();
                LOG.info("onFailure code:" + code);
                LOG.info("onFailure body:" + response.body().string());
                if (101 != code) {
                    LOG.info("connection failed");
                }
            }
        } catch (IOException e) {
            LOG.error("Fail when resolve response body:",e);
        } finally {
        	closeWebSocket(webSocket);
        }
	}

	private void closeWebSocket(WebSocket webSocket) {
		webSocket.close(1000, "");
	}
	
	private String getAue(PracticeVoiceTypeEnum voiceType) {
		String result = null;
		switch(voiceType) {
		case MP3:
			result = "lame";
			break;
		case PCM:
			result = "raw";
			break;
		default:
			break;
		}
		return result;
	}
}
