package com.yizhi.practice.application.third.xunfei;

import java.util.concurrent.ExecutorService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import com.yizhi.practice.application.third.aliyun.AliyunUtil;

import okhttp3.OkHttpClient;

/**
 * 
 * @author xiaoyu
 *
 */
@Configuration
public class XunfeiConfig {
	
	// 语音识别 api url
	@Value("${third.xunfei.recognizeUrl}")
	private String recognizeUrl;
	
	// 语音合成 api url
	@Value("${third.xunfei.synthesisUrl}")
	private String synthesisUrl;
	
	// 语音评测 api url
	@Value("${third.xunfei.evaluateUrl}")
	private String evaluateUrl;
	
	@Value("${third.xunfei.appId}")
	private String appId;
	
	@Value("${third.xunfei.apiSecret}")
	private String apiSecret;
	
	@Value("${third.xunfei.apiKey}")
	private String apiKey;
	
	@Value("${third.xunfei.maxWait.ms}")
	private Long maxWait;
	
	@Value("${third.xunfei.eachWait.ms}")
	private Long waitEachRound;
	
	@Autowired
	private AliyunUtil uploader;
	
	@Autowired
	@Qualifier("practiceThreadPool")
	private ExecutorService threadPool;
	
	@Value("${third.xunfei.recognize.framesize}")
	private Integer recognizeFactor;
	
	@Value("${third.xunfei.evaluate.framesize}")
	private Integer evaluateFactor;
	
	@Bean
	public OkHttpClient okHttpClient() {
		return new OkHttpClient.Builder().build();
	}
	
	@Bean
	public XunfeiUtil xunfeiUtil() {
		return XunfeiUtil.builder()
				.recognizeUrl(this.recognizeUrl)
				.synthesisUrl(this.synthesisUrl)
				.evaluateUrl(this.evaluateUrl)
				.appId(this.appId)
				.apiKey(this.apiKey)
				.apiSecret(this.apiSecret)
				.aliyunUtil(this.uploader)
				.okHttpClient(okHttpClient())
				.threadPool(this.threadPool)
				.maxWait(this.maxWait)
				.waitEachRound(this.waitEachRound)
				.evaluateFactor(this.evaluateFactor)
				.recognizeFactor(this.recognizeFactor)
				.build();
	}
	
	
}
