package com.yizhi.practice.application.xservice;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.yizhi.practice.application.pojo.vo.*;
import com.yizhi.practice.application.request.*;
import com.yizhi.util.application.domain.BizResponse;
import com.yizhi.util.application.page.PageInfo;

import java.util.Date;
import java.util.List;

/**
 * @Date 2020/10/22 1:43 下午
 * @Author lvjianhui
 **/
public interface PracticeXservice {


    /**
     * 初始化空白草稿状态的陪练
     * @return 陪练信息
     */
    PracticeBlankDraftVo initBlankDraftPractice();

    /**
     * 保存智能陪练配置信息
     */
    BizResponse savePracticeConfInfo(PracticeConfigInfoReq practiceConfigInfoReq);

    /**
     * 保存智能陪练策略配置信息
     */
    BizResponse savePracticeStrategyInfo(PracticeConfigInfoReq practiceConfigInfoReq);

    /**
     * 保存对话
     */
    BizResponse savePracticeDialogue(List<PracticeSceneDialogueReq> sceneNodeList);

    /**
     * 获取机器人列表
     */
    List<RobotVo> getRobotList();

    /**
     * 查询陪练列表
     * @param practicesGetReq 高级查询参数
     * @return 陪练列表
     */
    BizResponse<PageInfo<PracticeBaseInfoVo>> getPracticeList(PracticesGetReq practicesGetReq);

    /**
     * 查询陪练配置信息：基本信息、策略信息
     * @param practiceId 陪练id
     */
    PracticeConfigInfoVo getPracticeConfigInfo(Long practiceId);

    /**
     * 查询陪练下所有场景对话
     * @param practiceId 陪练id
     */
    List<PracticeSceneDialogueVo> getPracticeSceneChatList(Long practiceId,Long sceneId) throws JsonProcessingException;

    /**
     * 智能陪练上架、下架
     * @param practiceId 陪练id
     * @param status 陪练状态 上架、下架
     */
    BizResponse updatePracticeStatus(Long practiceId, Integer status) throws Exception;

    /**
     * 保存智能陪练用户信息
     * @param practiceAccountReq 陪练用户或者部门信息
     */
    BizResponse addPracticeAccount(PracticeAccountReq practiceAccountReq);

    /**
     * 查询陪练下可见范围的用户和部门
     * @param practiceId 陪练id
     */
    PracticeVisibleAccountVo getPracticeVisibleList(Long practiceId);

    /**
     * 导入陪练对话
     * @param sceneChatImportReq 对话文件信息
     */
    BizResponse importPracticeChat(SceneChatImportReq sceneChatImportReq) throws Exception;

    /**
     * 清空场景对话
     * @param sceneId 场景id
     */
    BizResponse clearSceneChat(Long sceneId);
    
    /**
     * 复制 陪练
     * @param practiceId
     * @return
     */
    public Long copyPractice(Long practiceId) throws Exception;

    /**
     * 删除陪练
     * @param practiceId 陪练id
     */
    BizResponse delete(Long practiceId);

    /**
     * 导入用户
     */
    BizResponse importAccount(PracticeImportAccountReq importAccountReq);

    /**
     * 导出可见范围
     */
    BizResponse exportAccount(Long practiceId);

    /**
     * 校验陪练下场景以及场景对话是否符合规范
     * @param practiceId 陪练id
     */
    BizResponse checkPracticeSceneValid(Long practiceId);

    /**
     * 查询指定时间范围内更新的陪练信息
     */
    List<PracticeVo> selectRangeList(Date startDate, Date endDate);

    /**
     * 查询指定时间范围内更新的陪练信息
     */
    List<PracticeVisibleVo> selectVisibleRangeList(Date startOfDate, Date endOfDate);
}
