package com.yizhi.practice.application.xservice.impl;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.mysql.jdbc.StringUtils;
import com.yizhi.practice.application.pojo.dto.PracticeChartRoundLogDto;
import com.yizhi.practice.application.pojo.vo.LocalJSONObject;
import com.yizhi.practice.application.pojo.vo.PracticeChatLogBaseInfoVo;
import com.yizhi.practice.application.pojo.vo.PracticeChatLogSceneVo;
import com.yizhi.practice.application.pojo.vo.PracticeChatRoundLogDetailVo;
import com.yizhi.practice.application.pojo.vo.PracticeChatRoundLogVo;
import com.yizhi.practice.application.service.IPracticeChatLogService;
import com.yizhi.practice.application.service.IPracticeChatRoundLogService;
import com.yizhi.practice.application.util.LocalStringUtil;
import com.yizhi.practice.application.xservice.PracticeChatRoundLogXService;

@Service
public class PracticeChatRoundLogXServiceImpl implements PracticeChatRoundLogXService {

	@Autowired
	private IPracticeChatRoundLogService roundService;
	
	@Autowired
	private IPracticeChatLogService chatService;
	
	@Override
	public PracticeChatLogBaseInfoVo getLogByPracticeChatId(Long practiceChatId) {
		PracticeChatLogBaseInfoVo base = chatService.get(practiceChatId);
		List<PracticeChatRoundLogVo> details = roundService.getByPracticeChatId(practiceChatId);
		List<PracticeChatRoundLogVo> maxScoreDetails = new ArrayList<PracticeChatRoundLogVo>();
		
		// 按照场景ID,对话层级分组，选出同一个场景，同一个对话层级中分数最高的（同一个会话可以重复作答，选出分数最高的）
		details.stream().collect(Collectors.groupingBy(PracticeChatRoundLogVo::getSceneId
														, Collectors.groupingBy(PracticeChatRoundLogVo::getRoundLevel
																, Collectors.maxBy(Comparator.comparing(PracticeChatRoundLogVo::getCreatedAt)))))
		.forEach((k, v) -> {v.forEach((k1, v1) -> maxScoreDetails.add(v1.get()));});
		
		// 对上一步的结果 按照 场景sort，对话层级 进行排序
		List<PracticeChatLogSceneVo> scenes = maxScoreDetails.stream()
				.sorted(Comparator.comparing(PracticeChatRoundLogVo::getSort).thenComparing(Comparator.comparing(PracticeChatRoundLogVo::getRoundLevel)))
		
		// 对上一步排序后的结果 按照 场景ID进行分组
				.collect(Collectors.groupingBy(PracticeChatRoundLogVo::getSceneId, LinkedHashMap::new, Collectors.toList())) 
				.entrySet().stream()
				// 分组后的场景 组装成VO
				.map(kvSet -> PracticeChatLogSceneVo.builder()
						.sceneId(kvSet.getKey())
						.sceneName(kvSet.getValue().get(0).getSceneName())
						.sort(kvSet.getValue().get(0).getSort())
						.chats(kvSet.getValue()).build())
				.collect(Collectors.toList());
		base.setScenes(scenes);
		return base;
	}
	
	/**
	 * 返回json对象集合，json对象生成规则<br/>
	 * ：1.如果关键词命中，则key为关键词，value为空字符串；<br/>
	 * 2.如果关键词没命中，如果拓展词命中，则key为关键词，value是命中的首个拓展词<br/>
	 * 3.如果关键词没命中，拓展词没命中，则key为关键词，value为空字符串
	 * @param hitKeywordString 命中的关键词，以及拓展词 的字符串，分号分隔
	 * @param keywordString 话术配置的关键词以及拓展词的字符串，形如："{'关键词1':'拓展词11;拓展词12', '关键词2':'拓展词2'}"
	 * @return 返回这样的json对象集合，形如：[{'关键词1':'拓展词2'}, {'关键词2':''},...]
	 */
	public List<LocalJSONObject> constructKeyWordInfo(String hitKeywordString, String keywordString){
		// 没配置关键词拓展词
		if(StringUtils.isNullOrEmpty(keywordString)) {
			return null;
		}
		
		List<LocalJSONObject> result = new ArrayList<LocalJSONObject>();
		
		// 解析配置的关键词拓展词
		Map<String, Object> keywordMap = (Map<String, Object>) JSONObject.parse(keywordString);
		
		// 没命中任何关键词拓展词
		if(StringUtils.isNullOrEmpty(hitKeywordString)) {
			for(String keyword : keywordMap.keySet()) {
				result.add(LocalJSONObject.builder().key(keyword).value("").build());
			}
			return result;
		}
		
		// 解析命中的关键词拓展词
		List<String> hitWords = LocalStringUtil.hit(hitKeywordString);
		
		// 遍历配置的关键词拓展词，构造json对象
		for(String keyword : keywordMap.keySet()) {
			// 关键词直接命中
			if(hitWords.contains(keyword)) {
				result.add(LocalJSONObject.builder().key(keyword).value("").build());
			}else {
				// 关键词没命中，判断拓展词是否命中
				result.add(LocalJSONObject.builder()
						.key(keyword)
						.value(findHitExtendWord((String)keywordMap.get(keyword), hitWords))
						.build());
			}
		}
		
		return result;
	}
	
	private String findHitExtendWord(String allExtendWord, List<String> hitWords) {
		String hitExtendWord = "";
		for(String hit : hitWords) {
			if(allExtendWord.indexOf(hit) != -1) {
				hitExtendWord = hit;
				break;
			}
		}
		return hitExtendWord;
	}

	@Override
	public PracticeChatRoundLogDetailVo analysis(Long chatRoundId) {
		PracticeChartRoundLogDto origin = roundService.getRoundLogById(chatRoundId);
		
		return PracticeChatRoundLogDetailVo.builder()
				.keyWord(constructKeyWordInfo(origin.getAccountHitKeyword(), origin.getKeyWord()))
				.keywordHit(LocalStringUtil.hit(origin.getAccountHitKeyword()))
				.forbidHit(LocalStringUtil.hit(origin.getForbidHit()))
				.totalScore(origin.getTotalScore())
				.keywordScore(origin.getKeywordScore())
				.completeScore(origin.getCompleteScore())
				.smoothScore(origin.getSmoothScore())
				// TODO 分数段 提示语 map
				.smoothHint("您的话术很流畅")
				.expressScore(origin.getExpressScore())
				// TODO 分数段 提示语 map
				.expressHint("您的表达能力很不错")
				.politeScore(origin.getPoliteScore())
				// TODO 分数段 提示语 map
				.politeHint("您未使用礼貌用语")
				.accountContent(origin.getAccountContent())
				.accountContentVoiceUrl(origin.getAccountContentVoiceUrl())
				.accountDuration(origin.getAccountDuration())
				.robotContent(origin.getRobotContent())
				.robotContentUrl(origin.getRobotContentUrl())
				.robotDuration(origin.getRobotDuration())
				.build();
	}
	
}
