package com.yizhi.practice.application.xservice.impl

import cn.hutool.core.util.RandomUtil
import com.alibaba.fastjson.JSON
import com.alibaba.fastjson.JSONObject
import com.yizhi.application.orm.id.IdGenerator
import com.yizhi.core.application.context.ContextHolder
import com.yizhi.core.application.context.RequestContext
import com.yizhi.core.application.enums.InternationalEnums
import com.yizhi.core.application.exception.BizException
import com.yizhi.point.application.feign.PointRedisFeignClients
import com.yizhi.point.application.vo.PointParamVO
import com.yizhi.practice.application.config.AudioUtils
import com.yizhi.practice.application.config.PracticeLevelEnum
import com.yizhi.practice.application.config.PracticeRunningStatusEnum
import com.yizhi.practice.application.constants.PracticeConstant
import com.yizhi.practice.application.constants.PracticePointConstant
import com.yizhi.practice.application.enums.*
import com.yizhi.practice.application.enums.PracticePassResultEnum.PASS
import com.yizhi.practice.application.enums.PracticePassResultEnum.UN_PASS
import com.yizhi.practice.application.mapper.PracticeChatRoundLogMapper
import com.yizhi.practice.application.pojo.domain.*
import com.yizhi.practice.application.pojo.dto.PracticeAllScoreDto
import com.yizhi.practice.application.pojo.po.AnalyzeStudentVoicePo
import com.yizhi.practice.application.pojo.po.StartPracticePO
import com.yizhi.practice.application.pojo.vo.*
import com.yizhi.practice.application.repository.*
import com.yizhi.practice.application.service.*
import com.yizhi.practice.application.third.aliyun.AliyunUtil
import com.yizhi.practice.application.third.baidu.BaiduUtil
import com.yizhi.practice.application.third.util.OssFileUtil
import com.yizhi.practice.application.third.xunfei.XunfeiUtil
import com.yizhi.practice.application.xservice.PracticeStuChatXService
import com.yizhi.system.application.system.remote.OrganizationClient
import com.yizhi.wechat.application.feign.AccessTokenClient
import com.yizhi.wechat.application.feign.WeChatTokenClient
import org.apache.commons.io.FileUtils
import org.apache.commons.lang.StringUtils
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Service
import ws.schild.jave.*
import java.io.File
import java.io.InputStream
import java.math.BigDecimal
import java.math.RoundingMode
import java.net.URL
import java.util.*
import kotlin.collections.ArrayList
import kotlin.collections.HashMap
import kotlin.random.Random


/**
 * @Date 2020/10/28 4:49 下午
 * @Author lvjianhui
 **/
@Suppress("NULLABILITY_MISMATCH_BASED_ON_JAVA_ANNOTATIONS")
@Service
class PracticeStuXServiceImpl : PracticeStuChatXService {

    @Autowired
    lateinit var practiceChatLogRepository: PracticeChatLogRepository

    @Autowired
    lateinit var practiceChatLogService: IPracticeChatLogService

    @Autowired
    lateinit var practiceConfigRepository: PracticeConfigRepository

    @Autowired
    lateinit var sceneConfigService: ISceneConfigService

    @Autowired
    lateinit var sceneChatConfigService: ISceneChatConfigService

    @Autowired
    lateinit var practiceChatRoundLogService: IPracticeChatRoundLogService

    @Autowired
    lateinit var practiceChatroundLogRepository: PracticeChatRoundLogRepository

    @Autowired
    lateinit var sceneChatConfigRepository: SceneChatConfigRepository

    @Autowired
    lateinit var iPracticePointRuleConfigService: IPracticePointRuleConfigService

    @Autowired
    lateinit var iPracticePointGivenLogService: IPracticePointGivenLogService

    @Autowired
    lateinit var idGenerator: IdGenerator
    private val logger = org.slf4j.LoggerFactory.getLogger(javaClass)

    @Autowired
    lateinit var xunfeiUtil: XunfeiUtil

    @Autowired
    lateinit var baiduUtil: BaiduUtil

    @Value("\${yizhi.practice.polite.word}")
    var politeWord: String? = null

    @Autowired
    lateinit var practicePointGivenLogRepository: PracticePointGivenLogRepository

    @Autowired
    lateinit var practiceChatRoundLogMapper: PracticeChatRoundLogMapper

    //@Autowired
   // lateinit var amqpTemplate: AmqpTemplate

    @Autowired
    lateinit var pointRedisFeignClients: PointRedisFeignClients

    @Autowired
    lateinit var organizationClient: OrganizationClient

    @Autowired
    lateinit var accessTokenClient: AccessTokenClient
    @Autowired
    lateinit var weChatTokenClient: WeChatTokenClient

    @Autowired
    lateinit var aliyunUtil: AliyunUtil

    constructor()

    override fun getRobotLevelChat(practiceId: Long, sceneId: Long, chatRoundConfigId: Long, useType: Int): List<ChatContextVo> {
        val list = sceneChatConfigService.selectChatNodes(practiceId, sceneId, PracticeNodeEnum.ROBOT.status, chatRoundConfigId, null)
        val vos = arrayListOf<ChatContextVo>()
        list.forEach {
            val vo = ChatContextVo()
            vo.chatId = it.id
            vo.context = it.robotContent
            vo.voiceUrl = it.robotContentVoiceUrl
            vo.voiceDuration = it.robotContentVoiceDuration
            vos.add(vo)
        }
        return vos.shuffled()
    }

    override fun deletedAccountUploadId(accountUploadId: Long, context: RequestContext) {
        val log = practiceChatroundLogRepository.findById(accountUploadId).get()
        log.deleted = true
        practiceChatroundLogRepository.save(log)
    }

    override fun getLogResult(practiceLogId: Long, accountId: Long): PracticeEndResultVo? {
        val log = practiceChatLogService.get(practiceLogId) ?: let {
            logger.error("陪练结束时查询陪练记录出错 practiceLogId：$practiceLogId")
            throw BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2)
        }
        val vo = PracticeEndResultVo()
        return vo
    }

    override fun endPractice(practiceId: Long, practiceLogId: Long, useType: Int, requestContext: RequestContext, errorEnd: Boolean): PracticeEndResultVo {

        val practiceConfig = practiceConfigRepository.getOne(practiceId) ?: let {
            logger.error("查询陪练参数有误 practiceId:${practiceId}")
            throw BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2)
        }
        val log = practiceChatLogRepository.findById(practiceLogId).get()

        if (log.status != PracticeChatLogStatusEnum.ING.status) {
            logger.error("陪练结束时陪练状态出错 practiceLogId：$practiceLogId status:${log.status}")
            throw BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2)
        }
        val vo = PracticeEndResultVo()
        if (errorEnd) {
            log.status = PracticeChatLogStatusEnum.QUIT.status
            log.endAt = Date()
            practiceChatLogRepository.save(log)
            return vo
        }
        log.status = PracticeChatLogStatusEnum.DONE.status
        log.endAt = Date()
        val dto = practiceChatRoundLogMapper.getScoreByAccount(practiceLogId, requestContext.accountId, requestContext.orgId, requestContext.siteId, requestContext.companyId)
        logger.info("all Score" + JSONObject.toJSONString(dto) + "practiceLogId :$practiceLogId accountIdL:${requestContext.accountId} orgId:${requestContext.orgId}")
        log.keywordScore = dto?.keyWordScore?.toInt() ?: 0
        log.keywordRate = practiceConfig.scoreRuleKeyword
        log.completeScore = dto?.complateScore?.toInt() ?: 0
        log.completeRate = practiceConfig.scoreRuleComplete
        log.politeScore = dto?.politeScore?.toInt() ?: 0
        log.politeRate = practiceConfig.scoreRulePolite
        log.expressScore = dto?.expressScore?.toInt() ?: 0
        log.expressRate = practiceConfig.scoreRuleExpress
        log.smoothScore = dto?.fluencyScore?.toInt() ?: 0
        log.smoothRate = practiceConfig.scoreRuleSmooth
        log.averageScore = dto?.averageScore?.toInt() ?: 0
        vo.score = log.averageScore
        if (null == log.passMinScore) {
            practiceChatLogRepository.save(log)
            return vo
        }
        if (log.averageScore >= log.passMinScore) {
            log.passResult = PASS.status
            vo.passSuccess = true
        } else {
            log.passResult = UN_PASS.status
            vo.passSuccess = false
        }
        practiceChatLogRepository.save(log)

        // 送积分
        givenPoint(log, practiceConfig.name)
        return vo
    }

    private fun givenPoint(practiceLog: PracticeChatLog, practiceName: String) {

        if (practiceLog.useType == PracticeModelStatusEnum.PRACTICE.status) {
            return
        }

        if (practiceLog.givenPointType != PracticePointGivenEnum.TRUE.status) {
            return
        }

        val score = practiceLog.averageScore ?: 0
        val configRule = iPracticePointRuleConfigService.selectPointRulesByPractice(practiceLog.practiceId, score, false)
                ?: return

        // 统计已送出的分数
        val pointSum = iPracticePointGivenLogService.sumPointByUserAndPractice(
                accountId = practiceLog.accountId,
                practiceId = practiceLog.practiceId,
                companyId = practiceLog.companyId,
                siteId = practiceLog.siteId,
                orgId = practiceLog.orgId)
        val planGivenPoint = configRule.point ?: let {
            return
        }
        val actualPoint = planGivenPoint - pointSum
        if (actualPoint <= 0) {
            logger.info("完成对话不赠送积分,得分已够  accountId:${practiceLog.accountId} ${practiceLog.accountFullName} planGivenPoint:$planGivenPoint pointSum:$pointSum")
            return
        }

        // 更新log
        practiceLog.status = PracticeChatLogStatusEnum.DONE.status
        practiceLog.endAt = Date()
        practiceLog.givenPointCount = actualPoint
        practiceChatLogRepository.save(practiceLog)

        // 送积分
        val givenLog = PracticePointGivenLog()
        givenLog.accountId = practiceLog.accountId
        givenLog.companyId = practiceLog.companyId
        givenLog.siteId = practiceLog.siteId
        givenLog.orgId = practiceLog.orgId
        givenLog.practiceId = practiceLog.practiceId
        givenLog.practiceScore = practiceLog.averageScore
        givenLog.point = actualPoint
        givenLog.pointRuleId = configRule.id
        givenLog.id = idGenerator.generate()
        practicePointGivenLogRepository.save(givenLog)
        mqSendPoint(givenLog, practiceName)
    }

    private fun mqSendPoint(pointLog: PracticePointGivenLog, practiceName: String) {
        val pointParamVO = PointParamVO()
        pointParamVO.eventName = PracticePointConstant.POINT_EVENT_NAME //事件名称
        pointParamVO.activityType = PracticePointConstant.POINT_ACTIVITY_TYPE //事件类型
        pointParamVO.activitySource = PracticePointConstant.POINT_ACTIVITY_SOURCE //学习活动来源
        pointParamVO.activityName = practiceName //学习活动名称
        pointParamVO.releaseCondition = "范围发放" //发放条件
        pointParamVO.releaseRules = "按照范围取积分" //发放规则
        pointParamVO.createPointTime = Date() //积分创建时间
        pointParamVO.sourceId = pointLog.id //积分来源ID，（业务ID主键）
        pointParamVO.operatingPoint = pointLog.point //积分数量，  正数 加积分，  负数减积分
        pointParamVO.accountId = pointLog.accountId //学员ID
        pointParamVO.companyId = pointLog.companyId //企业ID
        pointParamVO.siteId = pointLog.siteId //站点ID
        pointParamVO.orgId = pointLog.orgId //组织ID
        pointParamVO.score = pointLog.point
        val sitePointId = pointRedisFeignClients.addPointRedis(pointParamVO) //调用积分接口添加积分
        //amqpTemplate.convertAndSend(pointParamVO.eventName, sitePointId) //发送积分获取消息
    }

    // 第二步 查看是否保存上句机器人话术log,如果没有 则保存，否则nothing
    // 第三步 调用第三发打分，计算得分，保存log
    // 第四步 如果是考试，判断语意相识度
    // 第五步 根据考试或者陪练，计算是否通过。
    // 最后返回结果
    override fun analyzeStudentVoice(po: AnalyzeStudentVoicePo): ChatResultVo {
        if (!po.ignore) {
            buildAliOSSUrl(po)
        }
        var inputStream: InputStream? = null
        var pcmFile: File? = null
        try {
            if (!PracticeModelStatusEnum.getAllStatus().contains(po.useType)) {
                logger.error("陪练参数错误:${po.useType}")
                throw BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2)
            }
            val practiceConfig = practiceConfigRepository.getOne(po.practiceId!!) ?: let {
                logger.error("查询陪练参数有误 practiceId:${po.practiceId}")
                throw BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2)
            }
            // 得分
            var allScore: PracticeAllScoreDto? = null
            // 学员话术
            var accountUploadContext: String? = null

            var similar: Pair<Int, Long>? = null

            var accountChatConfigId = po.chatRoundConfigId
            var accountChatConfig = sceneChatConfigRepository.getOne(accountChatConfigId)
            if (!po.ignore) {
                if (StringUtils.isNotEmpty(po.voiceUrl)) {
                    pcmFile = AudioUtils.wavToPcm(po.voiceUrl!!)
                    inputStream = pcmFile!!.inputStream()
                }
                // 解析学员话术
                accountUploadContext = getAccountContent(inputStream!!)
                // 根据学员话术相似度判断走那条分支
                similar = similarScore(po, accountUploadContext)
                accountChatConfigId = similar.second
                accountChatConfig = sceneChatConfigRepository.getOne(accountChatConfigId)
                // 获取学员话术分支
                inputStream = pcmFile!!.inputStream()
                allScore = getAllScore(inputStream, practiceConfig, accountUploadContext, accountChatConfig!!, PracticeModelStatusEnum.getByStatus(po.useType))
                inputStream.close()
                pcmFile.deleteOnExit()
            }

            val robotChatConfig = sceneChatConfigRepository.getOne(accountChatConfig?.pid ?: accountChatConfigId)
            // 保存机器人话术log
            saveRobotChatLog(po, robotChatConfig)

            val accountLog = saveAccountChatLog(
                    practiceConfig.examPassMinScore,
                    practiceConfig.similarityRate,
                    similar?.first,
                    po,
                    accountChatConfig,
                    allScore,
                    accountUploadContext)
            val vo = ChatResultVo()
            vo.accountUploadId = accountLog.id
            vo.hitTabooWord = allScore?.hitTabooWord ?: arrayListOf<String>()
            vo.duration = accountLog.accountContentVoiceDuration
            vo.score = allScore?.average
            vo.voiceUrl = accountLog.accountContentVoiceUrl
            vo.content = accountUploadContext
            vo.chatRoundConfigId = accountChatConfigId
            vo.contentHitKeyWord = accountLog.accountHitKeyword.split(PracticeConstant.KEY_WORD_SPERATOR_EN)

            if (po.ignore) {
                vo.passSuccess = true
                vo.content = accountChatConfig.accountContent
            } else {
                vo.passSuccess = getPassResult(po, allScore!!, practiceConfig, accountChatConfigId, similar)
            }
            if (vo.passSuccess) {
                return vo
            }

            // 失败次数大于允许次数，则弹窗提示
            var limitPushCount = practiceConfig.examFailedMaxCount
            if (po.useType!! == PracticeModelStatusEnum.PRACTICE.status) {
                limitPushCount = practiceConfig.practiceFailedMaxCount
                vo.customizeFileMsg = "您的回答与标准话术不符，请重新回答。"
            } else {
                vo.customizeFileMsg = PracticeConstant.EXAM_FAIL_MESSAGE

            }
            // 未通过且次数大于限制次数
            val pushCount = getLevelChatCunt(po.chatRoundLevel!!, po.practiceLogId!!, practiceConfig.id!!, po.requestContext!!)
            if (pushCount > limitPushCount) {
                vo.unPassTipSskip = true
            }

            // 考核模式下 未通过时且 设置自定已回复
            if (po.useType!! == PracticeModelStatusEnum.ASSESSMENT.status
                    && !vo.passSuccess
                    && practiceConfig.similarityFailedMsgType == PracticeSimilaryFailedEnum.FALSE.status) {
                vo.customizeFileMsg = practiceConfig.similarityFailedMsg.split(PracticeConstant.KEY_WORD_SPERATOR_EN).shuffled().firstOrNull()
            }
            return vo

        } catch (e: Exception) {
            e.printStackTrace()
            logger.error("用户上传语音异常====${po.requestContext!!.accountFullName} ===${po.requestContext!!.orgName} ====${po.requestContext!!.companyName}", e)
            throw BizException(InternationalEnums.TPPLANCONTROLLER6)
        } finally {
            inputStream?.close()
            pcmFile?.deleteOnExit()
        }

    }

    /**
     * 获取是否通过的结果
     */
    private fun getPassResult(po: AnalyzeStudentVoicePo, allScore: PracticeAllScoreDto, practiceConfig: PracticeConfig, accountChatConfigId: Long?, similar: Pair<Int, Long>?): Boolean {
        var passSuccess = po.ignore
        if (passSuccess) return passSuccess
        if (po.useType!! == PracticeModelStatusEnum.PRACTICE.status) {
            when {
                po.ignore -> {
                    passSuccess = true
                }
                practiceConfig.startNextRoundType == PracticeDiaConditionEnum.FALSE.status -> {
                    passSuccess = true
                }
                practiceConfig.startNextRoundScore < allScore.average -> {
                    passSuccess = true
                }

            }
        } else if (po.useType!! == PracticeModelStatusEnum.ASSESSMENT.status) {
            when {
                practiceConfig.manyRoundChat == PracticeManyRoudChatEnum.FALSE.status -> {
                    passSuccess = true
                }
                similar!!.first >= practiceConfig.similarityRate -> {
                    passSuccess = true
                }
                checkPushCount(po.practiceLogId!!, accountChatConfigId!!, po.requestContext!!) > practiceConfig.examFailedMaxCount -> {
                    passSuccess = true
                }

            }
        }
        return passSuccess
    }


    private fun saveAccountChatLog(
            passMinScore: Int?,
            passSimilarityRate: Int?,
            similarityScore: Int?,
            po: AnalyzeStudentVoicePo,
            accountChatConfig: SceneChatConfig,
            allScore: PracticeAllScoreDto?,
            accountVoiceContent: String?
    ): PracticeChatRoundLog {

        val context = po.requestContext!!
        var log = PracticeChatRoundLog();
        log.id = idGenerator.generate()
        log.practiceId = po.practiceId
        log.sceneId = po.senceId
        log.sceneChatId = accountChatConfig.id
        log.practiceChatId = po.practiceLogId
        log.accountId = context.accountId
        log.accountName = context.accountName
        log.accountFullName = context.accountFullName
        log.accountContent = accountVoiceContent
        log.accountContentVoiceDuration = accountChatConfig.accountContentVoiceDuration
        log.accountContentVoiceUrl = accountChatConfig.accountContentVoiceUrl
        log.passMinScore = passMinScore
        log.passSimilarityRate = passSimilarityRate
        log.roundSkipType = PracticeSkipEnum.SKIP.code
        log.roundLevel = po.chatRoundLevel
        log.accountHitTabooWord = allScore?.hitTabooWord?.joinToString(PracticeConstant.KEY_WORD_SPERATOR_EN)
        log.accountHitKeyword =
                JSON.parseObject(accountChatConfig.accountContentKeyword, Map::class.java).keys.joinToString(PracticeConstant.KEY_WORD_SPERATOR_EN)
        if (!po.ignore) {
            log.roundSkipType = PracticeSkipEnum.NOT_SKIP.code
            log.similarityScore = similarityScore
            log.accountContentVoiceUrl = po.voiceUrl
            log.accountContentVoiceDuration = po.voiceDuration
            log.accountHitKeyword = allScore!!.keyWordScore!!.second.joinToString(PracticeConstant.KEY_WORD_SPERATOR_EN)
            log.keywordScore = allScore.keyWordScore!!.first
            log.completeScore = allScore.complateScore
            log.smoothScore = allScore.fluencyScore
            log.expressScore = allScore.expressScore
            log.politeScore = allScore.politeScore
            log.averageScore = allScore.average
        } else {
            log.accountContent = accountChatConfig.accountContent
        }
        log.orgId = context.orgId
        log.siteId = context.siteId
        log.orgName = context.orgName
        log.companyId = context.companyId
        log = practiceChatroundLogRepository.save(log)
        return log
    }

    private fun saveRobotChatLog(po: AnalyzeStudentVoicePo, robotChatConfig: SceneChatConfig) {
        // 首先查看机器人话术有没有保存
        val context = po.requestContext!!
        insertRobotRoundLog(po.practiceId!!,po.senceId!!,po.practiceLogId!!,context,robotChatConfig)
    }

    // 查看指定对话提交次数
    fun checkPushCount(practiceLogId: Long, chatRoundConfigId: Long, context: RequestContext): Long {
        val count = practiceChatRoundLogService.getCountByUserId(
                practiceLogId,
                chatRoundConfigId,
                context.accountId,
                context.orgId,
                context.siteId,
                context.companyId)
        return count
    }

    fun getLevelChatCunt(level: Int, practiceLogId: Long, practiceId: Long, context: RequestContext): Long {
        return practiceChatRoundLogService.getLevelCountByAccount(level, practiceLogId, practiceId, context.accountId, context.orgId, context.siteId, context.companyId)
                ?: 0
    }


    fun getAccountContent(inputStream: InputStream): String {
        return xunfeiUtil.voiceToString(inputStream, PracticeVoiceTypeEnum.PCM)
    }

    /**
     * 相似度得分
     */
    private fun similarScore(po: AnalyzeStudentVoicePo, accountContent: String): Pair<Int, Long> {

        val list = arrayListOf<SceneChatConfig>()

        if (po.useType == PracticeModelStatusEnum.PRACTICE.status) {
            val sceneConfig = sceneChatConfigRepository.findById(po.chatRoundConfigId).get()
            list.add(sceneConfig)
        } else {
            val manyRound = sceneChatConfigService.selectChatNodes(po.practiceId, po.senceId, PracticeNodeEnum.STUDENT.status, null, po.chatRoundLevel)
                    ?: let {
                        logger.error("查询学员标准话术失败。accountId:${po.requestContext!!.accountId},practiceId:${po.practiceId},sceneId:${po.senceId},level:${po.chatRoundLevel}")
                        throw BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2)
                    }
            list.addAll(manyRound)
        }
        val map = HashMap<Int, Long>()
        list.forEach {
            val score = baiduUtil.getSimilarRate(accountContent, it.accountContent).multiply(BigDecimal(100)).toInt()
            map[score] = it.id
        }
        val similarScore = map.keys.sortedDescending().firstOrNull() ?: 0
        // 相似度
        val scenChatId = map[similarScore]!!
        return Pair(similarScore, scenChatId)

    }

    /**
     * 获取各个维度得分
     * 1、评分从五个维度打分：完整性、关键词命中、流畅性、表达能力、礼貌用语；五个维度加起来为100分，各个维度为【0,100】的正整数。
    默认占比：关键词：40、完整性：30、流畅性：20、表达能力：5、礼貌用语：5
    1)关键词命中得分：通过对话中的关键词命中判断得分，一段标准答案中有n个关键词，命中m（m<=n）个，则该段对话关键词得分为: m/n*关键词分数，
    训练模式下：只有关键词命中才算得分
    考核模式下：关键词及关键词对应的拓展词命中算得分
    2) 完整性得分：完整性得分系数（讯飞接口）%*完整性设置的分数，
    3) 流畅性得分：流畅性得分系数（讯飞接口）%*流畅性设置的分数
    4) 表达能力得分：（声韵分系数+调型分系数）/2%*表达能力设置的分数
    5) 礼貌用语得分：使用礼貌用语得分、未使用礼貌用语或命中敏感词则不得分
    1~5中得分四舍五入取整
    2、一句话的综合得分为   K=完整性得分+关键词命中得分+流畅性得分+表达能力得分+礼貌用语得分
    3、陪练综合得分每题得分的平均分，四舍五入取整
    4、若一句对话中命中禁忌词，则该句所有分数项为0分
     */
    fun getAllScore(inputStream: InputStream,
                    practiceconfig: PracticeConfig,
                    accountContent: String?,
                    accountSceneChatConfig: SceneChatConfig,
                    practiceModel: PracticeModelStatusEnum): PracticeAllScoreDto {
        val dto = PracticeAllScoreDto()
        if (StringUtils.isEmpty(accountContent)) {
            return dto;
        }
        val hitTabooWord = arrayListOf<String>()
        if (!StringUtils.isEmpty(accountSceneChatConfig.accountContentTabooWord)) {
            val taboo = accountSceneChatConfig.accountContentTabooWord.replace(PracticeConstant.KEY_WORD_SPERATOR_CN, PracticeConstant.KEY_WORD_SPERATOR_EN).split(PracticeConstant.KEY_WORD_SPERATOR_EN)
            // 命中禁忌词，所有分数为0
            taboo.filter { !StringUtils.isEmpty(it) && accountContent!!.contains(it) }.forEach {
                hitTabooWord.add(it)
            }
            dto.hitTabooWord = hitTabooWord
            if (hitTabooWord.size > 0) {
                return dto
            }
        }

        val result = xunfeiUtil.evaluate(accountSceneChatConfig.accountContent, inputStream, PracticeVoiceTypeEnum.PCM)
        logger.info("打分结果 result:$result")
        dto.fluencyScore = result.fluencyScore.multiply(BigDecimal(practiceconfig.scoreRuleSmooth)).toInt()
        dto.complateScore = result.integrityScore.multiply(BigDecimal(practiceconfig.scoreRuleComplete)).toInt()
        dto.keyWordScore = getKeyWordScore(accountContent!!,
                practiceconfig.scoreRuleKeyword,
                accountSceneChatConfig.accountContentKeyword,
                practiceModel)
        dto.expressScore = (result.phoneScore.add(result.toneScore)).divide(BigDecimal(2)).multiply(BigDecimal(practiceconfig.scoreRuleExpress)).toInt()
        var politeScore = 0
        val politeList = replaceSperatorCn(politeWord!!)
        politeList.filter { accountContent.contains(it) }.forEach { _ ->
            politeScore = practiceconfig.scoreRulePolite
        }
        dto.politeScore = politeScore
        dto.average = dto.fluencyScore + dto.complateScore + dto.keyWordScore.first + dto.expressScore + dto.politeScore
        return dto
    }

    private fun replaceSperatorCn(content: String): List<String> {
        return content.replace(PracticeConstant.KEY_WORD_SPERATOR_CN, PracticeConstant.KEY_WORD_SPERATOR_EN).split(PracticeConstant.KEY_WORD_SPERATOR_EN)
    }

    /**
     * 获取关键词得分
     */
    private fun getKeyWordScore(accountContent: String,
                                scoreRuleKeyword: Int,
                                accountContentKeyword: String,
                                practiceModel: PracticeModelStatusEnum): Pair<Int, List<String>> {
        val keyWord = accountContentKeyword
        val map = JSON.parseObject(keyWord, Map::class.java)
        var count = 0

        val list = arrayListOf<String>()
        // 训练模式
        if (practiceModel.status == PracticeModelStatusEnum.PRACTICE.status) {
            map.keys.filter { !StringUtils.isEmpty(it as String) && accountContent.contains(it) }.forEach {
                count++
                list.add(it as String)
            }
            val score = BigDecimal(count).divide(BigDecimal(map.keys.size), 2, RoundingMode.HALF_UP).multiply(BigDecimal(scoreRuleKeyword))
            return Pair(score.toInt(), list)
        }

        // 考核模式 命中关键词和拓展词均得分
        map.forEach {
            val key = it.key as String
            val valueList = replaceSperatorCn(it.value as String).toMutableList()
            valueList.add(key)
            for (value in valueList) {
                if (!StringUtils.isEmpty(value) && accountContent.contains(value)) {
                    count++
                    list.add(value)
                    return@forEach
                }
            }
        }
        var score = BigDecimal(count).divide(BigDecimal(map.keys.size), 2, RoundingMode.HALF_UP).multiply(BigDecimal(scoreRuleKeyword)).toInt()
        if (score > scoreRuleKeyword) {
            score = scoreRuleKeyword
        }
        return Pair(score, list)
    }

    override fun getStudentContent(sceneId: Long, useType: Int, practiceId: Long, practiceChatLogId: Long, robotChatId: Long): List<StudentChatContextVo>? {
        var list = sceneChatConfigService.selectChatNodes(practiceId, sceneId, PracticeNodeEnum.STUDENT.status, robotChatId, null)
        if (list == null || list.isEmpty()) {
            //TODO 如果学员话术为空，则保存一条机器人话术至roundLog
            val context = ContextHolder.get()
            val robotChatConfig = sceneChatConfigRepository.findById(robotChatId).get()
            insertRobotRoundLog(practiceId, sceneId, practiceChatLogId, context,robotChatConfig)
            return null
        }
        // 如果是考试 只返回随机一条
        if (useType == PracticeModelStatusEnum.ASSESSMENT.status) {
            list = arrayListOf(list[Random.nextInt(0, list.size)])
        }

        val vos = arrayListOf<StudentChatContextVo>()
        list.forEach {
            val vo = StudentChatContextVo()
            vo.chatRoundConfigId = it.id
            vo.context = it.accountContent
            vo.voiceUrl = it.accountContentVoiceUrl
            vo.keyWord = getKeyWord(it.accountContentKeyword)
            vo.level = it.level
            vos.add(vo)
        }
        return vos
    }

    private fun insertRobotRoundLog(practiceId: Long, sceneId: Long, practiceChatLogId: Long, context: RequestContext,robotSceneChatConfig: SceneChatConfig) {
        val count = practiceChatRoundLogService.getCountByUserId(
                practiceChatLogId,
                robotSceneChatConfig.id,
                context.accountId,
                context.orgId,
                context.siteId,
                context.companyId)
        if (count > 0) {
            return
        }
        val log = PracticeChatRoundLog();
        log.id = idGenerator.generate()
        log.practiceId = practiceId
        log.sceneId = sceneId
        log.sceneChatId = robotSceneChatConfig.id
        log.practiceChatId = practiceChatLogId
        log.accountId = context.accountId
        log.accountName = context.accountName
        log.accountFullName = context.accountFullName
        log.robotContent = robotSceneChatConfig.robotContent
        log.robotContentVoiceUrl = robotSceneChatConfig.robotContentVoiceUrl
        log.robotContentVoiceDuration = robotSceneChatConfig.robotContentVoiceDuration
        log.roundLevel = robotSceneChatConfig.level
        log.orgId = context.orgId
        log.siteId = context.siteId
        log.orgName = context.orgName
        log.companyId = context.companyId
        practiceChatroundLogRepository.save(log)
    }

    private fun getKeyWord(keyWordContext: String): List<String> {
        return JSONObject.parseObject(keyWordContext).toMap().keys.toList()
    }

    override fun getRobotFirstChat(practiceId: Long, sceneId: Long, useType: Int): List<ChatContextVo>? {
        val list = sceneChatConfigService.selectChatNodes(practiceId, sceneId, PracticeNodeEnum.ROBOT.status, null, 1)
        if (list == null || list.isEmpty()) {
            return null
        }
        // 如果是考试 只返回随机一条
        if (useType == PracticeModelStatusEnum.ASSESSMENT.status) {
            val chatConfig = list.shuffled().first()
            list.clear()
            list.add(chatConfig)
        }
        val vos = arrayListOf<ChatContextVo>()
        list.forEach {
            val vo = ChatContextVo()
            vo.chatId = it.id
            vo.context = it.robotContent
            vo.voiceUrl = it.robotContentVoiceUrl
            vo.voiceDuration = it.robotContentVoiceDuration
            vos.add(vo)
        }
        return vos
    }

    override fun getAllScene(practiceId: Long): List<SceneChatVO>? {

        val list = sceneConfigService.findSceneByPracticeId(practiceId) ?: let {
            return null
        }

        val vos = ArrayList<SceneChatVO>()
        list.forEach {
            val vo = SceneChatVO();
            vo.sceneDescription = it.description
            vo.sceneName = it.name
            vo.sceneId = it.id
            vos.add(vo)
        }
        return vos
    }

    override fun startPractice(params: StartPracticePO): StuPracticeLogVo {

        if (!PracticeModelStatusEnum.getAllStatus().contains(params.useType)
                || !PracticeLevelEnum.getAlllevel().contains(params.level)) {
            logger.error("startPractice useType :{} is erro Or level :{} is error", params.useType, params.level)
            throw BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2)
        }
        val practice = practiceConfigRepository.getOne(params.practiceId!!)

        if (practice == null) {
            logger.error("startPractice practiceId:{} ,未查询到陪练配置", params.practiceId)
            throw BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2)
        }

        val context = params.requestContext!!

        // 考试需要考虑次数限制
        if (practice.useType != PracticeModelStatusEnum.PRACTICE.status) {
            if (practice.maxExamCount > 0
                    && practice.maxExamCount <= practiceChatLogService.getExamCountByPractice(context.accountId, params.practiceId)) {
                throw BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2)
            }
        }

        // 新建一条陪练
        var log = PracticeChatLog()
        log.id = idGenerator.generate()
        log.practiceId = params.practiceId
        log.accountId = context.accountId
        log.accountName = context.accountName
        log.accountFullName = context.accountFullName
        log.useType = params.useType
        val orgInfo = organizationClient.listByOrgIds(listOf(context.orgId))
        if (null != orgInfo && orgInfo.size > 0) {
            log.orgName = orgInfo[0].name
        }
        if (params.useType == PracticeModelStatusEnum.PRACTICE.status) {
            log.level = params.level
        }
        log.status = PracticeRunningStatusEnum.RUNNING.status
        log.passMinScore = practice.examPassMinScore
        log.givenPointType = practice.examGivenPointType
        log.siteId = context.siteId
        log.orgId = context.orgId
        log.companyId = context.companyId
        log.passMinScore = practice.examPassMinScore
        log = practiceChatLogRepository.save(log)
        val vo = StuPracticeLogVo()
        vo.useType = log.useType
        vo.practiceLogId = log.id
        vo.practiceId = log.practiceId
        return vo
    }

    private fun buildAliOSSUrl(po: AnalyzeStudentVoicePo) {
        if (!StringUtils.isEmpty(po.voiceUrl)) {
            return
        }
        if (po.platform == "H5") {
            return
        }
        val mediaId = po.wechatMediaId
        val accessToken = weChatTokenClient.getWechatAccessTokenByAppId(po.wechatAppId,po.agentId)
        if (accessToken == null || mediaId == null) {
            logger.error("mediaId$mediaId accessToken$accessToken wechatAppId:${po.wechatAppId} agentId:${po.agentId}")
            throw BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2)
        }
        var wechatMediaUrl = "https://api.weixin.qq.com/cgi-bin/media/get?access_token=$accessToken&media_id=$mediaId"
        if (po.platform == "QYWECHAT") {
            wechatMediaUrl = "https://qyapi.weixin.qq.com/cgi-bin/media/get?access_token=$accessToken&media_id=$mediaId"
        }
        logger.info("wechatMediaUrl $wechatMediaUrl")
        val fileName = RandomUtil.randomString(10)
        val file = File(AudioUtils.dir, "$fileName.amr")
        val target = File(AudioUtils.dir, "$fileName.wav")
        logger.info("file url $wechatMediaUrl")
        FileUtils.copyURLToFile(URL(wechatMediaUrl), file)
        logger.info("source file size :${file.totalSpace}")
        val audio = AudioAttributes()
        audio.setCodec("libmp3lame")
        val attrs = EncodingAttributes()
        attrs.format = "wav"
        attrs.audioAttributes = audio
        val encoder = Encoder()
        try {
            val multimediaObject = MultimediaObject(file)
            encoder.encode(multimediaObject, target, attrs)
        } catch (e: Exception) {
            logger.error("文件转换异常  amr>>>>>>>>>wav",e);
        }
        logger.info("target file size :${target.totalSpace}")
        val multimediaObject = MultimediaObject(target)
        po.voiceDuration = multimediaObject.info.duration
        val inputStream = target.inputStream()
        val context = po.requestContext!!
        val path = OssFileUtil.getManageStandardVoiceUri(
                context.companyId,
                context.siteId,
                "PRACTICE",
                "SUDENT/UPLOAD",
                context.accountId,
                RandomUtil.randomLong())
        po.voiceUrl = aliyunUtil.uploadByStream(path, inputStream)
        inputStream.close()
        file.deleteOnExit()
        target.deleteOnExit()
    }
}
