package com.yizhi.practice.application.xservice.impl

import com.alibaba.fastjson.JSONObject
import com.yizhi.core.application.cache.RedisCache
import com.yizhi.core.application.context.RequestContext
import com.yizhi.core.application.enums.InternationalEnums
import com.yizhi.core.application.exception.BizException
import com.yizhi.practice.application.config.PracticeLevelEnum
import com.yizhi.practice.application.config.PracticeRunningStatusEnum
import com.yizhi.practice.application.enums.PracticeModelStatusEnum
import com.yizhi.practice.application.pojo.vo.PracticeConfigVo
import com.yizhi.practice.application.pojo.vo.PracticeJoinedVo
import com.yizhi.practice.application.repository.PracticeConfigRepository
import com.yizhi.practice.application.service.IPracticeChatLogService
import com.yizhi.practice.application.service.IPracticeConfigService
import com.yizhi.practice.application.service.IPracticeVisibleService
import com.yizhi.practice.application.xservice.PracticeStudentXSerice
import com.yizhi.system.application.system.remote.OrganizationClient
import com.yizhi.util.application.domain.BizResponse
import com.yizhi.util.application.page.PageInfo
import org.apache.commons.lang.StringUtils
import org.jetbrains.annotations.TestOnly
import org.slf4j.LoggerFactory
import org.springframework.beans.BeanUtils
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service


/**
 * @Date 2020/10/26 3:19 下午
 * @Author lvjianhui
 **/
@Service
class PracticeStudentXSericeImpl : PracticeStudentXSerice {

    @Autowired
    lateinit var iPracticeConfigService: IPracticeConfigService

    @Autowired
    lateinit var iPracticeChatLogService: IPracticeChatLogService

    @Autowired
    lateinit var practiceConfigRepository: PracticeConfigRepository

    @Autowired
    lateinit var iPracticeVisibleService: IPracticeVisibleService

    @Autowired
    lateinit var organizationClient: OrganizationClient

    @Autowired
    lateinit var redisCache: RedisCache

    private val logger = LoggerFactory.getLogger(PracticeXserviceImpl::class.java)

    override fun getTabIndex(requestContext: RequestContext): Int {
        var index = 1
        val runningCount = getRunningCount(
                requestContext.accountId,
                requestContext.orgId,
                requestContext.siteId,
                requestContext.companyId
        )
        if (runningCount > 0) {
            return index
        }
        val unStartCount = getRunningList(
                requestContext.accountId,
                requestContext.orgId,
                requestContext.siteId,
                requestContext.companyId,
                PracticeRunningStatusEnum.NOT_STARTED.status,
                1,
                1
        )?.records?.size ?: 0
        if (unStartCount > 0) {
            return 0
        }
        val endCount = getRunningList(
                requestContext.accountId,
                requestContext.orgId,
                requestContext.siteId,
                requestContext.companyId,
                PracticeRunningStatusEnum.ENDED.status,
                1,
                1
        )?.records?.size ?: 0
        if (endCount > 0) {
            return 2
        }
        return 1

    }

    override fun getPracticeJoined(practiceId: Long, context: RequestContext): PracticeJoinedVo {
        val vo = PracticeJoinedVo()
        val map = iPracticeChatLogService.getPracticeLogByLevel(practiceId, context.accountId, context.companyId, context.siteId, context.orgId)
                ?: let { return vo }
        vo.primaryLevelCount = map.get(PracticeLevelEnum.PRIMARY_LEVEL.level) ?: 0
        vo.intermediateLevel = map.get(PracticeLevelEnum.INTERMEDIATE_LEVEL.level) ?: 0
        vo.seniorLevel = map.get(PracticeLevelEnum.SENIOR_LEVEL.level) ?: 0
        return vo
    }

    override fun getPracticeInfo(practiceId: Long, accountId: Long, orgId: Long, siteId: Long, companyId: Long): PracticeConfigVo {
        val vo = PracticeConfigVo()
        val config = practiceConfigRepository.getOne(practiceId)
        // TODO Could not copy property 'description'
        BeanUtils.copyProperties(config, vo)
        val map = iPracticeChatLogService.getExamAndPracticeCount(accountId, practiceId, orgId, siteId, companyId)
                ?: return vo
        vo.joinExamCount = map[PracticeModelStatusEnum.ASSESSMENT.status]
        vo.joinPracticeCount = map[PracticeModelStatusEnum.PRACTICE.status]
        vo.robotName = config.robotName
        return vo
    }

    override fun getRunningList(accountId: Long, orgId: Long, siteId: Long, companyId: Long, state: Int, pageIndex: Int, pageSize: Int): PageInfo<PracticeConfigVo>? {
        if (!PracticeRunningStatusEnum.getAllStatus().contains(state)) {
            throw BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2)
        }
        val allIds = getVisiblePracticeIds(accountId, orgId, siteId, companyId, PracticeRunningStatusEnum.getEnumByState(state))
        logger.info("")
        val page = PageInfo<PracticeConfigVo>()
        page.pageSize = pageSize
        page.pageNo = pageIndex
        if (allIds == null || allIds.isEmpty()) {
            return page
        }
        var pageNum = allIds.size / pageSize
        if (allIds.size % pageSize != 0) {
            pageNum++
        }
        page.pageTotal = pageNum
        var startNum = (pageIndex - 1) * pageSize
        var endNum = startNum + pageSize

        if (startNum >= allIds.size) {
            startNum = allIds.size
        }

        if (endNum > allIds.size) {
            endNum = allIds.size
        }

        val idsPage = allIds.subList(startNum, endNum)
        page.records = iPracticeConfigService.getConfigByIds(idsPage)
        return page
    }


    override fun getRunningCount(accountId: Long, orgId: Long, siteId: Long, companyId: Long): Int {
        val allIds: List<Long> = getVisiblePracticeIds(accountId, orgId, siteId, companyId, PracticeRunningStatusEnum.RUNNING)
                ?: return 0
        return allIds.size
    }


    /**
     * 获取 用户可见的指定状态的陪练IDS
     *
     * @param accountId
     * @param orgId
     * @param siteId
     * @param companyId
     * @param state
     * @return
     */
    private fun getVisiblePracticeIds(accountId: Long, orgId: Long, siteId: Long, companyId: Long, state: PracticeRunningStatusEnum): List<Long>? {
        val allIds = iPracticeConfigService.getRunningCount(orgId, siteId, companyId, null, state)
        logger.info("当前站点：{} 当前部门：{}可参加的陪练Id:{}", siteId, orgId, JSONObject.toJSON(allIds).toString())
        if (allIds == null || allIds.isEmpty()) {
            return null
        }
        // 查询改部门的父部门，父部门可见的话，子部门的用户也是可见的。
        val parentIdsAndOrgIdList = getParentOrgIdByChildOrgId(orgId)
        parentIdsAndOrgIdList.add(orgId)
        val visiableId = iPracticeVisibleService.getIdsByAccountId(accountId, parentIdsAndOrgIdList, siteId, companyId)
        val siteVisiableIds = iPracticeConfigService.getSiteAllConfig(siteId, companyId)
        visiableId.addAll(siteVisiableIds)
        logger.info("当前站点：{} 当前部门：{} 当前用户：{}可见的陪练Id:{}", siteId, orgId, accountId, JSONObject.toJSON(visiableId).toString())
        if (visiableId.isEmpty()) {
            return null
        }
        return allIds.intersect(visiableId).toList()
    }

    /**
     * 缓存父部门ID
     */
    private fun getParentOrgIdByChildOrgId(orgId: Long): MutableList<Long> {
//        val key = "practice_stu_org_parent_ids"
//        val orgStr = orgId.toString()
//        val orgIds = redisCache.hget(key, orgStr)
//        if (null != orgIds && !StringUtils.isEmpty(JSONObject.toJSONString(orgIds))) {
//            return JSONObject.parseArray(JSONObject.toJSONString(orgIds), Long::class.java)
//        }

        val parentIds = organizationClient.orgIdParentIds(orgId) ?: arrayListOf()
//        redisCache.hset(key, orgStr, JSONObject.toJSONString(parentIds))
        return parentIds
    }

    override fun getUserPracticeAuth(requestContext: RequestContext, practiceId: Long): BizResponse<Boolean> {

        val visiblePracticeIds = getVisiblePracticeIds(requestContext.accountId, requestContext.orgId, requestContext.siteId, requestContext.companyId, PracticeRunningStatusEnum.ALL)

        if (visiblePracticeIds != null && visiblePracticeIds.contains(practiceId)) {
            return BizResponse.ok(true)
        }
        return BizResponse.ok(false)
    }
}