package com.yizhi.practice.application.xservice.impl;

import com.alibaba.fastjson.JSONObject;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.enums.InternationalEnums;
import com.yizhi.core.application.exception.BizException;
import com.yizhi.practice.application.enums.PracticeStatusEnum;
import com.yizhi.practice.application.pojo.domain.PracticeConfig;
import com.yizhi.practice.application.pojo.domain.SceneConfig;
import com.yizhi.practice.application.pojo.vo.SceneVo;
import com.yizhi.practice.application.repository.PracticeConfigRepository;
import com.yizhi.practice.application.repository.SceneChatConfigRepository;
import com.yizhi.practice.application.repository.SceneConfigRepository;
import com.yizhi.practice.application.request.SceneReq;
import com.yizhi.practice.application.service.ISceneChatConfigService;
import com.yizhi.practice.application.service.ISceneConfigService;
import com.yizhi.practice.application.xservice.PracticeXSceneservice;
import com.yizhi.util.application.domain.BizResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

@Service
public class PracticeXSceneserviceImpl implements PracticeXSceneservice {

    private static Logger logger = LoggerFactory.getLogger(PracticeXSceneserviceImpl.class);

    @Autowired
    PracticeConfigRepository practiceConfigRepository;
    @Autowired
    IdGenerator idGenerator;
    @Autowired
    SceneConfigRepository sceneConfigRepository;
    @Autowired
    SceneChatConfigRepository chatConfigRepository;
    @Autowired
    ISceneConfigService sceneConfigService;
    @Autowired
    ISceneChatConfigService sceneChatConfigService;


    @Override
    @Transactional
    public BizResponse save(SceneReq sceneReq) {
        RequestContext context = ContextHolder.get();
        Long practiceId = sceneReq.getPracticeId();
        String newSceneName = sceneReq.getSceneName();
        String desc = sceneReq.getDesc();
        if (StringUtils.isEmpty(newSceneName) || newSceneName.length() > 10
                || (!StringUtils.isEmpty(desc) && desc.length() > 500)
        ){
            logger.error("[PracticeXSceneServiceImpl:  save]陪练场景，场景名称为空或长度大于10,或描述长度大于500! 请求入参: {},accountId={}"
                    , JSONObject.toJSONString(sceneReq),context.getAccountId());
            throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"场景名称不符合规范");
        }
        PracticeConfig practice = practiceConfigRepository.getOne(practiceId);
        if (null == practice){
            logger.error("[PracticeXSceneserviceImpl:  save]陪练场景，未查询到陪练信息! practiceId = {} ,场景名称= {}",practiceId,sceneReq.getSceneName());
            throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"未查询到陪练");
        }

        SceneConfig scene = new SceneConfig();
        Long sceneId = sceneReq.getSceneId();
        if (null != sceneId){
            scene = sceneConfigRepository.getOne(sceneId);
            String oldSceneName = scene.getName();
            if (!StringUtils.isEmpty(oldSceneName) && !oldSceneName.equals(newSceneName)){
                //场景名称修改,同步修改场景对话中名称；
                chatConfigRepository.updateSceneNameBySceneId(newSceneName,sceneId);
            }
            scene.setUpdatedUid(ContextHolder.get().getAccountId());
        }else {
            scene.setId(idGenerator.generate());
            scene.setPracticeId(practiceId);
            setCreatedAccountInfo(scene);
        }

        scene.setName(sceneReq.getSceneName());
        scene.setDescription(sceneReq.getDesc());
        scene.setSort(sceneReq.getSort());
        sceneConfigRepository.save(scene);
        return BizResponse.ok();
    }

    @Override
    public SceneVo selectScene(Long sceneId) {
        SceneConfig scene = sceneConfigRepository.getOne(sceneId);
        SceneVo sceneVo = new SceneVo();
        if (null != scene){
            sceneVo.setId(scene.getId());
            sceneVo.setName(scene.getName());
            sceneVo.setDescription(scene.getDescription());
            sceneVo.setPracticeId(scene.getPracticeId());
        }
        return sceneVo;
    }

    @Override
    @Transactional
    public BizResponse deleteScene(Long sceneId) {
        RequestContext context = ContextHolder.get();
        SceneConfig scene = sceneConfigRepository.getOne(sceneId);
        if (null == scene || null == scene.getId()){
            logger.error("[PracticeXSceneServiceImpl:deleteScene]删除场景失败。为查询到场景 sceneId = {}. accountId = {}",sceneId,context.getAccountId());
            throw new BizException(InternationalEnums.CLASSIFYCONTROLLER4);
        }
        Long practiceId = scene.getPracticeId();
        if (null == practiceId || practiceId.longValue() == 0){
            logger.error("智能陪练删除场景[PracticeXServiceImpl:deleteScene] 场景陪练id为空.sceneId={}",sceneId);
            throw new BizException(InternationalEnums.CLASSIFYCONTROLLER4);
        }
        PracticeConfig practice = practiceConfigRepository.getOne(practiceId);
        Integer status = practice.getStatus();
        if (PracticeStatusEnum.UP.getStatus().equals(status)){
            logger.error("智能陪练删除场景失败。上架的陪练不能删除场景；sceneId = {}",sceneId);
            throw new BizException(InternationalEnums.CLASSIFYCONTROLLER4);
        }
        //删除场景
        long delCount = sceneConfigService.deleteScene(practiceId, sceneId);
        if (-1 == delCount){
            logger.error("[PracticeXSceneServiceImpl:deleteScene]删除场景失败。删除场景传入参数都为空.sceneId = {}. accountId = {}",sceneId,context.getAccountId());
            throw new BizException(InternationalEnums.CLASSIFYCONTROLLER4);
        }
        //删除场景对话
        sceneChatConfigService.deleteChatNodes(scene.getPracticeId(),sceneId,null,null,null);
        return BizResponse.ok();
    }

    @Override
    public List<SceneVo> selectSceneList(Long practiceId) {
        List<SceneVo> sceneVoList = new ArrayList<>();
        List<SceneConfig> sceneConfigList = sceneConfigService.findSceneByPracticeId(practiceId);
        if (null != sceneConfigList && sceneConfigList.size() > 0){
            for (SceneConfig sceneConfig : sceneConfigList) {
                SceneVo sceneVo = new SceneVo();
                sceneVo.setId(sceneConfig.getId());
                sceneVo.setPracticeId(sceneConfig.getPracticeId());
                sceneVo.setDescription(sceneConfig.getDescription());
                sceneVo.setName(sceneConfig.getName());
                sceneVo.setSort(sceneConfig.getSort());
                sceneVoList.add(sceneVo);
            }
        }
        return sceneVoList;
    }

    @Override
    @Transactional
    public BizResponse updateSceneSort(List<SceneReq> sceneReqList) {

        if (null != sceneReqList && sceneReqList.size() > 0){
            for (SceneReq sceneReq : sceneReqList) {
                Long sceneId = sceneReq.getSceneId();
                int sort = sceneReq.getSort();
                if (null != sceneId){
                    sceneConfigService.updateSceneSort(sceneId,sort);
                }
            }
        }
        return BizResponse.ok();
    }

    /**设置数据更新人信息*/
    private void setCreatedAccountInfo(SceneConfig scene) {
        RequestContext requestContext = ContextHolder.get();
        scene.setCreatedUid(requestContext.getAccountId());
        scene.setSiteId(requestContext.getSiteId());
        scene.setCompanyId(requestContext.getCompanyId());
        scene.setOrgId(requestContext.getOrgId());
    }

}
