package com.yizhi.practice.application.xservice.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.enums.InternationalEnums;
import com.yizhi.core.application.exception.BizException;
import com.yizhi.core.application.file.domain.Template;
import com.yizhi.core.application.file.domain.TemplateColumn;
import com.yizhi.core.application.log.LogEventPublisher;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.practice.application.config.AudioUtils;
import com.yizhi.practice.application.constants.PracticeConstant;
import com.yizhi.practice.application.enums.*;
import com.yizhi.practice.application.excel.*;
import com.yizhi.practice.application.pojo.domain.*;
import com.yizhi.practice.application.pojo.dto.PracticeBaseInfoDto;
import com.yizhi.practice.application.pojo.dto.PracticeStrategyInfoDto;
import com.yizhi.practice.application.pojo.vo.*;
import com.yizhi.practice.application.repository.*;
import com.yizhi.practice.application.request.*;
import com.yizhi.practice.application.service.*;
import com.yizhi.practice.application.third.util.OssFileUtil;
import com.yizhi.practice.application.third.xunfei.XunfeiUtil;
import com.yizhi.practice.application.util.NumberUtil;
import com.yizhi.practice.application.xservice.PracticeXservice;
import com.yizhi.util.application.domain.BizResponse;
import com.yizhi.util.application.page.PageInfo;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.lang.reflect.InvocationTargetException;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @Date 2020/10/22 1:44 下午
 * @Author lvjianhui
 **/
@Service
public class PracticeXserviceImpl implements PracticeXservice {

    //关键词正则 : 只能是中文/数字/点(.)/%
    private String keywordRegex = "[0-9\u4e00-\u9fa5.%]+";
    //机器人和学员标准话术正则：包含英文
    private String contentRegex = ".*[a-zA-Z].*";
    //拓展词、禁忌词正则: 只能包含中文和英文分号、数字
    private String commentRegex = "[\u4e00-\u9fa5;0-9]+";

    @Autowired
    IPracticeConfigService iPracticeConfigService;
    @Autowired
    PracticeConfigRepository practiceConfigRepository;
    @Autowired
    IdGenerator idGenerator;
    @Autowired
    IPracticeVisibleService iPracticeVisibleService;
    @Autowired
    PracticePointRuleConfigRepository pointRuleConfigRepository;
    @Autowired
    IPracticePointRuleConfigService pointRuleConfigService;
    @Autowired
    PracticeVisibleRepository visibleRepository;
    @Autowired
    SceneChatConfigRepository chatConfigRepository;
    @Autowired
    XunfeiUtil xunfeiUtil;
    @Autowired
    RobotConfigRepository robotConfigRepository;
    @Autowired
    SceneConfigRepository sceneConfigRepository;
    @Autowired
    ISceneChatConfigService chatConfigService;
    @Autowired
    ISceneConfigService sceneConfigService;
    @Autowired
    LogEventPublisher publisher;
    @Autowired
    TaskExecutor taskExecutor;
    @Autowired
    PracticeAccountImport accountImport;
    @Autowired
    PracticeAccountExport accountExport;
    @Autowired
    PracticeExcelImporter excelImporter;




    private Logger logger = LoggerFactory.getLogger(PracticeXserviceImpl.class);

    /**
     * 初始化空白草稿状态的陪练
     *
     * @return 陪练信息
     */
    @Override
    @Transactional(propagation = Propagation.REQUIRED)
    public PracticeBlankDraftVo initBlankDraftPractice() {
        PracticeConfig practiceConfig = new PracticeConfig();
        Long practiceId = idGenerator.generate();
        practiceConfig.setId(practiceId);
        practiceConfig.setStatus(PracticeStatusEnum.DRAFT.getStatus());
        setCreatedAccountInfo(practiceConfig);
        practiceConfig = practiceConfigRepository.save(practiceConfig);
        PracticeBlankDraftVo baseInfoVo = new PracticeBlankDraftVo();
        baseInfoVo.setId(practiceConfig.getId());
        baseInfoVo.setStatus(practiceConfig.getStatus());
        return baseInfoVo;
    }


    /**
     * 保存智能陪练配置信息
     */
    @Override
    @Transactional
    public BizResponse savePracticeConfInfo(PracticeConfigInfoReq practiceReq) {

        Long practiceId = practiceReq.getId();
        PracticeConfig practice = practiceConfigRepository.getOne(practiceId);
        if (null == practice) {
            logger.error("根据陪练id未查询到陪练信息! practiceId = {}", practiceId);
            throw new BizException(InternationalEnums.MATERIALCONTROLLER1);
        }
        PracticeBaseInfoDto baseInfoDto = new PracticeBaseInfoDto();
        BeanUtils.copyProperties(practiceReq, baseInfoDto);
        checkPracticeBaseParamsValid(baseInfoDto);
        BeanUtils.copyProperties(baseInfoDto, practice);
        String logoUrl = baseInfoDto.getLogoUrl();
        if (StringUtils.isEmpty(logoUrl)){
            String defaultUrl = PracticeConstant.PRACTICE_DEFAULT_LOGO_URL;
            practice.setLogoUrl(defaultUrl);
        }
        practice.setId(practiceId);
        practice.setRobotId(baseInfoDto.getRobotId());
        practice.setRobotName(baseInfoDto.getRobotAliasName());
        setUpdatedAccountInfo(practice);
        practiceConfigRepository.save(practice);
        return BizResponse.ok();
    }

    @Override
    @Transactional
    public BizResponse savePracticeStrategyInfo(PracticeConfigInfoReq practiceConfigInfoReq) {

        PracticeStrategyInfoDto strategyInfoDto = new PracticeStrategyInfoDto();
        BeanUtils.copyProperties(practiceConfigInfoReq, strategyInfoDto);
        Long practiceId = strategyInfoDto.getId();
        PracticeConfig practice = practiceConfigRepository.getOne(practiceId);
        if (null == practice) {
            logger.error("根据陪练id未查询到陪练信息! practiceId = {}", practiceId);
            return BizResponse.fail("新增陪练策略信息失败");
        }
        strategyInfoDto.setUseType(practice.getUseType());
        checkPracticeStrategyParamsValid(strategyInfoDto);
        BeanUtils.copyProperties(strategyInfoDto, practice);
        setUpdatedAccountInfo(practice);
        //未识别回复类型 80:默认回复，10:自定义回复
        Integer similarityFailedMsgType = strategyInfoDto.getSimilarityFailedMsgType();
        if (PracticeManyRoudChatEnum.TRUE.equals(similarityFailedMsgType)){
            strategyInfoDto.setSimilarityFailedMsg(PracticeConstant.EXAM_FAIL_MESSAGE);
            practice.setSimilarityFailedMsg(PracticeConstant.EXAM_FAIL_MESSAGE);
        }
        //可见范围修改
        Integer oldVisibleType = practice.getVisibleType();
        Integer newVisibleType = strategyInfoDto.getVisibleType();
        //由part->all 删除关系表原来所有可见范围；其他改变均由‘指定用户’接口做修改
        if (PracticeVisibleTypeEnum.PART.getStatus().equals(oldVisibleType) && PracticeVisibleTypeEnum.ALL.getStatus().equals(newVisibleType)){
            visibleRepository.deleteVisibleRangeByPractice(practiceId);
        }
        // 积分规则修改
        Integer newExamGivenPointType = strategyInfoDto.getExamGivenPointType();
        List<PracticePointVo> practicePointList = practiceConfigInfoReq.getPracticePointList();
        saveOrUpdatePracticePointRule(newExamGivenPointType, practiceId, practicePointList);
        practiceConfigRepository.save(practice);
        return BizResponse.ok();
    }

    @Override
    @Transactional
    public BizResponse savePracticeDialogue(List<PracticeSceneDialogueReq> sceneNodeList) {

        RequestContext context = ContextHolder.get();
        for (PracticeSceneDialogueReq dialogueReq : sceneNodeList){
            Long practiceId = dialogueReq.getPracticeId();
            Long sceneId = dialogueReq.getSceneId();
            if (null == practiceId || null == sceneId ){
                logger.error("保存对话传入id为空; practiceId = {} ,sceneId = {} ,accountId = {}",practiceId,sceneId,context.getAccountId());
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
            }
            List<PracticeChatNodeReq> chatNodes = dialogueReq.getChatNodes();
            if (null == chatNodes || chatNodes.size() == 0){
                logger.error("保存对话传入对话内容为空; practiceId = {} ,sceneId = {} ,accountId = {}",practiceId,sceneId,context.getAccountId());
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
            }
            //校验对话节点是否符合‘开始-机器人-学员-机器人-学员-...’结构要求
            checkChatNodeOrderValid(chatNodes);
            //校验对话节点参数
            checkChatNodeParams(chatNodes);
            PracticeConfig practice = practiceConfigRepository.getOne(practiceId);
            //保存场景对话节点信息.将原有的节点都逻辑删除,然后再进行保存和更新操作
            chatConfigRepository.deleteSceneNodesByPracticeId(sceneId);
            saveOrUpdatePracticeDialogue(practice,sceneId,0L,chatNodes);
        }
        return BizResponse.ok();
    }

    /**
     * 校验对话节点参数是否符合要求
     * @param chatNodeList 节点对话数据
     */
    private void checkChatNodeParams(List<PracticeChatNodeReq> chatNodeList) {
        if (null == chatNodeList || chatNodeList.size() == 0){
            return;
        }
        for (PracticeChatNodeReq chatNode : chatNodeList) {
            Integer sceneNodeType = chatNode.getSceneNodeType();
            if (PracticeNodeEnum.STUDENT.getStatus().equals(sceneNodeType)){
                String accountContent = chatNode.getAccountContent();
                List<KeyWordsReq> keyWordList = chatNode.getKeyWordList();
                if (StringUtils.isEmpty(accountContent) || accountContent.length() > 250){
                    logger.error("[PracticeXServiceImpl:checkChatNodeParams]学员节点话术为空或长度超出限制,accountContent = {}",accountContent);
                    throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"话术配置有误，请重新编辑");
                }
                /*if (checkRegex(accountContent,contentRegex)){
                    //学员话术包含英文
                    logger.error("[PracticeXServiceImpl:checkChatNodeParams]学员节点话术包含英文,accountContent = {}",accountContent);
                    throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"话术配置有误，请重新编辑");
                }*/
                //关键词
                if (null == keyWordList || keyWordList.isEmpty()){
                    logger.error("[PracticeXServiceImpl:checkChatNodeParams]学员节点话术关键字为空,accountContent = {}",accountContent);
                    throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"话术配置有误，请重新编辑");
                }
                for (KeyWordsReq keyWordsReq : keyWordList) {
                    String key = keyWordsReq.getKey();
                    String extendWord = keyWordsReq.getValue();
                    //校验关键词和扩展词
                    boolean result = checkKeywordAndExtend(key, extendWord, new HashSet<>(), keywordRegex, commentRegex, 0, accountContent);
                    if (!result){
                        logger.error("[PracticeXServiceImpl:checkChatNodeParams]学员节点话术关键字/扩展词校验结果result={},accountContent = {},keywordList = {}",result,accountContent,JSONObject.toJSONString(keyWordList));
                        throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"话术配置有误，请重新编辑");
                    }
                }
                //禁忌词
                String accountContentTabooWord = chatNode.getAccountContentTabooWord();
                if (!StringUtils.isEmpty(accountContentTabooWord)){
                    if (!checkRegex(accountContentTabooWord,commentRegex)){
                        logger.error("[PracticeXServiceImpl:checkChatNodeParams]学员节点话术禁忌词包含中文英文分号以外字符,accountContent = {},tabooWord = {}",accountContent,accountContentTabooWord);
                        throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"话术配置有误，请重新编辑");
                    }
                }
            }else if (PracticeNodeEnum.ROBOT.getStatus().equals(sceneNodeType)){
                String robotContent = chatNode.getRobotContent();
                if (StringUtils.isEmpty(robotContent) || robotContent.length() > 250){
                    logger.error("[PracticeXServiceImpl:checkChatNodeParams]机器人节点话术为空或长度超出限制,robotContent = {}",robotContent);
                    throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"话术配置有误，请重新编辑");
                }
               /* if (checkRegex(robotContent,contentRegex)){
                    //学员话术包含英文
                    logger.error("[PracticeXServiceImpl:checkChatNodeParams]机器人节点话术包含中文,robotContent = {}",robotContent);
                    throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"话术配置有误，请重新编辑");
                }*/
            }
            List<PracticeChatNodeReq> childNodes = chatNode.getChildNodes();
            checkChatNodeParams(childNodes);
        }
    }

    /**
     * 校验对话节点顺序是否符合规则
     * @param chatNodeList 前端传入节点集合,包括开始节点；
     *  机器人节点下只能是学员；学员下只能是机器人；开始节点下只能是机器人
     */
    private void checkChatNodeOrderValid(List<PracticeChatNodeReq> chatNodeList) {

        if (null == chatNodeList || chatNodeList.size() == 0){
            return;
        }
        for (PracticeChatNodeReq chatNode : chatNodeList) {

            Long nodeId = chatNode.getId();
            Integer level = chatNode.getLevel();
            Integer sceneNodeType = chatNode.getSceneNodeType();
            List<PracticeChatNodeReq> startChildNodes = chatNode.getChildNodes();
            if (null == startChildNodes || startChildNodes.size() == 0){
                continue;
            }
            if (!PracticeNodeEnum.START.getStatus().equals(sceneNodeType) && !PracticeNodeEnum.STUDENT.getStatus().equals(sceneNodeType)
                    && !PracticeNodeEnum.ROBOT.getStatus().equals(sceneNodeType)){
                logger.error("[practiceXServiceImpl:checkChatNodeOrderValid]校验对话节点，没有查询到节点类型。chatId = {},chatNodeType = {},chatNodes = {}",nodeId,sceneNodeType,JSONObject.toJSONString(chatNode));
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"节点类型错误");
            }
            if (PracticeNodeEnum.START.getStatus().equals(sceneNodeType) || PracticeNodeEnum.STUDENT.getStatus().equals(sceneNodeType)){
                for (PracticeChatNodeReq startChildNode : startChildNodes) {
                    Integer childNodeType = startChildNode.getSceneNodeType();
                    if (!PracticeNodeEnum.ROBOT.getStatus().equals(childNodeType)){
                        logger.error("[practiceXServiceImpl:checkChatNodeOrderValid]校验对话节点顺序，开始节点子节点类型不是机器人，开始子节点列表chatNodes = {}",JSONObject.toJSONString(startChildNodes));
                        throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"开始节点下必须是机器人节点");
                    }
                }
            }else if (PracticeNodeEnum.ROBOT.getStatus().equals(sceneNodeType)){
                for (PracticeChatNodeReq startChildNode : startChildNodes) {
                    Integer childNodeType = startChildNode.getSceneNodeType();
                    if (!PracticeNodeEnum.STUDENT.getStatus().equals(childNodeType)){
                        logger.error("[practiceXServiceImpl:checkChatNodeOrderValid]校验对话节点顺序，机器人节点子节点类型不是学员，机器人子节点列表chatNodes = {}",JSONObject.toJSONString(startChildNodes));
                        throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"机器人节点下只能连接学员节点");
                    }
                }
            }
        }
    }

    @Override
    public List<RobotVo> getRobotList() {
        List<RobotVo> list = new ArrayList<>();
        List<RobotConfig> all = robotConfigRepository.findAll();
        if (null != all && all.size() > 0){
            all.forEach(robotConfig -> {
                RobotVo robotVo = new RobotVo();
                robotVo.setId(robotConfig.getId());
                robotVo.setRobotName(robotConfig.getName());
                robotVo.setRobotParams(robotConfig.getParams());
                robotVo.setContentVoiceUrl(robotConfig.getContentVoiceUrl());
                list.add(robotVo);
            });
        }
        return list;
    }

    @Override
    public BizResponse<PageInfo<PracticeBaseInfoVo>> getPracticeList(PracticesGetReq practicesGetReq) {

        List<Long> ids = new ArrayList<>();
        RequestContext context = ContextHolder.get();
        Long accountId = context.getAccountId();
        ids.add(accountId);
        
        for(Long id : context.getManagerIds()) {
        	ids.add(id);
        }
        practicesGetReq.setCompanyId(context.getCompanyId());
        practicesGetReq.setSiteId(context.getSiteId());
        // 不是超级管理员时，增加 分级授权 条件
        if(!context.isAdmin()) {
        	practicesGetReq.setIds(ids);
        }
        Long total =  iPracticeConfigService.getPracticeConfigCount(practicesGetReq);
        List<PracticeBaseInfoVo> list = iPracticeConfigService.getPracticeConfigList(practicesGetReq);
        PageInfo<PracticeBaseInfoVo> pageInfo = new PageInfo<>();
        pageInfo.setRecords(list);
        pageInfo.setPageTotal(NumberUtil.getPageCount(total, practicesGetReq.getPageSize()));
        pageInfo.setPageNo(practicesGetReq.getPageNo());
        pageInfo.setPageSize(practicesGetReq.getPageSize());
        pageInfo.setPageRecords(total.intValue());
        return BizResponse.ok(pageInfo);
    }

    @Override
    public PracticeConfigInfoVo getPracticeConfigInfo(Long practiceId) {
        PracticeConfig practice = practiceConfigRepository.getOne(practiceId);
        PracticeConfigInfoVo vo = new PracticeConfigInfoVo();
        BeanUtils.copyProperties(practice,vo);
        Integer examGivenPointType = practice.getExamGivenPointType();
        List<PracticePointVo> practicePointList = new ArrayList<>();
        if (null != examGivenPointType && PracticePointGivenEnum.TRUE.getStatus().equals(examGivenPointType)){
            //发放积分；查询积分数据
           List<PracticePointRuleConfig> pointRuleList = pointRuleConfigRepository.selectListByPracticeId(practiceId);
//           if (null == pointRuleList || pointRuleList.isEmpty()) {
//               return
//           }
           if (null != pointRuleList  && pointRuleList.size() > 0){
               pointRuleList.forEach(practicePointRuleConfig -> {
                   PracticePointVo pointVo = new PracticePointVo();
                   pointVo.setId(practicePointRuleConfig.getId());
                   pointVo.setMaxScore(practicePointRuleConfig.getMaxScore());
                   pointVo.setMinScore(practicePointRuleConfig.getMinScore());
                   pointVo.setPoint(practicePointRuleConfig.getPoint());
                   practicePointList.add(pointVo);
               });
           }
        }
        vo.setPracticePointList(practicePointList);
        //设置机器人信息
        Long robotId = practice.getRobotId();
        RobotConfig robot = robotConfigRepository.getOne(robotId);
        vo.setRobotId(robotId);
        vo.setRobotName(robot.getName());
        vo.setRobotParams(practice.getRobotParams());
        vo.setRobotAliasName(practice.getRobotName());
        vo.setRobotVoiceUrl(robot.getContentVoiceUrl());
        return vo;
    }

    @Override
    public List<PracticeSceneDialogueVo> getPracticeSceneChatList(Long practiceId,Long practiceSceneId) throws JsonProcessingException {

        List<PracticeSceneDialogueVo> sceneDialogueVoList = new ArrayList<>();
        List<SceneConfig> sceneConfigs = new ArrayList<>();
        if (null != practiceSceneId){
            SceneConfig scene = sceneConfigRepository.getOne(practiceSceneId);
            sceneConfigs.add(scene);
        }else {
            sceneConfigs = sceneConfigRepository.selectByPracticeId(practiceId);
        }
        if (null != sceneConfigs && sceneConfigs.size() > 0){
            for (SceneConfig scene : sceneConfigs){
                Long sceneId = scene.getId();
                PracticeSceneDialogueVo sceneDialogueVo = new PracticeSceneDialogueVo();
                sceneDialogueVo.setPracticeId(practiceId);
                sceneDialogueVo.setSceneId(sceneId);
                sceneDialogueVo.setSceneName(scene.getName());
                sceneDialogueVo.setSort(scene.getSort());
                sceneDialogueVo.setSceneDescription(scene.getDescription());
                //查询场景下开始及以下节点列表
                List<SceneChatConfig> startRobotNodes = chatConfigService.selectChatNodes(practiceId, sceneId, PracticeNodeEnum.START.getStatus(), 0L, 0);
                //遍历开始下节点，递归查询字节
                //将节点信息设置到vo中 递归获取节点和字节点
                List<PracticeChatNodeReq> chatNodeReqList = new ArrayList<>();
                if (null != startRobotNodes && startRobotNodes.size() > 0){
                    for (SceneChatConfig startChatNode : startRobotNodes){
                        PracticeChatNodeReq chatNodeReq = new PracticeChatNodeReq();
                        getSceneChatChildNodes(startChatNode,chatNodeReq);
                        chatNodeReqList.add(chatNodeReq);
                    }
                }
                sceneDialogueVo.setChatNodes(chatNodeReqList);
                sceneDialogueVoList.add(sceneDialogueVo);
            }
        }
        return sceneDialogueVoList;
    }

    @Override
    //@Transactional
    public BizResponse updatePracticeStatus(Long practiceId, Integer status) throws Exception {
        PracticeConfig practice = practiceConfigRepository.getOne(practiceId);
        RequestContext context = ContextHolder.get();
        if (null == practice){
            logger.error("[PracticeXserviceImpl:updatePracticeStatus]上下架失败!没有查询到陪练practiceId= {} ,status = {} ,accountId= {}",practiceId,status,context.getAccountId());
            return BizResponse.fail("操作失败!");
        }
        if (!PracticeStatusEnum.DOWN.getStatus().equals(status)  && !PracticeStatusEnum.UP.getStatus().equals(status) ){
            logger.error("[PracticeXserviceImpl:updatePracticeStatus]上下架失败!错误的陪练状态!practiceId= {} ,status = {} ,accountId= {}",practiceId,status,context.getAccountId());
            return BizResponse.fail("操作失败!");
        }
        if (PracticeStatusEnum.DOWN.getStatus().equals(status) ){
            //下架直接设置
            practice.setStatus(status);
            practiceConfigRepository.save(practice);
            return BizResponse.ok();
        }
        //上架需要校验参数
        PracticeBaseInfoDto practiceBaseInfoDto = new PracticeBaseInfoDto();
        BeanUtils.copyProperties(practice,practiceBaseInfoDto);
        practiceBaseInfoDto.setRobotAliasName(practice.getRobotName());
        checkPracticeBaseParamsValid(practiceBaseInfoDto);
        //校验场景和对话
        List<SceneConfig> sceneConfigList = sceneConfigService.findSceneByPracticeId(practiceId);
        if (null == sceneConfigList || sceneConfigList.size() == 0){
            logger.error("[PracticeXserviceImpl:updatePracticeStatus]上架失败!该陪练下没有场景不能上架!practiceId= {} ,status = {} ,accountId= {}",practiceId,status,context.getAccountId());
            return BizResponse.fail("上架失败");
        }
        for (SceneConfig sceneConfig : sceneConfigList) {
            Long sceneId = sceneConfig.getId();
            //校验场景下对话项
            checkSceneChatValid(practiceId,sceneId);
        }
        PracticeStrategyInfoDto strategyInfoDto = new PracticeStrategyInfoDto();
        BeanUtils.copyProperties(practice,strategyInfoDto);
        //获取积分
        List<PracticePointRuleConfig> pointList = pointRuleConfigService.selectPointRulesByPractice(practiceId, false);
        List<PracticePointVo> pointVoList = new ArrayList<>();
        if (null != pointList && !pointList.isEmpty()){
            pointList.forEach(point -> {
                PracticePointVo pointVo = new PracticePointVo();
                pointVo.setPoint(point.getPoint());
                pointVo.setMinScore(point.getMinScore());
                pointVo.setMaxScore(point.getMaxScore());
                pointVo.setId(point.getId());
                pointVoList.add(pointVo);
            });
        }
        strategyInfoDto.setPracticePointList(pointVoList);
        //校验策略
        checkPracticeStrategyParamsValid(strategyInfoDto);
        //语音合成 查询陪练下所有场景对话标准话术未合成的节点
        List<SceneChatConfig> list = chatConfigService.selectChatVoiceIsEmpty(practiceId);
        if (null == list || list.isEmpty()){
            practice.setStatus(status);
            practice.setUpdatedUid(context.getAccountId());
            practiceConfigRepository.save(practice);
            return BizResponse.ok();
        }
        //获取需要更新的语音合成对话
        getChatNodeVoiceUps(practice, context, list);
        //再次验证语音合成
        for (SceneConfig sceneConfig : sceneConfigList) {
            Long sceneId = sceneConfig.getId();
            List<SceneChatConfig> sceneChatConfigs = chatConfigService.selectChatNodes(practiceId, sceneId, null, null, null);
            checkVoice(sceneChatConfigs);
        }
        practice.setStatus(status);
        practiceConfigRepository.save(practice);
        return BizResponse.ok();
    }

    /**
     * 获取需要更新的语音合成对话
     * @param practice 陪练
     * @param context 上下文信息
     * @param list 需要语音合成的节点列表
     */
    @NotNull
    private List<SceneChatConfig> getChatNodeVoiceUps(PracticeConfig practice, RequestContext context, List<SceneChatConfig> list) throws Exception {
        List<SceneChatConfig> updateChatList = new ArrayList<>();

        list.parallelStream().forEach(sceneChatConfig -> {
            Integer sceneNodeType = sceneChatConfig.getSceneNodeType();
            if (PracticeNodeEnum.STUDENT.getStatus().equals(sceneNodeType)){
                String accountContentVoiceUrl = sceneChatConfig.getAccountContentVoiceUrl();
                String accountContent = sceneChatConfig.getAccountContent();
                if (StringUtils.isEmpty(accountContent)){
                    logger.error("智能陪练上架失败,学员标准话术文本为空.accountId= {},siteId={},chatId= {},practiceId = {}"
                            ,context.getAccountId(),context.getSiteId(),sceneChatConfig.getId(),practice.getId());
                    throw new BizException(InternationalEnums.DOCUMENTRELATIONCONTROLLER6);
                }
                if (StringUtils.isEmpty(accountContentVoiceUrl)){
                    String accountVoiceUrl = null;
                    try {
                        accountVoiceUrl = upFileToOss(practice, accountContent,context);
                    } catch (Exception e) {
                        logger.error("智能陪练语音合成异常",e);
                        throw new BizException(InternationalEnums.DOCUMENTRELATIONCONTROLLER6);
                    }
                    sceneChatConfig.setAccountContentVoiceUrl(accountVoiceUrl);
                    long audioDuration = AudioUtils.INSTANCE.getAudioDuration(accountVoiceUrl);
                    sceneChatConfig.setAccountContentVoiceDuration(audioDuration);
                    sceneChatConfig.setUpdatedUid(context.getAccountId());
                    //updateChatList.add(sceneChatConfig);
                }
            }else if (PracticeNodeEnum.ROBOT.getStatus().equals(sceneNodeType)){
                String robotContentVoiceUrl = sceneChatConfig.getRobotContentVoiceUrl();
                String robotContent = sceneChatConfig.getRobotContent();
                if (StringUtils.isEmpty(robotContent)){
                    logger.error("智能陪练上架失败,机器人标准话术文本为空.accountId= {},siteId={},chatId= {},practiceId = {}"
                            ,context.getAccountId(),context.getSiteId(),sceneChatConfig.getId(),practice.getId());
                    throw new BizException(InternationalEnums.DOCUMENTRELATIONCONTROLLER6);
                }
                if (StringUtils.isEmpty(robotContentVoiceUrl)){
                    String robotVoiceUrl = null;
                    try {
                        robotVoiceUrl = upFileToOss(practice, robotContent,context);
                    } catch (Exception e) {
                        logger.error("智能陪练语音合成异常",e);
                        throw new BizException(InternationalEnums.DOCUMENTRELATIONCONTROLLER6);
                    }
                    sceneChatConfig.setRobotContentVoiceUrl(robotVoiceUrl);
                    long audioDuration = AudioUtils.INSTANCE.getAudioDuration(robotVoiceUrl);
                    sceneChatConfig.setRobotContentVoiceDuration(audioDuration);
                    sceneChatConfig.setUpdatedUid(context.getAccountId());
                    //updateChatList.add(sceneChatConfig);
                }
            }
            chatConfigRepository.save(sceneChatConfig);
        });
        return updateChatList;
    }

    @Override
    @Transactional
    public BizResponse addPracticeAccount(PracticeAccountReq practiceAccountReq) {
        RequestContext context = ContextHolder.get();
        Long practiceId = practiceAccountReq.getPracticeId();
        List<PracticeVisibleAccount> accountList = practiceAccountReq.getAccountList();
        if (null == accountList || null == practiceId){
            logger.error("[PracticeXServiceImpl:addPracticeAccount] 保存陪练用户部门信息包含null;accountId = {} ,practiceId = {}",context.getAccountId(),practiceId);
            throw new BizException(InternationalEnums.MATERIALCONTROLLER1);
        }
        //逻辑删除所有用户
        iPracticeVisibleService.deleteByPracticeId(practiceId);
        if (accountList.size() == 0){
            return BizResponse.ok();
        }
        //否则遍历保存或者修改
        for (PracticeVisibleAccount visibleAccount : accountList) {
            //全部新增
            PracticeVisible practiceVisible = new PracticeVisible();
            practiceVisible.setId(idGenerator.generate());
            Integer visibleType = visibleAccount.getVisibleType();
            practiceVisible.setVisibleType(visibleType);
            practiceVisible.setCompanyId(context.getCompanyId());
            practiceVisible.setVisibleSiteId(context.getSiteId());
            practiceVisible.setCreatedUid(context.getAccountId());
            practiceVisible.setPracticeId(practiceId);
            if (PracticeVisibleTypeEnum.PART.getStatus().equals(visibleType)){
                //用户
                practiceVisible.setVisibleAccountId(visibleAccount.getVisibleId());
                practiceVisible.setVisibleAccountWorkNum(visibleAccount.getWorkNum());
                practiceVisible.setVisibleAccountFullName(visibleAccount.getFullName());
                practiceVisible.setVisibleAccountName(visibleAccount.getName());
            }
            if (PracticeVisibleTypeEnum.ORG.getStatus().equals(visibleType)){
                //部门
                practiceVisible.setVisibleOrgId(visibleAccount.getVisibleId());
                practiceVisible.setVisibleOrgName(visibleAccount.getName());
            }
            visibleRepository.save(practiceVisible);
        }
        return BizResponse.ok();
    }

    @Override
    public PracticeVisibleAccountVo getPracticeVisibleList(Long practiceId) {

        PracticeVisibleAccountVo visibleAccountVo = new PracticeVisibleAccountVo();
        visibleAccountVo.setPracticeId(practiceId);
        List<PracticeVisibleInfoVo> practiceVisibleInfoVoList = new ArrayList<>();
        List<PracticeVisible> practiceVisibleList = iPracticeVisibleService.getListByPracticeId(practiceId);
        if (practiceVisibleList.isEmpty()){
            visibleAccountVo.setVisibleInfoList(practiceVisibleInfoVoList);
            return visibleAccountVo;
        }
        for (PracticeVisible practiceVisible : practiceVisibleList) {

            Integer visibleType = practiceVisible.getVisibleType();
            PracticeVisibleInfoVo visibleInfoVo = new PracticeVisibleInfoVo();
            visibleInfoVo.setId(practiceVisible.getId());
            visibleInfoVo.setVisibleType(practiceVisible.getVisibleType());

            if (PracticeVisibleTypeEnum.ORG.getStatus().equals(visibleType)){
                //部门
                visibleInfoVo.setName(practiceVisible.getVisibleOrgName());
                visibleInfoVo.setVisibleId(practiceVisible.getVisibleOrgId());

            }else if (PracticeVisibleTypeEnum.PART.getStatus().equals(visibleType)){
                //用户
                visibleInfoVo.setName(practiceVisible.getVisibleAccountName());
                visibleInfoVo.setVisibleId(practiceVisible.getVisibleAccountId());
                visibleInfoVo.setFullName(practiceVisible.getVisibleAccountFullName());
                visibleInfoVo.setWorkNum(practiceVisible.getVisibleAccountWorkNum());
            }
            practiceVisibleInfoVoList.add(visibleInfoVo);
        }
        visibleAccountVo.setVisibleInfoList(practiceVisibleInfoVoList);
        return visibleAccountVo;
    }

    @Override
    @Transactional
    public BizResponse importPracticeChat(SceneChatImportReq sceneChatImportReq) throws Exception {

        RequestContext context = ContextHolder.get();
        String fileName = sceneChatImportReq.getFileName();
        String fileUrl = sceneChatImportReq.getFileUrl();
        Long practiceId = sceneChatImportReq.getPracticeId();
        Long sceneId = sceneChatImportReq.getSceneId();
        List<List<String>> parse = excelImporter.parse(fileUrl, createChatImportTemplate());
        if (null == parse || parse.size() == 0){
            logger.error("智能陪练导入对话失败。解析excel数据为空; accountId = {} , 入参: {}",context.getAccountId(),JSONObject.toJSONString(sceneChatImportReq));
            return BizResponse.fail("导入对话失败,解析对话excel数据为空");
        }
        logger.info("智能演练导入对话数据: {}",JSONObject.toJSONString(parse));
        SceneConfig scene = sceneConfigRepository.getOne(sceneId);
        PracticeConfig practice = practiceConfigRepository.getOne(practiceId);
        //清空原有对话
        clearSceneChat(sceneId);
        //默认开始节点坐标
        int startXAxis = PracticeConstant.CHAT_DEFAULT_XAXIS;
        int startYAxis = PracticeConstant.CHAT_DEFAULT_YAXIS;
        //纵坐标依次递增数值
        int autoIncrement = PracticeConstant.YAXIS_STEP_LEN;
        AtomicInteger latestLevel = new AtomicInteger(0);
        AtomicInteger latestYAxis = new AtomicInteger(startYAxis);
        SceneChatConfig startNode = new SceneChatConfig();
        Long startNodeId = idGenerator.generate();
        logger.info("导入对话开始节点id:={} ,accountId = {},参数 = {} ",startNodeId,context.getAccountId(),JSONObject.toJSONString(sceneChatImportReq));
        startNode.setId(startNodeId);
        setChatProperties(startNode,0L,latestLevel.get(),PracticeNodeEnum.START,String.valueOf(startXAxis),String.valueOf(startYAxis),practice,scene,context);
        chatConfigRepository.save(startNode);
        Long pId = startNodeId;
        //错误信息的列表
        Set<String> errMsgs = new TreeSet<>();
        //需要新增的对话节点
        HashMap<Long,SceneChatConfig> addNodeMap = new HashMap<>();
        List<SceneChatConfig> addChatNodeList = new ArrayList<>();
        //key=excel第一列序列号: value=学生节点SceneChatConfig ;
        Map<String,SceneChatConfig> orderMap = new HashMap<>();
        logger.info("获取到excel: {},序号列集合size = {}",fileName,parse.size());
        int maxOrderValue = getMaxOrder(parse);
        logger.info("智能演练导入对话解析到文件{}最大对话序列号= {}",fileName,maxOrderValue);

        for (List<String> rowData : parse) {

            String orderValue = rowData.get(0);
            if (StringUtils.isEmpty(orderValue)){
                continue;
            }
            int intOrderValue = Double.valueOf(orderValue).intValue();
            String robotContent = rowData.get(1);
            if (StringUtils.isEmpty(robotContent)){
                logger.info("机器人节点话术内容为空该行数据无效!accountId = {} ,行数据为: {} ,文件数据为: {}",context.getAccountId(),JSONObject.toJSONString(rowData),JSONObject.toJSONString(sceneChatImportReq));
                errMsgs.add("[第"+intOrderValue+"对话序机器人话术为空.]");
               continue;
            }
            if (robotContent.length() > 250){
                errMsgs.add("[第"+intOrderValue+"对话序机器人话术长度大于250.]");
                continue;
            }
            /*if (checkRegex(robotContent,contentRegex)){
                errMsgs.add("[第"+intOrderValue+"对话序机器人话术包含英文]");
            }*/
            if (orderMap.containsKey(orderValue)){
                SceneChatConfig stuNode = orderMap.get(orderValue);
                String accountContentKeyword = stuNode.getAccountContentKeyword();
                ObjectMapper mapper = new ObjectMapper();
                HashMap<String,String> keywordMap = new HashMap<>();
                if (!StringUtils.isEmpty(accountContentKeyword)){
                    try {
                        keywordMap = mapper.readValue(accountContentKeyword, HashMap.class);
                    } catch (JsonProcessingException e) {
                        logger.error("智能陪练关键字解析失败;accountId = {} ,accountContentKeyword = {} ;",context.getAccountId(),accountContentKeyword,e);
                        errMsgs.add("[第"+intOrderValue+"对话序关键字解析失败]");
                    }
                }
                //关键词
                String keyword = rowData.get(3);
                if (!StringUtils.isEmpty(keyword)){
                    String extendWord = rowData.get(4);
                    boolean valid = checkKeywordAndExtend(keyword, extendWord, errMsgs, keywordRegex, commentRegex, intOrderValue,stuNode.getAccountContent());
                    if (!valid){
                        continue;
                    }
                    keywordMap.put(keyword,extendWord);
                    stuNode.setAccountContentKeyword(JSONObject.toJSONString(keywordMap));
                    addNodeMap.put(stuNode.getId(),stuNode);
                    orderMap.put(orderValue,stuNode);
                }
                //禁忌词汇
                String tabooWord = rowData.get(5);
                if (!StringUtils.isEmpty(tabooWord)){
                    boolean valid = checkTabooWord(tabooWord, stuNode.getAccountContentTabooWord(), errMsgs, commentRegex, intOrderValue);
                    if (!valid){
                        continue;
                    }
                    stuNode.setAccountContentTabooWord(tabooWord);
                }
            }else {
               //机器人对话节点
               SceneChatConfig robotNode = new SceneChatConfig();
               Long robotNodeId = idGenerator.generate();
               robotNode.setId(robotNodeId);
                int robotLevel = latestLevel.incrementAndGet();
                setChatProperties(robotNode,pId,robotLevel,PracticeNodeEnum.ROBOT,String.valueOf(startXAxis),String.valueOf(latestYAxis.addAndGet(autoIncrement)),practice,scene,context);
                robotNode.setRobotContent(robotContent);
                addNodeMap.put(robotNodeId,robotNode);
                //学员对话节点
                SceneChatConfig stuNode = new SceneChatConfig();
                String studentContent = rowData.get(2);
                if (StringUtils.isEmpty(studentContent)){
                    //判断是否是最后结束语句
                    if (maxOrderValue != intOrderValue){
                        errMsgs.add("[第"+intOrderValue+"对话序学员话术内容为空]");
                    }else {
                        logger.info("智能演练导入对话,学员话术为空,解析对话结束=========序列号={},机器人语句={}",intOrderValue,robotContent);
                        break;
                    }
                }
                if (studentContent.length() > 250){
                    errMsgs.add("[第"+intOrderValue+"对话序学员话术长度大于250.]");
                    continue;
                }
                /*if (checkRegex(studentContent,contentRegex)){
                    errMsgs.add("[第"+intOrderValue+"对话序学员话术包含英文.]");
                }*/
                Long stuNodeId = idGenerator.generate();
                stuNode.setId(stuNodeId);
                int stuLevel = latestLevel.incrementAndGet();
                setChatProperties(stuNode,robotNodeId,stuLevel,PracticeNodeEnum.STUDENT,String.valueOf(startXAxis),String.valueOf(latestYAxis.addAndGet(autoIncrement)),practice,scene,context);
                stuNode.setAccountContent(studentContent);
                Map<String,String> keywordMap = new HashMap<>();
                //关键词
                String keyword = rowData.get(3);
                if (!StringUtils.isEmpty(keyword)){
                    //扩展词
                    String extendWord = rowData.get(4);
                    boolean valid = checkKeywordAndExtend(keyword, extendWord, errMsgs, keywordRegex, commentRegex, intOrderValue,studentContent);
                    if (!valid){
                        continue;
                    }
                    keywordMap.put(keyword,extendWord);
                    stuNode.setAccountContentKeyword(JSONObject.toJSONString(keywordMap));
                }
                //禁忌词汇
                String tabooWord = rowData.get(5);
                if (!StringUtils.isEmpty(tabooWord)){
                    boolean valid = checkTabooWord(tabooWord, stuNode.getAccountContentTabooWord(), errMsgs, commentRegex, intOrderValue);
                    if (!valid){
                        continue;
                    }
                    stuNode.setAccountContentTabooWord(tabooWord);
                }
                addNodeMap.put(stuNodeId,stuNode);
                pId = stuNodeId;
                orderMap.put(orderValue,stuNode);
            }
        }
        addNodeMap.forEach((id,chatNode) -> {
            Integer sceneNodeType = chatNode.getSceneNodeType();
            if (PracticeNodeEnum.STUDENT.getStatus().equals(sceneNodeType)){
                String accountContentKeyword = chatNode.getAccountContentKeyword();
                String accountContent = chatNode.getAccountContent();
                String msg = accountContent;
                if (accountContent.length() > 5){
                    msg = accountContent.substring(0,5);
                }
                if (StringUtils.isEmpty(accountContentKeyword)){
                    errMsgs.add("['"+msg+"...' 对话内容关键词为空]");
                }else {
                    ObjectMapper mapper = new ObjectMapper();
                    try {
                        Map<String,String> map = mapper.readValue(accountContentKeyword, Map.class);
                        if (null == map || map.isEmpty()){
                            errMsgs.add("['"+msg+"...' 对话内容关键词为空]");
                        }
                    } catch (JsonProcessingException e) {
                        logger.error("导入对话关键字解析异常",e);
                        errMsgs.add("['"+msg+"...' 对话内容关键词解析异常;keyword = "+accountContentKeyword+"']");
                    }
                }
            }
            addChatNodeList.add(chatNode);
        });
        if (errMsgs.size() > 0){
            logger.error("导入对话失败；accountId = {},practiceId = {},sceneId = {},错误信息: {}",context.getAccountId(),practiceId,sceneId,JSONArray.toJSON(errMsgs));
            throw new BizException(InternationalEnums.MATERIALCONTROLLER1.getCode(), JSONArray.toJSONString(errMsgs));
        }
        chatConfigRepository.saveAll(addChatNodeList);
        return BizResponse.ok();
    }

    /**
     * 校验禁忌词
     * @param tabooWord 禁忌词文本
     * @param accountContentTabooWord 已设置的禁忌词
     */
    private boolean checkTabooWord(String tabooWord,String accountContentTabooWord,Set<String> errMsgs,String commentRegex,int intOrderValue){

        if (!StringUtils.isEmpty(accountContentTabooWord) && !accountContentTabooWord.equals(tabooWord)){
            errMsgs.add("[第"+intOrderValue+"对话序禁忌词格式错误]");
            return false;
        }
        if (!checkRegex(tabooWord,commentRegex)){
            errMsgs.add("[第"+intOrderValue+"对话序禁忌词包含中文/英文分号以外其他字符]");
            return false;
        }
        return true;
    }

    //校验关键词和扩展词汇
    private boolean checkKeywordAndExtend(String keyword,String extendWord,Set<String> errMsgs,String keywordRegex,String commentRegex,int intOrderValue,String accountContent){
        if (keyword.length() > 10){
            errMsgs.add("[第"+intOrderValue+"对话序关键字长度大于10]");
            return false;
        }
        if (!checkRegex(keyword,keywordRegex)){
            errMsgs.add("[第"+intOrderValue+"对话序关键词包含非中文]");
            return false;
        }
        //扩展词
        if (!StringUtils.isEmpty(extendWord)){
            if (!checkRegex(extendWord,commentRegex)){
                errMsgs.add("[第"+intOrderValue+"对话序拓展词包含中文/英文分号以外其他字符]");
                return false;
            }
        }
        if (!accountContent.contains(keyword)){
            errMsgs.add("[第"+intOrderValue+"对话序,关键词不属于标准话术内容]");
            return false;
        }
        return true;
    }

    /**
     * 校验字符串是否全是中文
     * @param str 文本
     * @param regex 正则表达式
     * @return true 符合正则规则；false 不符合
     */
    private boolean checkRegex(String str,String regex) {
        Pattern p = Pattern.compile(regex);
        Matcher m = p.matcher(str);
        return m.matches();
    }

    //获取最大的序列号
    private int getMaxOrder(List<List<String>> parse) {
        Set<Integer> orderSet = new HashSet<>();
        for (List<String> rowData : parse) {
            String orderValue = rowData.get(0);
            if (StringUtils.isEmpty(orderValue)){
                continue;
            }
            Integer integer = Double.valueOf(orderValue).intValue();
            orderSet.add(integer);
        }
        Integer maxOrder = orderSet.stream().distinct().max((e1, e2) -> e1.compareTo(e2)).get();
        return maxOrder;
    }

    private void setChatProperties(SceneChatConfig chatNode,Long pId,int level,PracticeNodeEnum nodeType, String xAxis,String yAxis,PracticeConfig practice,SceneConfig scene,RequestContext context){
        chatNode.setLevel(level);
        chatNode.setPid(pId);
        chatNode.setSceneNodeType(nodeType.getStatus());
        chatNode.setxAxis(xAxis);
        chatNode.setyAxis(yAxis);
        chatNode.setPracticeId(practice.getId());
        chatNode.setSceneId(scene.getId());
        chatNode.setSceneName(scene.getName());
        chatNode.setOrgId(context.getOrgId());
        chatNode.setSiteId(context.getSiteId());
        chatNode.setCompanyId(context.getCompanyId());
        chatNode.setCreatedUid(context.getAccountId());
    }

    private Template createChatImportTemplate(){
        Template template = new Template();
        List<TemplateColumn> columns = new ArrayList<>();
        columns.add(new TemplateColumn("order","序列号",1));
        columns.add(new TemplateColumn("robotNode","机器人节点话术(必填)",2));
        columns.add(new TemplateColumn("studentNode","学员节点话术(必填)",3));
        columns.add(new TemplateColumn("keyword","关键词(必填)",4));
        columns.add(new TemplateColumn("extendWord","扩展词(非必填)",5));
        columns.add(new TemplateColumn("tabooWord","禁忌词",6));
        template.setColumns(columns);
        return template;
    }

    @Override
    @Transactional
    public BizResponse clearSceneChat(Long sceneId) {

        RequestContext context = ContextHolder.get();
        SceneConfig scene = sceneConfigRepository.getOne(sceneId);
        Long practiceId = scene.getPracticeId();
        if (null == practiceId){
            logger.error("[PracticeXServiceImpl:clearSceneChat]场景对应陪练id为空.sceneId = {},accountId = {}",sceneId,context.getAccountId());
            return BizResponse.fail("清空对话失败");
        }
        List<SceneChatConfig> sceneChatConfigs = chatConfigService.selectChatNodes(practiceId, sceneId, null, null, null);
        for (SceneChatConfig sceneChatConfig : sceneChatConfigs) {
            sceneChatConfig.setDeleted(true);
            sceneChatConfig.setUpdatedUid(context.getAccountId());
        }
        chatConfigRepository.saveAll(sceneChatConfigs);
        return BizResponse.ok();
    }

    /**
     * 递归获取节点及其子节点信息
     * @param chatNode 节点
     */
    private void getSceneChatChildNodes(SceneChatConfig chatNode,PracticeChatNodeReq chatNodeReq) throws JsonProcessingException {

        if (null == chatNode){
            return;
        }
        List<PracticeChatNodeReq> childChatNodes = new ArrayList<>();
        Long practiceId = chatNode.getPracticeId();
        Long sceneId = chatNode.getSceneId();
        BeanUtils.copyProperties(chatNode,chatNodeReq);
        List<SceneChatConfig> nextChatNodes = chatConfigService.selectChatNodes(practiceId, sceneId,null, chatNode.getId(), null);
        if (null != nextChatNodes && nextChatNodes.size() > 0 ){
            for (SceneChatConfig childChat : nextChatNodes){
                PracticeChatNodeReq childChatReq = new PracticeChatNodeReq();
                BeanUtils.copyProperties(childChat,childChatReq);
                childChatReq.setAccountRealVoiceUrl(childChat.getAccountContentVoiceUrl());
                String accountContentKeyword = childChat.getAccountContentKeyword();
                //解析关键字
                List<KeyWordsReq> keyWordsReqList = new ArrayList<>();
                if (!StringUtils.isEmpty(accountContentKeyword)){
                    ObjectMapper mapper = new ObjectMapper();
                    HashMap<String,String> keywordMap = mapper.readValue(accountContentKeyword, HashMap.class);
                    keywordMap.forEach((keyword,extendWord) -> {
                        KeyWordsReq keyWordsReq = new KeyWordsReq();
                        keyWordsReq.setKey(keyword);
                        keyWordsReq.setValue(extendWord);
                        keyWordsReqList.add(keyWordsReq);
                    });
                }
                childChatReq.setKeyWordList(keyWordsReqList);
                getSceneChatChildNodes(childChat,childChatReq);
                childChatNodes.add(childChatReq);
            }
        }
        chatNodeReq.setChildNodes(childChatNodes);
    }



    /**保存或者修改陪练场景对话信息*/
    private void saveOrUpdatePracticeDialogue(PracticeConfig practice,Long sceneId,Long pId, List<PracticeChatNodeReq> chatNodes) {

        if (null == chatNodes || chatNodes.size() == 0){
            return;
        }
        for (PracticeChatNodeReq chatNodeReq : chatNodes){

            SceneChatConfig conversation = new SceneChatConfig();
            SceneConfig scene = sceneConfigRepository.getOne(sceneId);
            conversation.setSceneName(scene.getName());
            Long chatNodeId = chatNodeReq.getId();
            if (null == chatNodeId || chatNodeId <= 0){
                Long id = idGenerator.generate();
                conversation.setId(id);
            }else {
                conversation  = chatConfigRepository.getOne(chatNodeId);
                conversation.setDeleted(false);
            }
            conversation.setSceneId(sceneId);
            conversation.setPid(pId);
            conversation.setPracticeId(practice.getId());
            conversation.setLevel(chatNodeReq.getLevel());
            conversation.setSceneNodeType(chatNodeReq.getSceneNodeType());
            conversation.setxAxis(chatNodeReq.getXAxis());
            conversation.setyAxis(chatNodeReq.getYAxis());
            Integer sceneNodeType = chatNodeReq.getSceneNodeType();
            if (PracticeNodeEnum.ROBOT.getStatus().equals(sceneNodeType)){
                //机器人节点
                String oldRobotContent = conversation.getRobotContent();
                String robotContent = chatNodeReq.getRobotContent();
                conversation.setRobotContent(robotContent);
                if (!StringUtils.isEmpty(oldRobotContent) && !oldRobotContent.equals(robotContent)){
                    //标准话术修改，合成语音置空
                    conversation.setRobotContentVoiceUrl(null);
                    conversation.setRobotContentVoiceDuration(null);
                }
            }
            if (PracticeNodeEnum.STUDENT.getStatus().equals(sceneNodeType)){
                //学员节点
                String oldStuContent = conversation.getAccountContent();
                String accountContent = chatNodeReq.getAccountContent();
                conversation.setAccountContent(accountContent);
                conversation.setAccountContentTabooWord(chatNodeReq.getAccountContentTabooWord());
                String accountRealVoiceUrl = chatNodeReq.getAccountRealVoiceUrl();
                String accountContentFileName = chatNodeReq.getAccountContentFileName();
                conversation.setAccountContentFileName(accountContentFileName);
                if (StringUtils.isEmpty(accountContentFileName)){
                    if (!StringUtils.isEmpty(oldStuContent) && !oldStuContent.equals(accountContent)){
                        //标准话术修改,合成语音置空
                        conversation.setAccountContentVoiceUrl(null);
                        conversation.setAccountContentVoiceDuration(null);
                    }
                }else{
                    conversation.setAccountContentVoiceUrl(accountRealVoiceUrl);
                    long audioDuration = AudioUtils.INSTANCE.getAudioDuration(accountRealVoiceUrl);
                    conversation.setAccountContentVoiceDuration(audioDuration);
                }
                List<KeyWordsReq> keyWordList = chatNodeReq.getKeyWordList();
                if (null != keyWordList){
                    HashMap<String,String> keyMap = new HashMap<>();
                    for (KeyWordsReq keyWordsReq : keyWordList) {
                        String keyword = keyWordsReq.getKey();
                        String extendWord = keyWordsReq.getValue();
                        if (!StringUtils.isEmpty(keyMap)){
                            keyMap.put(keyword,extendWord);
                        }
                    }
                    conversation.setAccountContentKeyword(JSONObject.toJSONString(keyMap));
                }
            }
            //获取该节点下节点信息
            List<PracticeChatNodeReq> childNodes = chatNodeReq.getChildNodes();
            RequestContext context = ContextHolder.get();
            conversation.setOrgId(context.getOrgId());
            conversation.setSiteId(context.getSiteId());
            conversation.setCompanyId(context.getCompanyId());
            conversation.setCreatedUid(context.getAccountId());
            conversation.setUpdatedUid(context.getAccountId());
            chatConfigRepository.save(conversation);
            //获取对话表id
            Long newChatNodeId = conversation.getId();
            //递归 保存节点下字节点信息
            saveOrUpdatePracticeDialogue(practice,sceneId,newChatNodeId,childNodes);
        }
    }

    //上传文件到oss
    private String upFileToOss(PracticeConfig practice, String content,RequestContext context) throws Exception {

        //RequestContext context = ContextHolder.get();
        logger.info("context.getAccountId--->>>{}",context.getAccountId());
        String filePath = OssFileUtil.getManageStandardVoiceUri(context.getCompanyId(), context.getSiteId(), PracticeConstant.BUSINESS_TYPE
                , PracticeConstant.SUB_BUSINESS_TYPE, context.getAccountId(),idGenerator.generate());
        return xunfeiUtil.stringToVoice(content, practice.getRobotParams(), filePath,PracticeVoiceTypeEnum.MP3);
    }


    /**保存或者修改陪练积分规则*/
    private void saveOrUpdatePracticePointRule(Integer newExamGivenPointType, Long practiceId, List<PracticePointVo> practicePointList) {

        // 1.新老相等 && 等于true   遍历saveOrUpdateOrDel
        //2.新老规则不相等；false-> true 遍历新增；true->false 查询所有删除；
        //查询陪练下所有积分规则，首先设置逻辑删除
        pointRuleConfigRepository.deleteRulesByPractice(practiceId);
        if (PracticePointGivenEnum.TRUE.getStatus().equals(newExamGivenPointType)) {
            //设置积分规则
            if (null != practicePointList && practicePointList.size() > 0) {
                List<PracticePointRuleConfig> pointRuleList = new ArrayList<>();
                practicePointList.forEach(pointVo -> {
                    Long pointId = pointVo.getId();
                    PracticePointRuleConfig pointRule = new PracticePointRuleConfig();
                    if (null == pointId || 0L == pointId) {
                        pointRule.setId(idGenerator.generate());
                    } else {
                        pointRule = pointRuleConfigRepository.getOne(pointId);
                        pointRule.setDeleted(false);
                    }
                    pointRule.setPracticeId(practiceId);
                    pointRule.setMinScore(pointVo.getMinScore());
                    pointRule.setMaxScore(pointVo.getMaxScore());
                    pointRule.setPoint(pointVo.getPoint());
                    pointRuleList.add(pointRule);
                });
                pointRuleConfigRepository.saveAll(pointRuleList);
            }
        }
    }

    /**
     * 校验策略配置参数是否符合规则
     * @param strategyInfoDto 策略配置入参
     */
    private void checkPracticeStrategyParamsValid(PracticeStrategyInfoDto strategyInfoDto) {
        // 校验策略参数
        RequestContext context = ContextHolder.get();
        Integer visibleType = strategyInfoDto.getVisibleType();
        if (null == visibleType
                || (!PracticeVisibleTypeEnum.ALL.getStatus().equals(visibleType)
                    && !PracticeVisibleTypeEnum.PART.getStatus().equals(visibleType)
                    )
        ){
            logger.error("[checkPracticeStrategyParamsValid]没有匹配的可见范围 accountId= {} ,入参: {}",context.getAccountId(),JSONObject.toJSONString(strategyInfoDto));
            throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
        }
        // 校验评分规则
        checkScoreRuleValid(strategyInfoDto);
        //陪练模式 80:训练+考核 ,10:训练 20:考核
        Integer useType = strategyInfoDto.getUseType();
        //每轮对话过关条件 80:开启 10:关闭
        Integer pracNextRoundCondition = strategyInfoDto.getStartNextRoundType();
        Integer skipType = strategyInfoDto.getSkipType();
        if (PracticeModelStatusEnum.ASSESSMENT.getStatus().equals(useType)){
            //考核模式
            if (null != skipType || null != pracNextRoundCondition){
                logger.error("[checkPracticeStrategyParamsValid]考核模式下无需设置训练策略配置 accountId= {} ,入参: {}",context.getAccountId(),JSONObject.toJSONString(strategyInfoDto));
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
            }
            //校验考核模式下的参数
            checkExamTypeParams(strategyInfoDto, context);
        }else if (PracticeModelStatusEnum.PRA_ASSESS.getStatus().equals(useType)){
            // 训练模式参数校验
            checkPracticeTypeParams(strategyInfoDto, context);
            //校验考核模式下的参数
            checkExamTypeParams(strategyInfoDto, context);
        }else {
            //训练模式
            Integer repeatPush = strategyInfoDto.getRepeatPush();
            Integer manyRoundChat = strategyInfoDto.getManyRoundChat();
            Integer examPassMinScore = strategyInfoDto.getExamPassMinScore();
            Integer examGivenPointType = strategyInfoDto.getExamGivenPointType();
            if (null != manyRoundChat || null != examPassMinScore || null != examGivenPointType || null != repeatPush){
                logger.error("[checkPracticeStrategyParamsValid]训练模式下无需设置考核策略配置参数； accountId= {} ,入参: {}",context.getAccountId(), JSONObject.toJSONString(strategyInfoDto));
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
            }
            // 训练模式参数校验
            checkPracticeTypeParams(strategyInfoDto, context);
        }
    }

    /**
     * 校验包含训练模式的参数
     * @param strategyInfoDto 入参
     * @param context requestContext
     */
    private void checkPracticeTypeParams(PracticeStrategyInfoDto strategyInfoDto, RequestContext context) {

        Integer skipType = strategyInfoDto.getSkipType();
        //每轮对话过关条件 80:开启 10:关闭
        Integer pracNextRoundCondition = strategyInfoDto.getStartNextRoundType();
        if (null == skipType || null == pracNextRoundCondition){
            logger.error("[checkPracticeStrategyParamsValid]训练策略配置必填参数包含空值 accountId= {} ,入参: {}",context.getAccountId(),JSONObject.toJSONString(strategyInfoDto));
            throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
        }
        if (pracNextRoundCondition.equals(PracticeManyRoudChatEnum.FALSE.getStatus())){
            return;
        }
        //开启每轮对话过关条件
        //过关分数
        Integer startNextRoundScore = strategyInfoDto.getStartNextRoundScore();
        //强制练习次数
        Integer practiceFailedMaxCount = strategyInfoDto.getPracticeFailedMaxCount();
        if (null == startNextRoundScore || null == practiceFailedMaxCount
                || 0 > startNextRoundScore || 0 >= practiceFailedMaxCount){
            logger.error("[checkPracticeStrategyParamsValid]训练策略配置开启过关条件参数填写错误. accountId= {} ,入参: {}",context.getAccountId(),JSONObject.toJSONString(strategyInfoDto));
            throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
        }
    }

    /**
     * 校验包含考核的参数
     * @param strategyInfoDto 入参
     * @param context requestContext
     */
    private void checkExamTypeParams(PracticeStrategyInfoDto strategyInfoDto, RequestContext context) {
        //考核重复提交、多轮对话、考核及格分
        Integer repeatPush = strategyInfoDto.getRepeatPush();
        Integer manyRoundChat = strategyInfoDto.getManyRoundChat();
        Integer examPassMinScore = strategyInfoDto.getExamPassMinScore();
        Integer examGivenPointType = strategyInfoDto.getExamGivenPointType();
        if (null == manyRoundChat || null == examPassMinScore || null == examGivenPointType || null == repeatPush){
            logger.error("[checkPracticeStrategyParamsValid]考核策略配置必填参数包含空值 accountId= {} ,入参: {}",context.getAccountId(), JSONObject.toJSONString(strategyInfoDto));
            throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
        }
        if (examPassMinScore < 0 || examPassMinScore > 100){
            logger.error("考核策略配置-考核及格分超出区间[0,100]; accountId= {} ,入参: {}",context.getAccountId(), JSONObject.toJSONString(strategyInfoDto));
            throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
        }
        if (examGivenPointType.equals(PracticePointGivenEnum.TRUE.getStatus())){
            //发放积分
            List<PracticePointVo> practicePointList = strategyInfoDto.getPracticePointList();
            //校验积分规则
            checkExamPointParams(strategyInfoDto, context, practicePointList);
        }
        if (PracticeManyRoudChatEnum.TRUE.getStatus().equals(manyRoundChat)){
            //开启多轮对话
            //相似度评分阈值
            Integer similarityRate = strategyInfoDto.getSimilarityRate();
            //强制对话轮次
            Integer examFailedMaxCount = strategyInfoDto.getExamFailedMaxCount();
            //未识别回复类型 80:默认回复，10:自定义回复
            Integer similarityFailedMsgType = strategyInfoDto.getSimilarityFailedMsgType();
            if (null == similarityRate || null == examFailedMaxCount || null == similarityFailedMsgType){
                logger.error("[checkPracticeStrategyParamsValid]考核策略配置开启多轮对话必填参数包含空值 accountId= {} ,入参: {}",context.getAccountId(),JSONObject.toJSONString(strategyInfoDto));
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
            }
            if (examFailedMaxCount <= 0){
                logger.error("[checkPracticeStrategyParamsValid]考核策略配置开启多轮对话,强制对话轮次小于0; accountId = {} ,入参: {}",context.getAccountId(),JSONObject.toJSONString(strategyInfoDto));
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
            }
            if (PracticeSimilaryFailedEnum.FALSE.getStatus().equals(similarityFailedMsgType)){
                //自定义未识别回复
                String similarityFailedMsg = strategyInfoDto.getSimilarityFailedMsg();
                if (StringUtils.isEmpty(similarityFailedMsg)){
                    logger.error("[checkPracticeStrategyParamsValid]考核策略配置开启多轮对话,自定义未识别回复为空; accountId = {} ,入参: {}",context.getAccountId(),JSONObject.toJSONString(strategyInfoDto));
                    throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
                }
            }

        }
    }

    private void checkExamPointParams(PracticeStrategyInfoDto strategyInfoDto, RequestContext context, List<PracticePointVo> practicePointList) {
        if (null == practicePointList || practicePointList.isEmpty()){
            logger.error("考核策略配置-开启发放积分,参数为空; accountId= {} ,入参: {}",context.getAccountId(), JSONObject.toJSONString(strategyInfoDto));
            throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
        }
        practicePointList.sort(Comparator.comparing(PracticePointVo::getMinScore));

        Integer preRuleMaxScore = -1;
        Integer preRulePoint = 0;
        for (PracticePointVo pointVo : practicePointList) {
            Integer minScore = pointVo.getMinScore();
            Integer maxScore = pointVo.getMaxScore();
            Integer point = pointVo.getPoint();
            if (null == minScore || null == maxScore || minScore < 0 || maxScore < 0 || null == point || point < 0){
                logger.error("考核策略配置-开启发放积分,积分数值错误; accountId= {} ,入参: {}",context.getAccountId(), JSONObject.toJSONString(strategyInfoDto));
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"得分或发放积分设置有误");
            }
            // 最大100分
            if (pointVo.getPoint() >= 100 ){
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"积分最大设置100分");
            }

            // 发放积分后面的大于前面的
            if (pointVo.getPoint() <= preRulePoint) {
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"发放积分数量设置有误");
            }
            //积分范围不能交叉
            if (pointVo.getMinScore() <= preRuleMaxScore) {
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"得分设置不能有交集");
            }
            preRuleMaxScore = pointVo.getMaxScore();
            preRulePoint = pointVo.getPoint();


        }
    }

    /**
     * 校验评分规则参数是否符合限制要求
     */
    private void checkScoreRuleValid(PracticeStrategyInfoDto strategyInfoDto) {
        RequestContext context = ContextHolder.get();
        Integer scoreRuleComplete = strategyInfoDto.getScoreRuleComplete();
        Integer scoreRuleExpress = strategyInfoDto.getScoreRuleExpress();
        Integer scoreRuleKeyword = strategyInfoDto.getScoreRuleKeyword();
        Integer scoreRulePolite = strategyInfoDto.getScoreRulePolite();
        Integer scoreRuleSmooth = strategyInfoDto.getScoreRuleSmooth();

        if (null == scoreRuleComplete || null == scoreRuleExpress
                || null == scoreRuleKeyword || null == scoreRulePolite || null == scoreRuleSmooth){
            logger.error("[checkScoreRuleValid]评分规则不能包含空值 accountId= {} ,入参: {}",context.getAccountId(),JSONObject.toJSONString(strategyInfoDto));
            throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
        }
        int sum = scoreRuleComplete + scoreRuleExpress + scoreRuleKeyword + scoreRulePolite + scoreRuleSmooth ;
        if (sum != 100){
            logger.error("[checkScoreRuleValid]评分规则总和不等于100 accountId= {} ,入参: {}",context.getAccountId(),JSONObject.toJSONString(strategyInfoDto));
            throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
        }
    }

    /**
     * 校验陪练基本信息参数是否符合规范
     * @param practiceReq 入参
     */
    private void checkPracticeBaseParamsValid(PracticeBaseInfoDto practiceReq) {
        RequestContext context = ContextHolder.get();
        String name = practiceReq.getName();
        if (StringUtils.isEmpty(name) || name.length() > 100){
            logger.error("[checkPracticeBaseParamsValid]名称长度不能大于100 accountId= {} ,入参: {}",context.getAccountId(),JSONObject.toJSONString(practiceReq));
            throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"陪练名称不符合限制条件");
        }
        //校验应用模式
        checkPracticeModelValid(practiceReq.getUseType(),practiceReq.getPracticeStartAt(),practiceReq.getPracticeEndAt()
                ,practiceReq.getExamStartAt(),practiceReq.getExamEndAt(),practiceReq.getMaxExamCount());
        // 校验关键词个数，长度，是否重复
        checkPracticeBaseInfoKeyWord(practiceReq.getKeyWord());
        //校验机器人称呼长度
        String robotAliasName = practiceReq.getRobotAliasName();
        if (StringUtils.isEmpty(robotAliasName) || robotAliasName.length() > 3){
            logger.error("[checkPracticeBaseParamsValid]机器人名称长度不符合要求 accountId= {} ,入参: {}",context.getAccountId(),JSONObject.toJSONString(practiceReq));
            throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
        }
        Long robotId = practiceReq.getRobotId();
        if (null == robotId){
            logger.error("[checkPracticeBaseParamsValid]机器人id不能为空 accountId= {} ,入参: {}",context.getAccountId(),JSONObject.toJSONString(practiceReq));
            throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
        }
    }

    /**校验关键词*/
    private void checkPracticeBaseInfoKeyWord(String keyWordStr) {
        RequestContext context = ContextHolder.get();
        // 校验关键词个数，长度，是否重复
        if (StringUtils.isEmpty(keyWordStr)){
            return;
        }
        String[] keywordArray = keyWordStr.split(",");
        List<String> keyList = new ArrayList<>();
        for (String elem : keywordArray) {
            String keyword = elem.trim();
            if (StringUtils.isEmpty(keyword)) {
                continue;
            }
            if (keyList.contains(keyword)) {
                logger.error("[checkPracticeBaseInfoKeyWord]关键词重复 accountId= {} ,关键词: {}", context.getAccountId(), keyWordStr);
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"关键词重复");
            }
            if (keyword.length() > 10) {
                logger.error("[checkPracticeBaseInfoKeyWord]关键词长度大于10 accountId= {} ,关键词: {}", context.getAccountId(), keyWordStr);
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"关键词长度大于10");
            }
            keyList.add(keyword);
        }
        if (keyList.size() > 5){
            logger.error("[checkPracticeBaseInfoKeyWord]最多添加5个关键词 accountId= {} ,关键词: {}",context.getAccountId(),keyWordStr);
            throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2.getCode(),"关键词个数大于5");
        }
    }

    /**
     * 校验应用模式
     */
    private void checkPracticeModelValid(Integer useType, Date practiceStartAt, Date practiceEndAt, Date examStartAt, Date examEndAt, Integer maxExamCount) {
        RequestContext context = ContextHolder.get();
        if (useType == null
                || (!PracticeModelStatusEnum.ASSESSMENT.getStatus().equals(useType)
                && !PracticeModelStatusEnum.PRA_ASSESS.getStatus().equals(useType)
                && !PracticeModelStatusEnum.PRACTICE.getStatus().equals(useType))
        ) {
            logger.error("[checkPracticeModelValid]没有匹配的应用模式 accountId= {} ,userType= {} ",context.getAccountId(),useType);
            throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
        }
        if (PracticeModelStatusEnum.ASSESSMENT.getStatus().equals(useType)) {
            if (null == examStartAt || null == examEndAt) {
                logger.error("[checkPracticeModelValid]考核应用模式下考核时间不能为空 accountId= {} ,userType= {} ",context.getAccountId(),useType);
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
            }
            if (examStartAt.after(examEndAt)){
                logger.error("[checkPracticeModelValid]考核应用模式下考核开始时间大于结束时间 accountId= {} ,userType= {} ,examStartAt= {} ,examEndAt= {}"
                        ,context.getAccountId(),useType,examStartAt,examEndAt);
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
            }
            // 校验考核次数数值
            if (null == maxExamCount || maxExamCount < 0) {
                logger.error("[checkPracticeModelValid]考核应用模式下考核次数填写不规范 accountId= {} ,userType= {} ,maxExamCount = {}",context.getAccountId(),useType,maxExamCount);
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
            }
        }else if (PracticeModelStatusEnum.PRACTICE.getStatus().equals(useType)) {
            if (null == practiceStartAt || null == practiceEndAt) {
                logger.error("[checkPracticeModelValid]训练应用模式下训练时间不能为空 accountId= {} ,userType= {} ",context.getAccountId(),useType);
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
            }
            if (practiceStartAt.after(practiceEndAt)){
                logger.error("[checkPracticeModelValid]训练应用模式下开始时间大于结束时间 accountId= {} ,userType= {} ,practiceStartAt = {},practiceEndAt = {}"
                        ,context.getAccountId(),useType,practiceStartAt,practiceEndAt);
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
            }
        }else if (PracticeModelStatusEnum.PRA_ASSESS.getStatus().equals(useType)) {
            if (null == practiceStartAt || null == practiceEndAt
                    || null == examStartAt || null == examEndAt) {
                logger.error("[checkPracticeModelValid]训练+考核应用模式下训练和考核时间都不能为空! accountId= {} ,userType= {} ",context.getAccountId(),useType);
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
            }
            if (null == maxExamCount || maxExamCount < 0) {
                logger.error("[checkPracticeModelValid]训练+考核应用模式下考核次数填写不规范 accountId= {} ,userType= {} ,maxExamCount = {}",context.getAccountId(),useType,maxExamCount);
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
            }
            if (practiceStartAt.after(practiceEndAt)){
                logger.error("[checkPracticeModelValid]训练+考核应用模式下训练开始时间大于结束时间 accountId= {} ,userType= {} ,practiceStartAt = {},practiceEndAt = {}"
                        ,context.getAccountId(),useType,practiceStartAt,practiceEndAt);
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
            }
            if (examStartAt.after(examEndAt)){
                logger.error("[checkPracticeModelValid]训练+考核应用模式下考核开始时间大于结束时间 accountId= {} ,userType= {} ,examStartAt= {} ,examEndAt= {}"
                        ,context.getAccountId(),useType,examStartAt,examEndAt);
                throw new BizException(InternationalEnums.SITECLASSIFYMANAGECOMTROLLER2);
            }
        }
    }

    /**
     * 设置陪练数据更新人信息
     */
    private void setCreatedAccountInfo(PracticeConfig practiceConfig) {
        RequestContext requestContext = ContextHolder.get();
        practiceConfig.setCreatedUid(requestContext.getAccountId());
        practiceConfig.setSiteId(requestContext.getSiteId());
        practiceConfig.setCompanyId(requestContext.getCompanyId());
        practiceConfig.setOrgId(requestContext.getOrgId());
    }

    /**
     * 设置陪练数据更新人信息
     */
    private void setUpdatedAccountInfo(PracticeConfig practiceConfig) {
        RequestContext requestContext = ContextHolder.get();
        practiceConfig.setUpdatedUid(requestContext.getAccountId());
    }


	@Override
	@Transactional(rollbackFor=Exception.class)
	public Long copyPractice(Long practiceId) throws IllegalAccessException, IllegalArgumentException, InvocationTargetException, SecurityException {
		RequestContext requestContext = ContextHolder.get();
		
		String[] ignoreProperties = new String[] {"id", "orgId", "siteId", "companyId", "createdUid", "createdAt", "updatedUid", "updatedAt"};
		
		Long targetId = idGenerator.generate();
		copyPracticeConfig(practiceId, targetId, ignoreProperties);
		
		// 复制practice_point_rule
		copyRule(practiceId, targetId, ignoreProperties, requestContext);
		
		// 复制practice_visible
		copyVisible(practiceId, targetId, ignoreProperties, requestContext);
		
		// 复制scene_config 这里不复制删除掉的场景，但是也不更新sort，例如1,2,3,4, 删除了3，但是1，2，4依然可以排序
		copyScene(practiceId, targetId, ignoreProperties, requestContext);
		
		return targetId;
	}

    @Override
    @Transactional
    public BizResponse delete(Long practiceId) {
        RequestContext context = ContextHolder.get();
        PracticeConfig practice = practiceConfigRepository.getOne(practiceId);
        if (null == practice || null == practice.getId()){
            logger.error("[PracticeXServiceImpl:delete]删除陪练失败! 没有查询到陪练信息 practiceId = {};accountId = {}",practiceId,context.getAccountId());
            throw new BizException(InternationalEnums.CLASSIFYCONTROLLER4);
        }
        //删除陪练
        Integer practiceStatus = practice.getStatus();
        if (!PracticeStatusEnum.DRAFT.getStatus().equals(practiceStatus)){
            //判断陪练状态；上架的不能删除
            logger.warn("[PracticeXServiceImpl:delete]删除陪练失败! 上架/下架状态的陪练不能删除! practiceId = {};accountId = {}",practiceId,context.getAccountId());
            throw new BizException(InternationalEnums.CLASSIFYCONTROLLER4);
        }
        iPracticeConfigService.delete(practiceId);
        //删除陪练场景
        sceneConfigService.deleteScene(practiceId, null);
        //删除陪练场景对话
        List<SceneChatConfig> sceneChatConfigs = chatConfigService.selectChatNodes(practiceId, null, null, null, null);
        List<Long> delChatIds = new ArrayList<>();
        if (null != sceneChatConfigs && !sceneChatConfigs.isEmpty()){
            for (SceneChatConfig sceneChatConfig : sceneChatConfigs) {
                chatConfigService.deleteChatNodes(practiceId,sceneChatConfig.getSceneId(),null,null,null);
                delChatIds.add(sceneChatConfig.getId());
            }
            logger.info("[PracticeXServiceImpl:delete]删除陪练场景对话: accountId = {},delChatIds= {}",context.getAccountId(),JSONObject.toJSONString(delChatIds));
        }
        return BizResponse.ok();
    }

    @Override
    public BizResponse importAccount(PracticeImportAccountReq importAccountReq) {
        Long taskId = idGenerator.generate();
        String serialNo = "PRACTICE-ACCOUNT-IMPORT-" + taskId;
        String taskName = "智能陪练可见范围导入-" + taskId.toString();
        String ossUrl = importAccountReq.getFileUrl();
        Long practiceId = importAccountReq.getPracticeId();
        PracticeAccountImportParam accountImportParam = new PracticeAccountImportParam();
        accountImportParam.setFileUrl(ossUrl);
        accountImportParam.setPracticeId(practiceId);
        accountImportParam.setSerialNo(serialNo);
        accountImportParam.setTaskId(taskId);
        accountImportParam.setTaskName(taskName);
        accountImportParam.setContext(ContextHolder.get());
        accountImport.execute(accountImportParam, true);
        return BizResponse.ok("任务序号:" + serialNo + ",任务名称:" + taskName);

    }

    @Override
    public BizResponse exportAccount(Long practiceId) {

        PracticeAccountExportParam exportParam = new PracticeAccountExportParam();
        RequestContext context = ContextHolder.get();
        exportParam.setContext(context);
        exportParam.setPracticeId(practiceId);

        //exportParam.setTaskContext(taskContext);
        return BizResponse.ok(accountExport.execute(exportParam));
        //return BizResponse.ok(serialNo);
    }

    @Override
    public BizResponse checkPracticeSceneValid(Long practiceId) {
        List<SceneConfig> sceneConfigList = sceneConfigService.findSceneByPracticeId(practiceId);
        if (null == sceneConfigList || sceneConfigList.isEmpty()){
            logger.error("陪练下场景对话校验不通过,未查询到陪练下场景列表. practiceId = {} ,accountId = {}",practiceId);
            throw new BizException(PracticeInternationEnum.SCENE_CHAT_INCOMPLETE.getCode(),PracticeInternationEnum.SCENE_CHAT_INCOMPLETE.getName());
        }
        for (SceneConfig sceneConfig : sceneConfigList) {
            Long sceneId = sceneConfig.getId();
            //校验场景下对话
            checkSceneChatValid(practiceId,sceneId);
        }
        return BizResponse.ok();
    }

    @Override
    public List<PracticeVo> selectRangeList(Date startDate, Date endDate) {

        List<PracticeConfig> list =  iPracticeConfigService.getListByTime(startDate,endDate);
        if (null == list || list.size() == 0){
            return Collections.emptyList();
        }
        List<PracticeVo> voList = new ArrayList<>();
        for (PracticeConfig practiceConfig : list) {
            PracticeVo  vo = new PracticeVo();
            BeanUtils.copyProperties(practiceConfig,vo);
            voList.add(vo);
        }
        return voList;
    }

    @Override
    public List<PracticeVisibleVo> selectVisibleRangeList(Date startDate, Date endDate) {
        List<PracticeVisible> list =  iPracticeVisibleService.getListByTime(startDate,endDate);
        if (null == list || list.size() == 0){
            return Collections.emptyList();
        }
        List<PracticeVisibleVo> voList = new ArrayList<>();
        for (PracticeVisible visible : list) {
            PracticeVisibleVo  vo = new PracticeVisibleVo();
            BeanUtils.copyProperties(visible,vo);
            voList.add(vo);
        }
        return voList;
    }

    /**
     * 校验陪练场景下对话是否符合规范
     * @param practiceId 陪练id
     * @param sceneId 场景id
     */
    private void checkSceneChatValid(Long practiceId, Long sceneId) {

        RequestContext context = ContextHolder.get();
        //场景对话至少包含 ：开始-机器人-学员 三个节点
        List<SceneChatConfig> startNode = chatConfigService.selectChatNodes(practiceId, sceneId, PracticeNodeEnum.START.getStatus(), 0L, 0);
        if (null == startNode || startNode.isEmpty()){
            logger.error("场景对话校验不通过.未查询到开始节点; practiceId = {} ,sceneId = {},accountId = {}",practiceId,sceneId,context.getAccountId());
            throw new BizException(PracticeInternationEnum.SCENE_CHAT_INCOMPLETE.getCode(),PracticeInternationEnum.SCENE_CHAT_INCOMPLETE.getName());
        }
        List<SceneChatConfig> robotNode = chatConfigService.selectChatNodes(practiceId, sceneId, PracticeNodeEnum.ROBOT.getStatus(), null, 1);
        if (null == robotNode || robotNode.isEmpty()){
            logger.error("场景对话校验不通过.未查询到开始节点下的机器人节点; practiceId = {} ,sceneId = {},accountId = {}",practiceId,sceneId,context.getAccountId());
            throw new BizException(PracticeInternationEnum.SCENE_CHAT_INCOMPLETE.getCode(),PracticeInternationEnum.SCENE_CHAT_INCOMPLETE.getName());
        }
        List<SceneChatConfig> stuNode = chatConfigService.selectChatNodes(practiceId, sceneId, PracticeNodeEnum.STUDENT.getStatus(), null, 2);
        if (null == stuNode || stuNode.isEmpty()){
            logger.error("场景对话校验不通过.未查询到开始节点下的学员节点; practiceId = {} ,sceneId = {},accountId = {}",practiceId,sceneId,context.getAccountId());
            throw new BizException(PracticeInternationEnum.SCENE_CHAT_INCOMPLETE.getCode(),PracticeInternationEnum.SCENE_CHAT_INCOMPLETE.getName());
        }
        List<SceneChatConfig> nodeList = chatConfigService.selectChatNodes(practiceId, sceneId, null, null, null);
        for (SceneChatConfig sceneChat : nodeList) {
            Long sceneChatId = sceneChat.getId();
            Integer sceneNodeType = sceneChat.getSceneNodeType();
            //校验节点必填信息是否完整
            if (PracticeNodeEnum.ROBOT.getStatus().equals(sceneNodeType)){
                String robotContent = sceneChat.getRobotContent();
                if (StringUtils.isEmpty(robotContent) || robotContent.length() > 250){
                    logger.error("场景对话校验不通过.机器人节点话术为空或长度超过限制; practiceId = {} ,sceneId = {},sceneChatId = {},accountId = {},robotContent = {}"
                            ,practiceId,sceneId,sceneChatId,context.getAccountId(),robotContent);
                    throw new BizException(PracticeInternationEnum.SCENE_CHAT_INCOMPLETE.getCode(),PracticeInternationEnum.SCENE_CHAT_INCOMPLETE.getName());
                }
            }else if (PracticeNodeEnum.STUDENT.getStatus().equals(sceneNodeType)){
                String accountContent = sceneChat.getAccountContent();
                String keyword = sceneChat.getAccountContentKeyword();
                if (StringUtils.isEmpty(accountContent) || accountContent.length() > 250 || StringUtils.isEmpty(keyword)){
                    logger.error("场景对话校验不通过.学员节点标准话术为空或长度超过限制或关键字为空; practiceId = {} ,sceneId = {},sceneChatId = {},accountId = {},accountContent = {}"
                            ,practiceId,sceneId,sceneChatId,context.getAccountId(),accountContent);
                    throw new BizException(PracticeInternationEnum.SCENE_CHAT_INCOMPLETE.getCode(),PracticeInternationEnum.SCENE_CHAT_INCOMPLETE.getName());
                }
            }
        }

    }

    /**
     * 校验语音合成
     */
    private void checkVoice(List<SceneChatConfig> nodeList){

        if (null == nodeList || nodeList.isEmpty()){
            return;
        }
        RequestContext context = ContextHolder.get();
        for (SceneChatConfig sceneChat : nodeList) {
            Integer sceneNodeType = sceneChat.getSceneNodeType();
            Long practiceId = sceneChat.getPracticeId();
            Long sceneId = sceneChat.getSceneId();
            //校验节点必填信息是否完整
            if (PracticeNodeEnum.ROBOT.getStatus().equals(sceneNodeType)){
                //上架操作校验语音合成
                String voiceUrl = sceneChat.getRobotContentVoiceUrl();
                Long duration = sceneChat.getRobotContentVoiceDuration();
                if (StringUtils.isEmpty(voiceUrl) || null == duration){
                    logger.error("场景对话校验不通过.机器人节点话术合成语音/时长为空; practiceId = {} ,sceneId = {},accountId = {}",practiceId,sceneId,context.getAccountId());
                    throw new BizException(PracticeInternationEnum.SCENE_CHAT_INCOMPLETE.getCode(),PracticeInternationEnum.SCENE_CHAT_INCOMPLETE.getName());
                }
            }else if (PracticeNodeEnum.STUDENT.getStatus().equals(sceneNodeType)){
                Long duration = sceneChat.getAccountContentVoiceDuration();
                String voiceUrl = sceneChat.getAccountContentVoiceUrl();
                if (StringUtils.isEmpty(voiceUrl) || null == duration){
                    logger.error("场景对话校验不通过.学员节点话术合成语音/时长为空; practiceId = {} ,sceneId = {},accountId = {}",practiceId,sceneId,context.getAccountId());
                    throw new BizException(PracticeInternationEnum.SCENE_CHAT_INCOMPLETE.getCode(),PracticeInternationEnum.SCENE_CHAT_INCOMPLETE.getName());
                }
            }
        }
    }

    private void copyChat(Long practiceId, Long targetId
			, SceneConfig oldScene, Long newSceneId
			, String[] ignoreProperties, RequestContext requestContext) throws IllegalAccessException, IllegalArgumentException, InvocationTargetException, SecurityException {
		List<SceneChatConfig> newChats = new ArrayList<SceneChatConfig>();
		List<SceneChatConfig> oldChats = chatConfigRepository.findByPracticeIdAndSceneId(practiceId, oldScene.getId());
		if(null == oldChats) {
			logger.warn("SceneChatConfig with practiceId {} not exist!", practiceId);
		}else {
			Map<Long, Long> oldNewId = new HashMap<Long, Long>();
			for(SceneChatConfig oldChat : oldChats) {
				SceneChatConfig newChat = new SceneChatConfig();
				BeanUtils.copyProperties(oldChat, newChat, ignoreProperties);
				
				Long newChatId = idGenerator.generate();
				newChat.setId(newChatId);
				newChat.setPracticeId(targetId);
				newChat.setSceneId(newSceneId);
				setCommonProperties(newChat, requestContext);
				if(oldNewId.containsKey(oldChat.getPid())) {
					newChat.setPid(oldNewId.get(oldChat.getPid()));
				}
				oldNewId.put(oldChat.getId(), newChatId);
				newChats.add(newChat);
			}
			if(null != newChats) {
				chatConfigRepository.saveAll(newChats);
			}
		}
	}
	
	private void copyScene(Long practiceId, Long targetId, String[] ignoreProperties, RequestContext requestContext) throws IllegalAccessException, IllegalArgumentException, InvocationTargetException, SecurityException {
		List<SceneConfig> oldScenes = sceneConfigRepository.selectByPracticeId(practiceId);
		if(null == oldScenes) {
			logger.warn("SceneConfig with practiceId {} not exist!", practiceId);
		}else {
			List<SceneConfig> newScenes = new ArrayList<SceneConfig>();
			for(SceneConfig oldScene : oldScenes) {
				SceneConfig newScene = new SceneConfig();
				BeanUtils.copyProperties(oldScene, newScene, ignoreProperties);
				Long newSceneId = idGenerator.generate();
				newScene.setId(newSceneId);
				newScene.setPracticeId(targetId);
				setCommonProperties(newScene, requestContext);
				newScenes.add(newScene);
				
				// 复制scene_chat_config 
				copyChat(practiceId, targetId, oldScene, newSceneId, ignoreProperties, requestContext);
			}
			
			if(null != newScenes) {
				sceneConfigRepository.saveAll(newScenes);
			}
			
		}
	}
	
	private void copyVisible(Long practiceId, Long targetId, String[] ignoreProperties, RequestContext requestContext) throws IllegalAccessException, IllegalArgumentException, InvocationTargetException, SecurityException {
		List<PracticeVisible> oldVisibles = visibleRepository.findByPracticeId(practiceId);
		if(null == oldVisibles) {
			logger.warn("PracticeVisible with practiceId {} not exist!", practiceId);
		}else {
			List<PracticeVisible> newVisibles = new ArrayList<PracticeVisible>();
			for(PracticeVisible oldVisible : oldVisibles) {
				PracticeVisible newVisible = new PracticeVisible();
				BeanUtils.copyProperties(oldVisible, newVisible, ignoreProperties);
				newVisible.setId(idGenerator.generate());
				newVisible.setPracticeId(targetId);
				setCommonProperties(newVisible, requestContext);
				newVisibles.add(newVisible);
			}
			visibleRepository.saveAll(newVisibles);
		}
	}
	
	private void copyRule(Long practiceId, Long targetId, String[] ignoreProperties, RequestContext requestContext) throws IllegalAccessException, IllegalArgumentException, InvocationTargetException, SecurityException {
		List<PracticePointRuleConfig> originRules = pointRuleConfigRepository.selectListByPracticeId(practiceId);
		if(null == originRules) {
			logger.warn("PracticePointRuleConfig with practiceId {} not exist!", practiceId);
		}else {
			List<PracticePointRuleConfig> newRules = new ArrayList<PracticePointRuleConfig>();
			for(PracticePointRuleConfig oldRule : originRules) {
				PracticePointRuleConfig newRule = new PracticePointRuleConfig();
				BeanUtils.copyProperties(oldRule, newRule, ignoreProperties);
				newRule.setId(idGenerator.generate());
				newRule.setPracticeId(targetId);
				setCommonProperties(newRule, requestContext);
				newRules.add(newRule);
			}
			pointRuleConfigRepository.saveAll(newRules);
		}
	}
	
	private void copyPracticeConfig(Long practiceId, Long targetId, String[] ignoreProperties) {
		PracticeConfig origin = practiceConfigRepository.getOne(practiceId);
		if(null == origin) {
			logger.warn("Practice with ID {} not exist!", practiceId);
			return ;
		}
			
		PracticeConfig target = new PracticeConfig();
		
		// 复制practice_config
		BeanUtils.copyProperties(origin, target, ignoreProperties);
		target.setId(targetId);
		target.setName(origin.getName()+"[复制]");
		target.setStatus(PracticeStatusEnum.DRAFT.getStatus());
		setCreatedAccountInfo(target);
		practiceConfigRepository.save(target);
	}
	
	private void setCommonProperties(Object target, RequestContext context) throws IllegalAccessException, IllegalArgumentException, InvocationTargetException, SecurityException {
		if(target instanceof PracticePointRuleConfig
				|| target instanceof PracticeVisible
				|| target instanceof SceneConfig
				|| target instanceof SceneChatConfig) {
			try {
				target.getClass().getMethod("setCompanyId", Long.class).invoke(target, context.getCompanyId());
			}catch(NoSuchMethodException e) {
				
			}
			try {
				target.getClass().getMethod("setSiteId", Long.class).invoke(target, context.getSiteId());
			}catch(NoSuchMethodException e) {
				
			}
			try {
				target.getClass().getMethod("setOrgId", Long.class).invoke(target, context.getOrgId());
			}catch(NoSuchMethodException e) {
				
			}
			try {
				target.getClass().getMethod("setCreatedUid", Long.class).invoke(target, context.getAccountId());
			}catch(NoSuchMethodException e) {
				
			}
			try {
				target.getClass().getMethod("setCreatedAt", Date.class).invoke(target, new Date());
			}catch(NoSuchMethodException e) {
				
			}
			
		}
	}
}
